/*
 * jET - An alternative scenery editor for the electic locomotive simulator, MaSzyna
 * Copyright (C) 2003 2004  Adam Domaski
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.jet;

import java.io.File;
import java.util.Vector;
import java.util.Iterator;
import java.awt.*;
import java.awt.event.*;
import java.awt.geom.Point2D;
import java.beans.PropertyVetoException;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;
import javax.swing.*;
import javax.swing.event.InternalFrameListener;
import javax.swing.event.InternalFrameAdapter;
import javax.swing.event.InternalFrameEvent;
import javax.swing.border.BevelBorder;
import javax.swing.filechooser.FileFilter;
import javax.swing.text.NumberFormatter;
import javax.swing.text.DefaultFormatterFactory;
import javax.vecmath.Point3d;

import prv.doman.util.ScaleFormat;
import org.jet.sce.*;
import org.jet.tracks.*;

/**
 * <b>jET</b> application.
 * Entry point. Creates frame, menu, reads configuration file.
 *
 * @author Adam Domaski
 * @version 0.1 21/02/2004
 */
public class Jet
{
	JFrame           frame;
	JDesktopPane     desktop;
	JMenuBar         menu;
	
	/** Current configuration. */
	public Configuration config = new Configuration();

	/** Scenery file filter <code>*.sc*</code>. */
	private SceneryFileFilter fileFilter = new SceneryFileFilter();

	/** Zoom combo box. */
	private JComboBox zoomCombo;

	/** Scenery palette. */
	private PaletteFrame sceneryPalette;

	/** Resource. */
	private JetResources resource;

	/** Zoom mode button. */
	private JToggleButton jZoomButton;

	/** Normal mode button. */
	private JToggleButton jNormalButton;

	// ---------------- Constructor ----------------

	/**
	 * Creates frame and initializes everything.
	 */
	public Jet()
	{
		resource = JetResources.getInstance();

		frame = new JFrame( "jET - " + resource.translate("Jet.frameTitle"));

		installActions();
		installListeners();

		// Load configuration
		config.load();
		SimulatorCatalog.initialize(config);

		// Add a menu
		frame.setJMenuBar( createMenu() );

		// Add a standard toolbar
		JPanel pane = new JPanel();

		pane.setLayout(new BorderLayout());
		pane.add( createStandardBar(), BorderLayout.NORTH );

		// Add a desktop pane where children frames are put
		pane.add( createDesktopPane() );
		
		// Add a status bar at bottom
		pane.add( createStatusBar(), BorderLayout.SOUTH );

		frame.getContentPane().add( pane );

		actEditorGrid.setGrid( config.getBoolean("EditorGrid") );
		actEditorRulers.setRulers( config.getBoolean("EditorRulers") );
		actEditorTrackNames.setTrackNames( config.getBoolean("EditorTrackNames") );

		// Add window state listener to ...
		frame.addComponentListener(maximizeHandler); 

		// Add window listener
		frame.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
		frame.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent we)
			{
				closeApplication();
			}
		});

		int x = config.getInt("WindowPosX");
		int y = config.getInt("WindowPosY");
		frame.setLocation(x, y);

		x = config.getInt("WindowWidth");
		y = config.getInt("WindowHeight");
		frame.setSize(x, y);

		frame.setVisible(true);
	}

	/**
	 * Entry point to the application.
	 *
	 * @param args Command line.
	 */
	public static void main(String args[])
	{
		UIDefaults uid = UIManager.getLookAndFeelDefaults();
		uid.addResourceBundle("i18n/swing");

		new Jet();
	}

	/**
	 * Create main menu.
	 *
	 * @return Menu.
	 */
	protected JMenuBar createMenu()
	{
		JMenuItem mi;
		JMenu menu;

		JMenuBar mBar = new JMenuBar();
		
		menu = new JMenu( resource.translate("MainMenu.file") );
		mi = new JMenuItem();
		mi.setAction(actNewScenery);
		mi.setIcon(null);
		menu.add(mi);
		mi = new JMenuItem();
		mi.setAction(actOpenScenery);
		mi.setIcon(null);
		menu.add(mi);
		mi = new JMenuItem();
		mi.setAction(actSaveScenery);
		mi.setIcon(null);
		menu.add(mi);
		mi = new JMenuItem();
		mi.setAction(actSaveAsScenery);
		menu.add(mi);
		mi = new JMenuItem();
		mi.setAction(actCloseScenery);
		menu.add(mi);
		menu.addSeparator();		
		mi = new JMenuItem( resource.translate("MainMenu.exit") );
		mi.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae)
			{
				closeApplication();
				frame.setVisible(false);
				System.exit(0);
			}
		});
		menu.add(mi);
		mBar.add(menu);

		menu = new JMenu( resource.translate("MainMenu.edit") );
		mi = new JMenuItem();
		mi.setAction( actDelete );
		menu.add(mi);
		menu.addSeparator();
		mi = new JMenuItem();
		mi.setAction( actBasePoint );
		menu.add(mi);
		mi = new JMenuItem();
		mi.setAction( actSelectionProperties );
		mi.setIcon(null);
		menu.add(mi);		
		menu.setVisible(false);
		mBar.add(menu);

		menu = new JMenu( resource.translate("MainMenu.view") );
		mi = new JCheckBoxMenuItem();
		mi.setAction( actEditorGrid );
		actEditorGrid.setMenuItem((JCheckBoxMenuItem) mi);
		menu.add(mi);
		mi = new JCheckBoxMenuItem();
		mi.setAction( actEditorRulers );
		actEditorRulers.setMenuItem((JCheckBoxMenuItem) mi);
		menu.add(mi);
		mi = new JCheckBoxMenuItem();
		mi.setAction( actEditorTrackNames );
		actEditorTrackNames.setMenuItem((JCheckBoxMenuItem) mi);
		menu.add(mi);
		menu.addSeparator();
		mi = new JMenuItem();
		mi.setAction( actGoTo );
		menu.add(mi);
		mBar.add(menu);

		menu = new JMenu( resource.translate("MainMenu.scenery") );
		mi = new JMenuItem();
		mi.setAction( actSceneryDescription );
		menu.add(mi);
		mi = new JMenuItem();
		mi.setAction( actTrackNames );
		menu.add(mi);
		menu.setVisible(false);
		mBar.add(menu);
		
		menu = new JMenu( resource.translate("MainMenu.tools") );
		mi = new JMenuItem();
		mi.setAction( actOptions );
		menu.add(mi);
		mBar.add(menu);

		menu = new JMenu( resource.translate("MainMenu.window") );
		mi = new JMenuItem();
		mi.setAction( actCascadeFrames );
		menu.add(mi);
		mi = new JMenuItem();
		mi.setAction( actHorizontalFrames );
		menu.add(mi);
		mi = new JMenuItem();
		mi.setAction( actVerticalFrames );
		menu.add(mi);
		menu.addSeparator();
		mi = new JMenuItem();
		mi.setAction( actMinimalizeFrames );
		menu.add(mi);
		mi = new JMenuItem();
		mi.setAction( actMaximizeFrames );
		menu.add(mi);
		mi = new JMenuItem();
		mi.setAction( actRestoreFrames );
		menu.add(mi);
		menu.setVisible(false);
		mBar.add(menu);

		menu = new JMenu( resource.translate("MainMenu.help") );
		mi = new JMenuItem( resource.translate("MainMenu.about") );
		mi.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae)
			{
				new AboutBox(frame);
			}
		});
		menu.add(mi);
		mBar.add(menu);

		this.menu = mBar;

		return mBar;
	}

	/**
	 * Creates standard toolbar.
	 *
	 * @return Standard toolbar.
	 */
	protected JToolBar createStandardBar()
	{
		ImageIcon img;
		JButton button;

		JToolBar bar = new JToolBar();
		bar.setFloatable(false);

		button = new JButton();
		button.setAction(actNewScenery);
		button.setText(null);		
		bar.add(button);

		button = new JButton();
		button.setAction(actOpenScenery);
		button.setText(null);
		bar.add(button);

		button = new JButton();
		button.setAction(actSaveScenery);
		button.setText(null);
		bar.add(button);

		bar.addSeparator();

		ButtonGroup bg = new ButtonGroup();

		jNormalButton = new JToggleButton();
		jNormalButton.setAction(actNormalMode);
		jNormalButton.setText(null);
		jNormalButton.setSelected(true);
		bar.add(jNormalButton);
		bg.add(jNormalButton);

		jZoomButton = new JToggleButton();
		jZoomButton.setAction(actZoomMode);
		jZoomButton.setText(null);
		bar.add(jZoomButton);
		bg.add(jZoomButton);

		bar.addSeparator();

		button = new JButton();
		button.setAction(actSelectionProperties);
		button.setText(null);
		bar.add(button);

		bar.addSeparator();

		JLabel lab = new JLabel(new ImageIcon("images/sb_zoom.png"));
		zoomCombo = createZoomCombo();		
		lab.setLabelFor(zoomCombo);
		bar.add(lab);
		bar.add(zoomCombo);

		return bar;
	}

	/**
	 * Creates zoom combo box.
	 *
	 * @return Combo box.
	 */
	protected JComboBox createZoomCombo()
	{
		Object[] scales = {new Double(1.0), new Double(0.5), new Double(0.2), 
			new Double(0.1), new Double(0.05), new Double(0.02), new Double(0.01),
			new Double(0.005), new Double(0.002), new Double(0.001),
			new Double(5.0E-4), new Double(2.0E-4), new Double(1.0E-4)};

		JComboBox cb = new JComboBox(scales);
		cb.setToolTipText(resource.translate("Action.zoomComboDesc"));
		cb.setEditable(true);
		cb.setAction(actSceneryZoom);

		ZoomComboBoxEditor editor = new ZoomComboBoxEditor();
		JFormattedTextField txt = editor.getTextField();
		NumberFormatter nf = new NumberFormatter();
		nf.setFormat(new ScaleFormat());
		nf.setMinimum(new Double(Double.MIN_VALUE));
		nf.setMaximum(new Double(1.0));
		txt.setFormatterFactory(new DefaultFormatterFactory(nf));

		cb.setEditor(editor);
		cb.setRenderer(new ZoomComboBoxRenderer());

		return cb;
	}

	/**
	 * Creates application workspace for internal frames.
	 *
	 * @return Application workspace (desktop pane).
	 */
	protected JDesktopPane createDesktopPane()
	{
		desktop = new JDesktopPane();
		desktop.setDragMode(JDesktopPane.OUTLINE_DRAG_MODE);
		desktop.setDesktopManager( new DefaultDesktopManager() {
			public void activateFrame(JInternalFrame i)
			{
				try
				{
					if (i instanceof PaletteFrame)
						i.setSelected(false);
					else
						super.activateFrame(i);
				}
				catch (PropertyVetoException pve) {}
			}
		});

		createToolbarFrame();
		return desktop;
	}

	/**
	 * Creates status bar at bottom of application's window.
	 *
	 * @return Status bar.
	 */
	protected JLabel createStatusBar()
	{
		JLabel status = new JLabel( resource.translate("StatusBar.ready") );
		status.setBorder(BorderFactory.createBevelBorder(BevelBorder.LOWERED));

		return status;
	}

	// ----------- Getting main frame --------------

	/**
	 * Gets a frame of application.
	 *
	 * @return Frame of application.
	 */
	public JFrame getFrame()
	{
		return frame;
	}

	// -------------------- Submenu showing / hiding -----------

	/**
	 * Shows some submenus.
	 */
	private void showMenu()
	{
		JMenu m;

		m = menu.getMenu(5);
		m.setVisible(true);
		m = menu.getMenu(3);
		m.setVisible(true);
		m = menu.getMenu(1);
		m.setVisible(true);
	}

	/**
	 * Hides some submenus.
	 */
	private void hideMenu()
	{
		JMenu m;

		m = menu.getMenu(5);
		m.setVisible(false);
		m = menu.getMenu(3);
		m.setVisible(false);
		m = menu.getMenu(1);
		m.setVisible(false);
	}

	// ---------------------- Scenery frames ----------------------------

	private Vector sceneries = new Vector();

	/**
	 * Adds scenery frame to the application workspace (desktop pane).
	 *
	 * @param sf Scenery frame to add.
	 */
	public void addSceneryFrame(SceneryFrame sf)
	{
		if (sceneries.isEmpty())
		{
			actSaveScenery.setEnabled(true);
			actSaveAsScenery.setEnabled(true);
			actCloseScenery.setEnabled(true);
			actEditorGrid.setEnabled(true);
			actEditorRulers.setEnabled(true);
			actEditorTrackNames.setEnabled(true);
			actSceneryZoom.setEnabled(true);
			actGoTo.setEnabled(true);
			actSelectionProperties.setEnabled(!sf.getSceneryEditor().isSelectionEmpty());

			// Dodaje menu
			showMenu();
		}

		// Dodaje nowe okno scenerii do listy okien.
		sceneries.add(sf);
	}

	/**
	 * Removes scenery frame from application workspace (desktop pane).
	 *
	 * @param sf Removing scenery frame.
	 */
	public void removeSceneryFrame(SceneryFrame sf)
	{
		sceneries.remove(sf);
		if (sceneries.isEmpty())
		{
			actSaveScenery.setEnabled(false);
			actSaveAsScenery.setEnabled(false);
			actCloseScenery.setEnabled(false);
			actEditorGrid.setEnabled(false);
			actEditorRulers.setEnabled(false);
			actEditorTrackNames.setEnabled(false);
			actSceneryZoom.setEnabled(false);
			actGoTo.setEnabled(false);
			actSelectionProperties.setEnabled(false);

			hideMenu();
		}
	}

	/**
	 * Creates new scenery frame and adds it to the application workspace 
	 * (desktop pane).
	 */
	protected void emptySceneryFrame()
	{
		SceneryFrame frame = new SceneryFrame(this);
		frame.addInternalFrameListener(iflSceneryFrames);
		frame.setGrid(actEditorGrid.getGrid());
		frame.setRulers(actEditorRulers.getRulers());
		frame.setTrackNames(actEditorTrackNames.getTrackNames());

		desktop.add(frame);
		desktop.getDesktopManager().activateFrame(frame);
		desktop.setSelectedFrame(frame);

		addSceneryFrame(frame);

		frame.setVisible(true);
	}

	/**
	 * Creates scenery frame with file and adds it to the application
	 * workspace.
	 *
	 * @param file Open file.
	 */
	protected void openSceneryFrame(File file)
	{
		SceneryFrame frame = new SceneryFrame(this, file);
		frame.addInternalFrameListener(iflSceneryFrames);
		frame.setGrid(actEditorGrid.getGrid());
		frame.setRulers(actEditorRulers.getRulers());
		frame.setTrackNames(actEditorTrackNames.getTrackNames());

		desktop.add(frame);
		desktop.getDesktopManager().activateFrame(frame);
		desktop.setSelectedFrame(frame);

		addSceneryFrame(frame);

		frame.setVisible(true);
	}

	/**
	 * Gets scenery collection.
	 */
	public Vector getEditorFrames()
	{
		return sceneries;
	}

	/**
	 * Gets application desktop workspace.
	 */
	public JDesktopPane getDesktopPane()
	{
		return desktop;
	}

	// ---------------- Okno palety narzdziowej --------------

	/**
	 * Creates internal frame within application workspace (desktop pane).
	 * The frame contains scenery toolbar.
	 */
	private void createToolbarFrame()
	{
		sceneryPalette = new PaletteFrame(this);
		Dimension dim = new Dimension();
		dim.width = config.getInt("PaletteWidth");
		dim.height = config.getInt("PaletteHeight");
		Point point = new Point();
		point.x = config.getInt("PalettePosX");
		point.y = config.getInt("PalettePosY");
		sceneryPalette.setLocation(point);
		sceneryPalette.setSize(dim);

		desktop.add(sceneryPalette, JLayeredPane.PALETTE_LAYER);

		sceneryPalette.setVisible(true);
	}

	// -------------- Subaction when application ---
	// ---- is going to close ----------------------

	/**
	 * Invoked when application is going to closes.
	 */
	protected void closeApplication()
	{
		// Zamknij wszystkie podokna
		while (sceneries.size() > 0)
		{
			if ( !((SceneryFrame) sceneries.elementAt(0)).closeFrame() )
				return;
		}

		// Zamknij okno palety
		Rectangle r = sceneryPalette.getBounds();
		config.put("PalettePosX", r.x);
		config.put("PalettePosY", r.y);
		config.put("PaletteWidth", r.width);
		config.put("PaletteHeight", r.height);

		config.put("EditorRulers", actEditorRulers.getRulers());
		config.put("EditorGrid", actEditorGrid.getGrid());
		config.put("EditorTrackNames", actEditorTrackNames.getTrackNames());

		Point p = maximizeHandler.getPosition();
		config.put("WindowPosX", p.x);
		config.put("WindowPosY", p.y);

		Dimension d = maximizeHandler.getSize();
		config.put("WindowWidth", d.width);
		config.put("WindowHeight", d.height);

		config.store();

		frame.dispose();
	}

	// --------------- Inne --------------

	/**
	 * Gets the file filter used in file chooser dialogs.
	 * <p><b>jET</b> uses following pattern: <code>*.sc*</code>.</p>
	 *
	 * @return Application file filter.
	 */
	public FileFilter getFileFilter()
	{
		return fileFilter;
	}

	// ----------------- Akcje ---------------------------

	private Action actNewScenery = null;
	private Action actOpenScenery = null;
	private Action actSaveScenery = null;
	private Action actSaveAsScenery = null;
	private Action actCloseScenery = null;
	private Action actMinimalizeFrames = null;
	private Action actMaximizeFrames = null;
	private Action actCascadeFrames = null;
	private Action actHorizontalFrames = null;
	private Action actVerticalFrames = null;
	private Action actRestoreFrames = null;	
	private EditorGridAction actEditorGrid = null;
	private EditorRulersAction actEditorRulers = null;
	private EditorTrackNamesAction actEditorTrackNames = null;
	private Action actOptions = null;
	private Action actSceneryDescription = null;
	private Action actTrackNames = null;
	private Action actSceneryZoom = null;
	private Action actGoTo = null;
	private SelectionPropertiesAction actSelectionProperties = null;
	private Action actNormalMode = null;
	private Action actZoomMode = null;
	private DeleteAction actDelete = null;
	private Action actBasePoint = null;

	// -------------- Listeners -----------------

	private InternalFrameListener iflSceneryFrames = null;

	/** Backed up size of frame. */
	private MaximizeHandler maximizeHandler = null;

	/**
	 * Registers all actions.
	 */
	protected void installActions()
	{
		actNewScenery = new NewSceneryAction();
		actOpenScenery = new OpenSceneryAction();
		actSaveScenery = new SaveSceneryAction();
		actSaveScenery.setEnabled(false);
		actSaveAsScenery = new SaveAsSceneryAction();
		actSaveAsScenery.setEnabled(false);
		actCloseScenery = new CloseSceneryAction();
		actCloseScenery.setEnabled(false);
		actMinimalizeFrames = new MinimalizeFramesAction();
		actMaximizeFrames = new MaximizeFramesAction();
		actCascadeFrames = new CascadeFramesAction();
		actHorizontalFrames = new HorizontalFramesAction();
		actVerticalFrames = new VerticalFramesAction();
		actRestoreFrames = new RestoreFramesAction();
		actEditorGrid = new EditorGridAction();
		actEditorGrid.setEnabled(false);
		actEditorRulers = new EditorRulersAction();
		actEditorRulers.setEnabled(false);
		actEditorTrackNames = new EditorTrackNamesAction();
		actEditorTrackNames.setEnabled(false);
		actOptions = new OptionsAction();
		actSceneryDescription = new SceneryDescriptionAction();
		actTrackNames = new TrackNamesAction();
		actSceneryZoom = new SceneryZoomAction();
		actSceneryZoom.setEnabled(false);
		actGoTo = new GoToAction();
		actGoTo.setEnabled(false);
		actSelectionProperties = new SelectionPropertiesAction();
		actSelectionProperties.setEnabled(false);
		actNormalMode = new NormalModeAction();
		actZoomMode = new ZoomModeAction();
		actDelete = new DeleteAction();
		actDelete.setEnabled(false);
		actBasePoint = new BasePointAction();
	}

	/**
	 * Installs all listeners.
	 */
	protected void installListeners()
	{
		iflSceneryFrames = new SceneryFrameHandler();
		maximizeHandler = new MaximizeHandler();
	}

	/*
	 * --------------------- Najwysza sceneria -------------------------
	 */
	
	/**
	 * Gets topmost scenery frame within the desktop.
	 *
	 * @return Topmost scenery frame or <code>null</code> if there is no
	 *   scenery frames at now.
	 */
	public SceneryFrame getTopmostFrame()
	{
		JInternalFrame[] frames = desktop.getAllFramesInLayer(
			JDesktopPane.DEFAULT_LAYER.intValue());
		
		int n = frames.length;
		if (n == 0)
			return null;
		
		int i = 0;
		for (; i < n; i++)
		{
			if (desktop.getPosition(frames[i]) == 0)
				break;
		}

		if (i == n)
			return null;

		return (SceneryFrame) frames[i];
	}

	// ---------------------- Definicje akcji ---------------------------

	/**
	 * Abstrakcyjna klasa uatwiajca uycie akcji w aplikacji.
	 * W akcji zdefiniowane s:
	 * <ul>
	 *   <li>tekst dla polecenia menu,
	 *   <li>obrazek dla przycisku z pasku narzdziowego,
	 *   <li>podpowied dla przycisku,
	 *   <li>duszy opis polecenia (<i>docelowo wywietlany w pasku 
	 *     statusu</i>),
	 *   <li>akcelerator klawiaturowy dla polecenia z menu.
	 * </ul>
	 */
	abstract class ApplicationAction extends AbstractAction
	{
		/**
		 * Konstruuje akcj dla aplikacji.
		 *
		 * @param text Tekst polecenia menu.
		 * @param icon Ikona przycisku w pasku narzdziowym.
		 * @param sDesc Krtki opis czynnoci jak akcja wykonuje.
		 * @param lDesc Duszy opis czynnoci jak akcja wykonuje.
		 * @param mnemonic Skrt klawiaturowy uruchamiajcy akcj.
		 */
		public ApplicationAction(String text, ImageIcon icon, 
			String sDesc, String lDesc, Object mnemonic)
		{
			super(text, icon);
			putValue(SHORT_DESCRIPTION, sDesc);
			putValue(LONG_DESCRIPTION, lDesc);
			putValue(ACCELERATOR_KEY, mnemonic);
		}
	}

	/**
	 * Action that creates a new scenery.
	 */
	class NewSceneryAction extends ApplicationAction
	{
		public NewSceneryAction()
		{
			super(
				resource.translate("Action.newFile"),
				new ImageIcon("images/sb_new.png"),
				resource.translate("Action.newFileDesc"),
				"",
				KeyStroke.getKeyStroke(KeyEvent.VK_N, ActionEvent.CTRL_MASK));
		}
		
		public void actionPerformed(ActionEvent ae)
		{
			emptySceneryFrame();
		}
	}

	/**
	 * Action that loads a scenery from file.
	 */
	class OpenSceneryAction extends ApplicationAction
	{
		public OpenSceneryAction()
		{
			super(
				resource.translate("Action.openFile"),
				new ImageIcon("images/sb_open.png"),
				resource.translate("Action.openFileDesc"),
				"",
				KeyStroke.getKeyStroke(KeyEvent.VK_O, ActionEvent.CTRL_MASK));
		}

		public void actionPerformed(ActionEvent ae)
		{
			JFileChooser fc = new JFileChooser();
			fc.addChoosableFileFilter(fileFilter);

			// Przejd do zapamitanego katalogu
			File currentDir = null;
			String dirName = (String) config.get("CurrentDirectory");
			if (dirName != null)
			{
				currentDir = new File(dirName);
			}
			fc.setCurrentDirectory( currentDir );

			int ret = fc.showOpenDialog(frame);

			// Zapamitaj katalog
			currentDir = fc.getCurrentDirectory();
			config.put("CurrentDirectory", currentDir.toString());

			if (ret == JFileChooser.APPROVE_OPTION)
			{
				openSceneryFrame(fc.getSelectedFile());
			}
		}
	}

	/**
	 * Action that saves a scenery to file.
	 */
	class SaveSceneryAction extends ApplicationAction
	{
		public SaveSceneryAction()
		{
			super(
				resource.translate("Action.saveFile"),
				new ImageIcon("images/sb_save.png"),
				resource.translate("Action.saveFileDesc"),
				"",
				KeyStroke.getKeyStroke(KeyEvent.VK_S, ActionEvent.CTRL_MASK));
		}

		public void actionPerformed(ActionEvent ae)
		{
			getTopmostFrame().saveScenery();
		}
	}

	/**
	 * Akcja zapisywania scenerii do pliku o wskazanej nazwie.
	 */
	class SaveAsSceneryAction extends AbstractAction
	{
		public SaveAsSceneryAction()
		{
			super( resource.translate("Action.saveAsFile") );
			putValue(LONG_DESCRIPTION, "");
		}

		public void actionPerformed(ActionEvent ae)
		{
			getTopmostFrame().saveSceneryAs();
		}
	}

	/**
	 * Akcja zamykania okna ze sceneri.
	 * Zamykane jest okno posiadajce w chwili wywoania fokus.
	 */
	class CloseSceneryAction extends AbstractAction
	{
		public CloseSceneryAction()
		{
			super( resource.translate("Action.closeFile") );
			putValue(LONG_DESCRIPTION, "");
		}

		public void actionPerformed(ActionEvent ae)
		{
			getTopmostFrame().closeFrame();
		}
	}

	/**
	 * Minimalizes all frames.
	 */
	class MinimalizeFramesAction extends AbstractAction
	{
		public MinimalizeFramesAction()
		{
			super( resource.translate("Action.minimize") );
			putValue(LONG_DESCRIPTION, "");
		}

		public void actionPerformed(ActionEvent ae)
		{
			JInternalFrame[] frames = desktop.getAllFrames();

			for (int i = 0; i < frames.length; i++)
			{
				if (!(frames[i] instanceof PaletteFrame))
				{
					try
					{
						frames[i].setMaximum(false);
						frames[i].setIcon(true);
					}
					catch (Exception e) {}
				}
			}
		}

	}

	/**
	 * Maksymalizauje wszystkie okna
	 */
	class MaximizeFramesAction extends AbstractAction
	{
		public MaximizeFramesAction()
		{
			super( resource.translate("Action.maximize") );
			putValue(LONG_DESCRIPTION, "");
		}

		public void actionPerformed(ActionEvent ae)
		{
			JInternalFrame[] frames = desktop.getAllFrames();

			for (int i = 0; i < frames.length; i++)
			{
				if (!(frames[i] instanceof PaletteFrame))
				{
					try
					{
						frames[i].setIcon(false);
						frames[i].setMaximum(true);
					}
					catch (Exception e) {}
				}
			}
		}
	}

	/**
	 * Uoenie okien kaskadowo.
	 */
	class CascadeFramesAction extends AbstractAction
	{
		public CascadeFramesAction()
		{
			super( resource.translate("Action.cascade") );
			putValue(LONG_DESCRIPTION, "");
		}

		public void actionPerformed(ActionEvent ae)
		{
			JInternalFrame[] frames = desktop.getAllFrames();
			int n = frames.length;
			DesktopManager dm = desktop.getDesktopManager();

			// Remove scenery toolbar frame.
			int j = 0;
			while (frames[j] instanceof SceneryFrame)
				j++;
			System.arraycopy(frames, j+1, frames, j, n-j-1);
			n--;

			for (int i = 0; i < n; i++)
			{
				dm.setBoundsForFrame(frames[i], i * SceneryFrame.xOffset,
					i * SceneryFrame.yOffset, SceneryFrame.WIDTH, SceneryFrame.HEIGHT);
			}
		}
	}

	/**
	 * Uoenie poziome.
	 */
	class HorizontalFramesAction extends AbstractAction
	{
		public HorizontalFramesAction()
		{
			super( resource.translate("Action.horizontal") );
			putValue(LONG_DESCRIPTION, "");
		}

		public void actionPerformed(ActionEvent ae)
		{
			JInternalFrame[] frames = desktop.getAllFrames();
			int n = frames.length;

			// Remove scenery toolbar frame.
			int j = 0;
			while (frames[j] instanceof SceneryFrame)
				j++;
			System.arraycopy(frames, j+1, frames, j, n-j-1);
			n--;
			
			Rectangle rect = desktop.getBounds();
			DesktopManager dm = desktop.getDesktopManager();

			for (int i = 0; i < n; i++)
			{
				dm.setBoundsForFrame(frames[i], 0,
					i * rect.height / n, rect.width, rect.height / n);
			}
		}
	}

	/**
	 * Uoenie pionowe.
	 */
	class VerticalFramesAction extends AbstractAction
	{
		public VerticalFramesAction()
		{
			super( resource.translate("Action.vertical") );
			putValue(LONG_DESCRIPTION, "");
		}

		public void actionPerformed(ActionEvent ae)
		{
			JInternalFrame[] frames = desktop.getAllFrames();
			DesktopManager dm = desktop.getDesktopManager();
			int n = frames.length;

			// Remove scenery toolbar frame.
			int j = 0;
			while (frames[j] instanceof SceneryFrame)
				j++;
			System.arraycopy(frames, j+1, frames, j, n-j-1);
			n--;

			Rectangle rect = desktop.getBounds();

			for (int i = 0; i < n; i++)
			{
				dm.setBoundsForFrame(frames[i], i * rect.width / n,
					0, rect.width / n, rect.height);
			}
		}
	}

	/**
	 * Przywrcenie okien.
	 */
	class RestoreFramesAction extends AbstractAction
	{
		public RestoreFramesAction()
		{
			super( resource.translate("Action.restoreWindows") );
			putValue(LONG_DESCRIPTION, "");
		}

		public void actionPerformed(ActionEvent ae)
		{
			JInternalFrame[] frames = desktop.getAllFrames();

			for (int i = 0; i < frames.length; i++)
			{
				if (!(frames[i] instanceof PaletteFrame))
				{
					try
					{
						frames[i].setMaximum(false);
						frames[i].setIcon(false);
					}
					catch (Exception e) {}
				}
			}
		}
	}

	/**
	 * Sets or unsets grid for scenery editors.
	 * In addition class has got property method executed while
	 * loading and saving config file.
	 */
	class EditorGridAction extends AbstractAction
	{
		public EditorGridAction()
		{
			super( resource.translate("Action.grid") );
			putValue(LONG_DESCRIPTION, "");
		}

		/**
		 * This action sets or unsets (depending on menu item state)
		 * editor's grid to all scenery windows.
		 *
		 * @param ae Action event.
		 */
		public void actionPerformed(ActionEvent ae)
		{
			JCheckBoxMenuItem mi = (JCheckBoxMenuItem) ae.getSource();
			grid = mi.isSelected();
			action();
		}

		/**
		 * Retrives state of menu item.
		 * If true, grids are enabled.
		 *
		 * @return Boolean value of grid enable.
		 */
		public boolean getGrid()
		{
			return grid;
		}

		/**
		 * Sets grid enabled to menu item.
		 *
		 * @param b If true, grid will be set.
		 */
		public void setGrid(boolean b)
		{
			grid = b;
			micb.setState(b);
			action();
		}

		/**
		 * Sets menu item component.
		 * Action is able later to set proper state for menu item.
		 */
		public void setMenuItem(JCheckBoxMenuItem micb)
		{
			this.micb = micb;
		}

		/**
		 * This action is executed after state changing or action event occuring.
		 * Action sets or unsets for every scenery editor grid.
		 */
		protected void action()
		{
			Iterator i = sceneries.iterator();
			while (i.hasNext())
			{
				SceneryFrame sf = (SceneryFrame) i.next();
				sf.setGrid( grid );
			}
		}

		private boolean grid;
		private JCheckBoxMenuItem micb;
	}

	/**
	 * Sets or unsets track names in editor.
	 */
	class EditorTrackNamesAction extends AbstractAction
	{
		public EditorTrackNamesAction()
		{
			super( resource.translate("Action.trackNames") );
			putValue(LONG_DESCRIPTION, "");
		}

		/**
		 * Lanched for dispatcher thread.
		 */
		public void actionPerformed(ActionEvent ae)
		{
			JCheckBoxMenuItem mi = (JCheckBoxMenuItem) ae.getSource();
			trackNames = mi.isSelected();
			action();
		}

		/**
		 * Acts will all scenery editors.
		 */
		protected void action()
		{
			Iterator i = sceneries.iterator();
			while (i.hasNext())
			{
				SceneryFrame sf = (SceneryFrame) i.next();
				sf.setTrackNames( trackNames );
			}
		}

		/**
		 * Sets menu item component.
		 * Action is able later to set proper state for menu item.
		 */
		public void setMenuItem(JCheckBoxMenuItem micb)
		{
			this.micb = micb;
		}

		/**
		 * Sets the state for this action.
		 *
		 * @param b If <code>true</code>, track names will be set.
		 */
		public void setTrackNames(boolean b)
		{
			trackNames = b;
			micb.setState(b);
			action();
		}

		/**
		 * Gets the state of track names visibility for this action.
		 *
		 * @return <code>True</code>, if track names are visible.
		 */
		public boolean getTrackNames()
		{
			return trackNames;
		}
	
		private boolean trackNames;
		private JCheckBoxMenuItem micb;
	}

	/**
	 * Action is executed when user selects or unselects "Rulers" menu item.
	 * Action causes all scenery editors to gain or lose rulers. 
	 */
	class EditorRulersAction extends AbstractAction
	{
		public EditorRulersAction()
		{
			super( resource.translate("Action.rulers") );
			putValue(LONG_DESCRIPTION, "");
		}

		/**
		 * This action sets or unsets (depending on menu item state)
		 * editor's rulers to all scenery windows.
		 *
		 * @param ae Action event.
		 */
		public void actionPerformed(ActionEvent ae)
		{
			JCheckBoxMenuItem mi = (JCheckBoxMenuItem) ae.getSource();
			rulers = mi.isSelected();
			action();
		}

		/**
		 * Retrives state of menu item.
		 * If true, rulers are enabled.
		 *
		 * @return Boolean value of rulers enable.
		 */
		public boolean getRulers()
		{
			return rulers;
		}

		/**
		 * Sets rulers enabled to menu item.
		 *
		 * @param b If true, grid will be set.
		 */
		public void setRulers(boolean b)
		{
			rulers = b;
			micb.setState(b);
			action();
		}

		/**
		 * Sets menu item component.
		 * Action is able later to set proper state for menu item.
		 */
		public void setMenuItem(JCheckBoxMenuItem micb)
		{
			this.micb = micb;
		}

		/**
		 * This action is executed after state changing or action event occuring.
		 * Action sets or unsets for every scenery editor rulers.
		 */
		protected void action()
		{
			Iterator i = sceneries.iterator();
			while (i.hasNext())
			{
				SceneryFrame sf = (SceneryFrame) i.next();
				sf.setRulers( rulers );
			}
		}

		private boolean rulers;
		private JCheckBoxMenuItem micb;
	}

	/**
	 * Option dialog box execution.
	 */
	class OptionsAction extends AbstractAction
	{
		public OptionsAction()
		{
			super( resource.translate("Action.preferences") );
			putValue(LONG_DESCRIPTION, "");
		}

		public void actionPerformed(ActionEvent ae)
		{
			new JetOptions(frame, config);
		}
	}

	/**
	 * Scenery description dialog action.
	 */
	class SceneryDescriptionAction extends AbstractAction
	{
		public SceneryDescriptionAction()
		{
			super( resource.translate("Action.description") );
			putValue(LONG_DESCRIPTION, "");
		}

		public void actionPerformed(ActionEvent ae)
		{
			new SceneryDescriptionDialog(frame, getTopmostFrame());
		}
	}

	/**
	 * Scenery track names dialog action.
	 */
	class TrackNamesAction extends AbstractAction
	{
		public TrackNamesAction()
		{
			super( resource.translate("Action.trackNamesDialog") );
			putValue(LONG_DESCRIPTION, "");
		}

		public void actionPerformed(ActionEvent ae)
		{
			JSceneryEditor editor = getTopmostFrame().getSceneryEditor();
			new TrackNamesDialog(frame, editor);
		}
	}

	/**
	 * Scenery zoom action
	 */
	class SceneryZoomAction extends AbstractAction
	{
		public SceneryZoomAction()
		{
			super( resource.translate("Action.zoom") );
			putValue(LONG_DESCRIPTION, "");
		}

		public void actionPerformed(ActionEvent ae)
		{
			JComboBox cb = (JComboBox) ae.getSource();
			double scale = ((Double) cb.getSelectedItem()).doubleValue();

			JSceneryEditor editor = getTopmostFrame().getSceneryEditor();
			Rectangle rect = editor.getVisibleRect();
			Point p = new Point(rect.x, rect.y);
			p.x += rect.width / 2;
			p.y += rect.height / 2;
			Point3d p3d = editor.viewToModel(p);

			editor.setScale( scale );

			editor.centerViewAt( editor.modelToView(p3d) );

			editor.requestFocusInWindow();
		}
	}

	/**
	 * <i>Go to point</i> action.
	 */
	class GoToAction extends AbstractAction
	{
		public GoToAction()
		{
			super( resource.translate("Action.goTo") );
			putValue(LONG_DESCRIPTION, "");
			putValue(ACCELERATOR_KEY, 
				KeyStroke.getKeyStroke(KeyEvent.VK_G, ActionEvent.CTRL_MASK));
		}

		public void actionPerformed(ActionEvent ae)
		{
			GoToDialog gotoDialog = new GoToDialog(Jet.this);
			Point3d point3d = gotoDialog.getPoint();
			if (point3d == null)
				return;

			JSceneryEditor editor = getTopmostFrame().getSceneryEditor();
			Point2D p2 = editor.modelToView(point3d);
			editor.centerViewAt(p2);
		}
	}

	/**
	 * Normal mode of scenery editor's cursor action.
	 */
	class NormalModeAction extends AbstractAction
	{
		public NormalModeAction()
		{
			putValue(SHORT_DESCRIPTION, resource.translate("Action.normalMode"));
			putValue(SMALL_ICON, new ImageIcon("images/sb_normal.png"));
		}

		public void actionPerformed(ActionEvent ae)
		{
			SceneryFrame sf = getTopmostFrame();
			if (sf != null)
			{
				JSceneryEditor editor = sf.getSceneryEditor();
				SceneryEditorUI seUI = editor.getUI();
				if (seUI instanceof BasicSceneryEditorUI)
				{
					BasicSceneryEditorUI editorUI = (BasicSceneryEditorUI) seUI;
					editorUI.setMouseMode( BasicSceneryEditorUI.NORMAL_MODE );
				}
			}
		}
	}
	
	/**
	 * Zoom mode of scenery editor action.
	 */
	class ZoomModeAction extends AbstractAction
	{
		public ZoomModeAction()
		{
			putValue(SHORT_DESCRIPTION, resource.translate("Action.zoomMode"));
			putValue(SMALL_ICON, new ImageIcon("images/sb_zoom.png"));
		}

		public void actionPerformed(ActionEvent ae)
		{
			SceneryFrame sf = getTopmostFrame();
			if (sf != null)
			{
				JSceneryEditor editor = sf.getSceneryEditor();
				SceneryEditorUI seUI = editor.getUI();
				if (seUI instanceof BasicSceneryEditorUI)
				{
					BasicSceneryEditorUI editorUI = (BasicSceneryEditorUI) seUI;
					editorUI.setMouseMode( BasicSceneryEditorUI.ZOOM_MODE );
				}
			}
		}
	}

	/**
	 * Selection properties action.
	 */
	class SelectionPropertiesAction extends ApplicationAction
		implements PropertyChangeListener, ScenerySelectionListener
	{
		public SelectionPropertiesAction()
		{
			super(
				resource.translate("Action.selectionProperties"), 
				new ImageIcon("images/sb_prop.png"),
				resource.translate("Action.selectionPropertiesDesc"),
				"",
				KeyStroke.getKeyStroke(KeyEvent.VK_F3, 0));
		}

		public void actionPerformed(ActionEvent ae)
		{
			SceneryFrame sf = getTopmostFrame();
			if (sf != null)
			{
				JSceneryEditor editor = sf.getSceneryEditor();
				if (!editor.isSelectionEmpty())
				{
					Object obj = editor.getSelectedObject();
					if (obj instanceof Switch)
					{
						new SwitchProperties(getFrame(), editor);
					}
					else if (obj instanceof Track)
					{
						new TrackProperties(getFrame(), editor);
					}
				}
			}
		}

		public void valueChanged(ScenerySelectionEvent sse)
		{
			setEnabled(sse.getChange() != null);			
		}

		public void propertyChange(PropertyChangeEvent pce)
		{
			if (pce.getPropertyName().equals(
				JSceneryEditor.SCENERY_SELECTION_PROPERTY))
			{
				ScenerySelectionModel oldSsm = 
					(ScenerySelectionModel) pce.getOldValue();
				oldSsm.removeScenerySelectionListener(this);
				ScenerySelectionModel newSsm =
					(ScenerySelectionModel) pce.getNewValue();
				newSsm.addScenerySelectionListener(this);
				setEnabled(!newSsm.isSelectionEmpty());
			}
		}
	}

	/**
	 * Delete action.
	 */
	class DeleteAction extends AbstractAction
		implements PropertyChangeListener, ScenerySelectionListener
	{
		public DeleteAction()
		{
			super( resource.translate("Action.delete") );
			putValue(LONG_DESCRIPTION, "");
			putValue(ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_DELETE, 0));
		}

		public void actionPerformed(ActionEvent ae)
		{
			SceneryFrame sf = getTopmostFrame();
			if (sf != null)
			{
				JSceneryEditor editor = sf.getSceneryEditor();
				if (!editor.isSelectionEmpty())
				{
					Object obj = editor.getSelectedObject();
					editor.clearSelection();
					Scenery scenery = editor.getScenery();

					Object[] aObjects = new Object[1];
					aObjects[0] = obj;

					scenery.remove(aObjects);
				}
			}
		}

		public void valueChanged(ScenerySelectionEvent sse)
		{
			setEnabled(sse.getChange() != null);			
		}

		public void propertyChange(PropertyChangeEvent pce)
		{
			if (pce.getPropertyName().equals(
				JSceneryEditor.SCENERY_SELECTION_PROPERTY))
			{
				ScenerySelectionModel oldSsm = 
					(ScenerySelectionModel) pce.getOldValue();
				oldSsm.removeScenerySelectionListener(this);
				ScenerySelectionModel newSsm =
					(ScenerySelectionModel) pce.getNewValue();
				newSsm.addScenerySelectionListener(this);
				setEnabled(!newSsm.isSelectionEmpty());
			}
		}
	}

	/**
	 * Base point action.
	 */
	class BasePointAction extends AbstractAction
	{
		public BasePointAction()
		{
			super( resource.translate("Action.basePoint") );
			putValue(LONG_DESCRIPTION, "");
			putValue(ACCELERATOR_KEY, 
				KeyStroke.getKeyStroke(KeyEvent.VK_B, ActionEvent.CTRL_MASK));
		}

		public void actionPerformed(ActionEvent ae)
		{
			SceneryFrame sf = getTopmostFrame();
			if (sf != null)
			{
				JSceneryEditor editor = sf.getSceneryEditor();
				new BasePointDialog(getFrame(), editor);
			}
		}
	}

	/**
	 * Event handler with changing window's position, size, etc. in 
	 * order with maximizing capabilities.
	 */
	private class MaximizeHandler extends ComponentAdapter
	{
		/**
		 * Invoked when window resizes.
		 */
		public void componentResized(ComponentEvent ce)
		{
			JFrame f = (JFrame) ce.getComponent();
			if (f.getExtendedState() == JFrame.NORMAL)
				oldSize = f.getSize();
		}

		/**
		 * Invoked when window moves.
		 */
		public void componentMoved(ComponentEvent ce)
		{
			JFrame f = (JFrame) ce.getComponent();
			if (f.getExtendedState() == JFrame.NORMAL)
				oldPosition = f.getLocation();
		}

		/**
		 * Gets a window location when it is in normal state.
		 *
		 * @return Location point.
		 */
		public Point getPosition()
		{
			return oldPosition;
		}

		/**
		 * Gets a window size when it is in normal state.
		 *
		 * @return Size of the window.
		 */
		public Dimension getSize()
		{
			return oldSize;
		}

		private Point oldPosition = new Point(0,0);
		private Dimension oldSize = new Dimension(0,0);
	}

	/**
	 * File chooser filter that filters file with <code>scn</code> extension.
	 */
	class SceneryFileFilter extends FileFilter
	{
		public boolean accept(File f)
		{
			if (f.isDirectory())
				return true;

			String name = f.getName();
			int dot = name.lastIndexOf('.');
			if ((++dot) < name.length())
			{
				String ext = name.substring(dot);
				return ext.startsWith("sc");
			}

			return false;
		}

		public String getDescription()
		{
			return resource.translate("SceneryFileFilter.sceneryFiles") + " (*.sc*)";
		}
	}

	/**
	 * Internal frame listener. When some scenery frame will be activated
	 * sets in the zoom combo box the proper scale.
	 */
	class SceneryFrameHandler extends InternalFrameAdapter
		implements PropertyChangeListener
	{
		public void internalFrameActivated(InternalFrameEvent ife)
		{
			SceneryFrame sf = (SceneryFrame) ife.getInternalFrame();
			zoomCombo.setSelectedItem(new Double(sf.getScale()));

			JSceneryEditor editor = sf.getSceneryEditor();

			editor.addPropertyChangeListener(
				JSceneryEditor.SCALE_PROPERTY, this);

			// Selection action
			editor.addScenerySelectionListener(
				actSelectionProperties);
			editor.addPropertyChangeListener(
				JSceneryEditor.SCENERY_SELECTION_PROPERTY,
				actSelectionProperties);
			actSelectionProperties.setEnabled(!editor.isSelectionEmpty());

			// Delete action
			editor.addScenerySelectionListener(
				actDelete);
			editor.addPropertyChangeListener(
				JSceneryEditor.SCENERY_SELECTION_PROPERTY,
				actDelete);
			actDelete.setEnabled(!editor.isSelectionEmpty());

			SceneryEditorUI seUI = editor.getUI();
			if (seUI instanceof BasicSceneryEditorUI)
			{
				BasicSceneryEditorUI editorUI = (BasicSceneryEditorUI) seUI;

				if (editorUI.getMouseMode() == BasicSceneryEditorUI.NORMAL_MODE)
				{
					jNormalButton.setSelected(true);
				}
				else
				{
					jZoomButton.setSelected(true);
				}
			}
		}

		public void internalFrameDeactivated(InternalFrameEvent ife)
		{
			SceneryFrame sf = (SceneryFrame) ife.getInternalFrame();
			sf.getSceneryEditor().removePropertyChangeListener(
				JSceneryEditor.SCALE_PROPERTY, this);

			// Selection action
			sf.getSceneryEditor().removePropertyChangeListener(
				JSceneryEditor.SCENERY_SELECTION_PROPERTY,
				actSelectionProperties);
			sf.getSceneryEditor().removeScenerySelectionListener(
				actSelectionProperties);
			
			// Delete action
			sf.getSceneryEditor().removePropertyChangeListener(
				JSceneryEditor.SCENERY_SELECTION_PROPERTY,
				actDelete);
			sf.getSceneryEditor().removeScenerySelectionListener(
				actDelete);
		}

		public void internalFrameClosed(InternalFrameEvent ife)
		{
			SceneryFrame sf = getTopmostFrame();
			if (sf != null)
				zoomCombo.setSelectedItem(new Double(sf.getScale()));
		}

		public void propertyChange(PropertyChangeEvent pce)
		{
			if (pce.getPropertyName().equals(JSceneryEditor.SCALE_PROPERTY))
			{
				zoomCombo.setSelectedItem( pce.getNewValue() );
			}
		}
	}
}
