/*
 * jET - An alternative scenery editor for the electic locomotive simulator, MaSzyna
 * Copyright (C) 2003 2004  Adam Domaski
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.jet;

import java.util.HashMap;
import java.awt.*;
import javax.swing.*;
import javax.swing.text.*;
import java.text.DecimalFormat;
import prv.doman.util.DegreeFormat;

/**
 * Abstract class that defines dialog boxes for <b>jET</b> application.
 * It is also factory for some usefull components.
 *
 * @author Adam Domaski
 * @version 0.1 16/02/2004
 */
public abstract class JetDialog extends JDialog
{
	/*
	 * ----------- Protected constructors ------------
	 */
	
	/**
	 * Creates a modal dialog box with title <code>title</code> and
	 * parent window <code>parent</code>.
	 *
	 * @param parent An owner window of this dialog box.
	 * @param title Title of the dialog box.
	 */
	protected JetDialog(Frame parent, String title)
	{
		super(parent, title, true);
	}

	/**
	 * Creates a modal dialog box with 
	 * parent window <code>parent</code>.
	 *
	 * @param parent An owner window of this dialog box.
	 */
	protected JetDialog(Frame parent)
	{
		super(parent, true);
	}

	/**
	 * Created a modal dialog box with title <code>title</code> and
	 * parent dialog <code>parent</code>.
	 *
	 * @param parent An owner window of this dialog box.
	 * @param title Title of the dialog box.
	 */
	protected JetDialog(Dialog parent, String title)
	{
		super(parent, title, true);
	}

	/**
	 * Created a modal dialog box parent dialog <code>parent</code>.
	 *
	 * @param parent An owner window of this dialog box.
	 */
	protected JetDialog(Dialog parent)
	{
		super(parent, true);
	}

	/*
	 * ----------- Location functions -----------
	 */

	/**
	 * Centers this dialog within owner frame.
	 */
	public void centerDialog()
	{
		Window frame = getOwner();

		Dimension pDim = frame.getSize();
		Point pPoint = frame.getLocation();
		Dimension tDim = getSize();

		pPoint.x += (pDim.width - tDim.width) / 2;
		pPoint.y += (pDim.height - tDim.height) / 2;
		pPoint.x = Math.max(pPoint.x, 0);
		pPoint.y = Math.max(pPoint.y, 0);

		setLocation(pPoint);
	}

	/*
	 * ------------- Components ------------
	 */
	
	/**
	 * Creates new formatted text field with long number formatter.
	 * Format <code>0.000</code>.
	 */
	public static JFormattedTextField createLongFormatTextBox()
	{
		DecimalFormat longFormat = new DecimalFormat("0.000");
		NumberFormatter nf = new NumberFormatter(longFormat);
		JFormattedTextField tf = new JFormattedTextField(nf);
		return tf;
	}

	/**
	 * Creates new text field decimal formatted with one
	 * obligatory decimal number.
	 * Format <code>0.0#</code>
	 */
	public static JFormattedTextField createMediumFormatTextBox()
	{
		DecimalFormat medFormat = new DecimalFormat("0.0#");
		NumberFormatter nf = new NumberFormatter(medFormat);
		JFormattedTextField tf = new JFormattedTextField(nf);
		return tf;
	}

	/**
	 * Creates new text field with degree format.
	 */
	public static JFormattedTextField createDegreeFormatTextBox()
	{
		DegreeFormat df = new DegreeFormat();
		JFormattedTextField tf = new JFormattedTextField(df);
		return tf;
	}

	/**
	 * Sets constaints on the formetted text field.
	 */
	protected void setConstraints(JFormattedTextField tft, Comparable min, Comparable max)
	{
		InternationalFormatter formatter = 
			(InternationalFormatter) tft.getFormatter();
		formatter.setMinimum(min);
		formatter.setMaximum(max);
	}

	/**
	 * Normalizes two buttons.
	 */
	public static void normalize(JButton btn1, JButton btn2)
	{
		Dimension okDim = btn1.getPreferredSize();
		Dimension caDim = btn2.getPreferredSize();

		int width = Math.max(okDim.width, caDim.width);
		okDim.width = width;
		caDim.width = width;

		btn1.setPreferredSize(okDim);
		btn2.setPreferredSize(caDim);
	}

	/*
	 * ------------ Inner classes -------------
	 */

	/**
	 * Map with components.
	 * It stores pairs: component name and component.
	 */
	public class ControlMap extends HashMap
	{
		/**
		 * Default constructor.
		 */
		public ControlMap()
		{
			super(40);
		}

		/**
		 * Puts component to the map.
		 */
		public Component putComponent(Component aComponent)
		{
			return (Component) put(aComponent.getName(), aComponent);
		}

		/**
		 * Gets component from the map.
		 */
		public Component getComponent(String name)
		{
			return (Component) get(name);
		}
	}
}
