/*
 * jET - An alternative scenery editor for the electic locomotive simulator, MaSzyna
 * Copyright (C) 2003 2004  Adam Domaski
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.jet;

import java.io.*;
import java.awt.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.util.Date;
import javax.swing.*;
import javax.swing.border.BevelBorder;
import javax.swing.event.*;
import javax.swing.filechooser.FileFilter;

import org.jet.sce.*;

/**
 * Scenery frame is an internal window with scenery editor.
 *
 * @author Adam Domaski
 * @version 0.1 13/03/2004
 */
public class SceneryFrame extends JInternalFrame
{
	/** X offset at desktop pane where this internal frame shows up. */
	public static final int xOffset = 24;

	/** Y offset at desktop pane there this internal frame shows up. */
	public static final int yOffset = 24;

	/** Initial width of this internal frame. */
	public static final int WIDTH = 300;

	/** Initial height of this internal frame. */
	public static final int HEIGHT = 240;

	/*
	 * ---------------- Constructors ---------------------
	 */

	/**
	 * Creates new empty scenery frame.
	 *
	 * @param app Reference to <b>jET</b>.
	 */
	public SceneryFrame(Jet app)
	{
		super();
		parent = app;
		initFrame();

		String title = 
			resource.translate("SceneryFrame.newTitle") +
			(++openFrameCount) + ".scn";
		setTitle(title);

		file = new File(title);
		changes = false;
		newbie = true;
		scEditor.getScenery().addSceneryChangeListener(new ChangeHandler());
	}

	/**
	 * Creates the scenery frame which is going to show
	 * scenery written in file.
	 *
	 * @param app Reference to <b>jET</b> application.
	 * @param file The file to open.
	 */
	public SceneryFrame(Jet app, File file)
	{
		super();
		parent = app;
		initFrame();

		this.file = file;
		changes = false;
		newbie = false;

		// Set title
		setTitle( file.getName() );

		try
		{
			InputStream in = new ProgressMonitorInputStream(
				parent.getFrame(),
				"",
				new FileInputStream(file));

			(new ReaderThread(in)).start();
		}
		catch (IOException ioe)
		{
			ioe.printStackTrace();
		}		
	}

	/**
	 * Prepares internal frame to be shown.
	 */
	private void initFrame()
	{
		setClosable(true);
		setIconifiable(true);
		setMaximizable(true);
		setResizable(true);

		resource = JetResources.getInstance();

		getContentPane().add( createPane() );

		setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
		//setDefaultCloseOperation(DISPOSE_ON_CLOSE);
		addInternalFrameListener(new InternalFrameAdapter() {
			public void internalFrameClosing(InternalFrameEvent ife)
			{
				closeFrame();
			}
		});

		setLocation(200 + openFrameCount*xOffset, openFrameCount*yOffset);
		setSize(WIDTH, HEIGHT);
	}

	/**
	 * Cleans up internal frame after its closing.
	 *
	 * @return <code>True</code>, if frame successfully closes.
	 */
	public boolean closeFrame()
	{
		if (changes)
		{
			int ret = JOptionPane.showConfirmDialog(
				parent.getFrame(),
				resource.translate("SceneryFrame.saveChanges") + " \"" + file.getName() + "\"?",
				"jET",
				JOptionPane.YES_NO_CANCEL_OPTION);
			if (ret == JOptionPane.CANCEL_OPTION)
				return false;
			else if (ret == JOptionPane.YES_OPTION)
			{
				if (!saveScenery())
					return false;
			}
		}

		// Uwolnij zasoby
		parent.removeSceneryFrame(this);
		dispose();

		return true;
	}

	/**
	 * Saves scenery as regular file.
	 *
	 * @return <code>True</code>, when save was really made.
	 */
	public boolean saveScenery()
	{
		if (newbie)
		{
			return saveSceneryAs();
		}
		else
		{
			saveSceneryLowLevel(file);
			return true;
		}
	}

	/**
	 * Saves scenery as other file.
	 *
	 * @return <code>True</code>, when saving was successfull.
	 */
	public boolean saveSceneryAs()
	{
		final JFileChooser fileChooser = new JFileChooser();
		fileChooser.addChoosableFileFilter( parent.getFileFilter() );

		// Przejd do zapamitanego katalogu
		File currentDir = null;
		String dirName = (String) parent.config.get("CurrentDirectory");
		if (dirName != null)
			currentDir = new File(dirName);
		fileChooser.setCurrentDirectory( currentDir );
		fileChooser.setSelectedFile( file );

		// Zapytaj czy nie nada domylnego rozszerzenia
		fileChooser.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae)
			{
				// Check if selected file exists.
				File selectedFile = fileChooser.getSelectedFile();

				// Add extension
				FileFilter fileFilter = fileChooser.getFileFilter();
				if (!fileFilter.accept(selectedFile))
				{
					int ret = JOptionPane.showConfirmDialog(
						fileChooser,
						resource.translate("SceneryFrame.appendExtension") +
							" \"" + selectedFile.getName() + "\"?",
						"jET",
						JOptionPane.YES_NO_OPTION);
					if (ret == JOptionPane.YES_OPTION)
					{
						fileChooser.setSelectedFile( new File(selectedFile.getPath() + ".scn") );
					}
				}
			}
		});

		// Poka okno
		int ret = fileChooser.showSaveDialog(parent.getFrame());
		if (ret == JFileChooser.APPROVE_OPTION)
		{
			File selectedFile = fileChooser.getSelectedFile();
			if (selectedFile.exists())
			{
				int ret2 = JOptionPane.showConfirmDialog(
					fileChooser,
					resource.translate("SceneryFrame.overwrite") + " \"" + 
						selectedFile.getName() + "\"?",
					"jET",
					JOptionPane.YES_NO_OPTION);
				if (ret2 == JOptionPane.NO_OPTION)
					return false;
			}

			// Zapamitaj katalog
			currentDir = fileChooser.getCurrentDirectory();
			parent.config.put("CurrentDirectory", currentDir.toString());

			// Save
			saveSceneryLowLevel( selectedFile );
			this.file = selectedFile;
			newbie = false;
			setTitle( selectedFile.getName() );

			return true;
		}

		return false;
	}

	/**
	 * Creates content pane for scenery frame.
	 *
	 * @return The content pane.
	 */
	private JPanel createPane()
	{
		JPanel pane = new JPanel();
		pane.setLayout(new BorderLayout());

		scEditor = new JSceneryEditor();
		scEditor.requestFocusInWindow();
		//scEditor.setEditable(false);

		scEditor.setBackground( (Color) parent.config.get("EditorBackground") );
		scEditor.setGridColor( (Color) parent.config.get("EditorGridColor") );
		scEditor.setMaxUnitIncrement( parent.config.getInt("EditorScrollIncrement") );
		scEditor.setScale(0.002);
		BasicSceneryEditorUI bseui = (BasicSceneryEditorUI) scEditor.getUI();
		bseui.setTrackColor( (Color) parent.config.get("EditorTrackColor") );
		bseui.setSwitchColor( (Color) parent.config.get("EditorSwitchColor") );

		scroll = new JScrollPane(scEditor);
		scroll.setPreferredSize( new Dimension(200, 200) );
		scroll.setViewportBorder(BorderFactory.createLineBorder(Color.black));
		scroll.setWheelScrollingEnabled(true);

		// Center viewport on scroll pane
		JViewport viewPort = scroll.getViewport();
		Dimension scSize = viewPort.getView().getPreferredSize();
		Dimension viSize = scroll.getPreferredSize();
		Point p = new Point();
		p.x = (scSize.width - viSize.width) / 2;
		p.y = (scSize.height - viSize.height) / 2;
		viewPort.setViewPosition(p);

		pane.add(scroll);

		return pane;
	}

	/**
	 * Sets grid for scenery editor included in this frame.
	 *
	 * @param b If <code>true</code>, the grid will be enabled.
	 */
	public void setGrid(boolean b)
	{
		scEditor.setGridEnabled(b);
	}

	/**
	 * Sets track names visibility to editor.
	 *
	 * @param b If <code>true</code>, the grid will be enabled.
	 */
	public void setTrackNames(boolean b)
	{
		BasicSceneryEditorUI ui = (BasicSceneryEditorUI) scEditor.getUI();
		ui.setTrackNamesVisibility(b);
	}

	/**
	 * Sets rulers for scenery editor included in this frame.
	 *
	 * @param b If <code>true</code>, the rulers will be enabled.
	 */
	public void setRulers(boolean b)
	{
		if (b)
		{
			Dimension dim = scEditor.getPreferredSize();
			columnRule.setPreferredWidth(dim.width);
			rowRule.setPreferredHeight(dim.height);
			scEditor.addMouseMotionListener(columnRule);
			scEditor.addMouseMotionListener(rowRule);
			scroll.setColumnHeaderView(columnRule);
			scroll.setRowHeaderView(rowRule);
			scroll.setCorner(JScrollPane.UPPER_RIGHT_CORNER, new JCorner());
			scroll.setCorner(JScrollPane.LOWER_LEFT_CORNER, new JCorner());
			corner.setBorder(BorderFactory.createBevelBorder(BevelBorder.RAISED));
			scroll.setCorner(JScrollPane.UPPER_LEFT_CORNER, corner);

			scroll.getViewport().addChangeListener( columnRule );
			scroll.getViewport().addChangeListener( rowRule );
		}
		else
		{
			scroll.getViewport().removeChangeListener( columnRule );
			scroll.getViewport().removeChangeListener( rowRule );

			scEditor.removeMouseMotionListener(columnRule);
			scEditor.removeMouseMotionListener(rowRule);
			scroll.setColumnHeaderView(null);
			scroll.setRowHeaderView(null);
			scroll.setCorner(JScrollPane.UPPER_RIGHT_CORNER, new JPanel());
			scroll.setCorner(JScrollPane.LOWER_LEFT_CORNER, new JPanel());
		}
	}

	/**
	 * Sets the scale for scenery editor.
	 */
	public void setScale(double scale)
	{
		scEditor.setScale(scale);
	}

	/**
	 * Gets the scale for scenery editor
	 */
	public double getScale()
	{
		return scEditor.getScale();
	}

	/**
	 * Gets a file of this scenery.
	 *
	 * @return File.
	 */
	public File getFile()
	{
		return file;
	}

	/**
	 * Gets scenery editor.
	 *
	 * @param Current scenery editor.
	 */
	public JSceneryEditor getSceneryEditor()
	{
		return scEditor;
	}

	/**
	 * Saves scenery to the file.
	 *
	 * @param fi Output file. If <code>null</code>, the current file
	 *   of the frame will be used.
	 */
	private void saveSceneryLowLevel(File f)
	{
		try
		{
			OutputStreamWriter out = new OutputStreamWriter(
				new BufferedOutputStream(new FileOutputStream(f)));
			PrintWriter pOut = new PrintWriter(out);

			pOut.println("// Scenery generated by jET");
			pOut.println("// Date: " + new Date());
			pOut.println();
			pOut.flush();

			scEditor.write(out);

			out.close();

			changes = false;
			setTitle(f.getName());
		}
		catch (IOException ioe)
		{
			ioe.printStackTrace();
		}
	}

	/**
	 * Checks if scenery in the editor has been changed.
	 *
	 * @return <code>True</code> if there are some changes.
	 */
	public boolean isChanged()
	{
		return changes;
	}

	/*
	 * ------------- Thread class ----------------
	 */

	/**
	 * This thread reads a scenery file and updates progress bar
	 * at GUI.
	 */
	class ReaderThread extends Thread
	{
		/** Input stream. */
		private InputStreamReader in;

		/**
		 * Creates new thread with input stream reading task.
		 *
		 * @param in Input stream to read.
		 */
		public ReaderThread(InputStream in)
		{
			this.in = new InputStreamReader(in);
		}

		/**
		 * Execution code of this thread.
		 */
		public void run()
		{
			try
			{
				scEditor.read(in);
				in.close();

				Runnable r = new Runnable() {
					public void run()
					{
						scEditor.getScenery().
							addSceneryChangeListener(new ChangeHandler());
					}
				};
				SwingUtilities.invokeLater(r);
			}
			catch (final EditorKit.SceneryException se)
			{
				Runnable r = new Runnable() {
					public void run()
					{
						String message = resource.translate("SceneryFrame.sceneryError")
						  + se.getLine() + ".";
						JOptionPane.showMessageDialog(
							parent.getFrame(),
							message, "jET",
							JOptionPane.ERROR_MESSAGE);
					}
				};
				SwingUtilities.invokeLater(r);
				this.closeFrame();
			}
			catch (InterruptedIOException iioe)
			{
				this.closeFrame();
			}
			catch (final IOException ioe)
			{
				Runnable r = new Runnable() {
					public void run()
					{
						JOptionPane.showMessageDialog(parent.getFrame(),
							ioe.getMessage());
					}
				};
				SwingUtilities.invokeLater(r);
				this.closeFrame();
			}
		}

		/**
		 * Closes scenery frame.
		 */
		private void closeFrame()
		{
			Runnable r = new Runnable() {
				public void run()
				{
					SceneryFrame.this.dispose();
					SceneryFrame.this.closeFrame();
				}
			};
			SwingUtilities.invokeLater(r);
		}
	}

	/*
	 * --------- Inner class ------------
	 */
	
	class ChangeHandler implements SceneryChangeListener
	{
		public void sceneryChanged(SceneryEvent sce)
		{
			if (!changes)
			{
				changes = true;
				String title = getTitle();
				setTitle(title + " *");
			}
		}
	}

	/*
	 * ------------ Members -------------
	 */

	/** Counter of opened frames. */
	private static int openFrameCount = 0;

	/** Reference to the application. */
	private Jet parent;

	/** Scroll pane for editor. */
	private JScrollPane scroll;

	/** The scenery editor. */
	private JSceneryEditor scEditor;

	/** File. */
	private File file;

	/** Horizontal rule. */
	private JRule columnRule = new JRule(JRule.HORIZONTAL);

	/** Vertical rule. */
	private JRule rowRule = new JRule(JRule.VERTICAL);

	/** Upper left corner. */
	private JPanel corner = new JPanel();

	/** Changes flag. */
	private boolean changes;

	/** New, empty frame. */
	private boolean newbie;

	/** Resource. */
	private JetResources resource;
}
