/*
 * jET - An alternative scenery editor for the electic locomotive simulator, MaSzyna
 * Copyright (C) 2003 2004  Adam Domaski
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.jet;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.vecmath.Point3d;

import org.jet.sce.JSceneryEditor;
import org.jet.tracks.*;

/**
 * Dialog box for track and switch names.
 *
 * @author Adam Domaski
 * @version 0.1 16/02/2004
 */
public class TrackNamesDialog extends JetDialog
{
	/**
	 * Shows dialog.
	 *
	 * @param owner Parent window.
	 * @param editor Active scenery editor.
	 */
	public TrackNamesDialog(Frame owner, JSceneryEditor editor)
	{
		super(owner);

		resource = JetResources.getInstance();
		setTitle( resource.translate("TrackNamesDialog.title") );

		setResizable(false);
		setDefaultCloseOperation(HIDE_ON_CLOSE);

		markAction = new MarkAction(editor);

		initialize();
		fillIn(editor);

		pack();
		centerDialog();
		setVisible(true);
	}

	private void initialize()
	{
		JPanel jPane = new JPanel();
		jPane.setName("TrackNamesPanel");
		jPane.setLayout(new BorderLayout());

		JPanel jContainer = createContents();
		jPane.add(jContainer);

		JPanel jControlPanel = new JPanel();
		GridBagLayout gb = new GridBagLayout();
		GridBagConstraints c = new GridBagConstraints();
		jControlPanel.setLayout(gb);
		jControlPanel.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));

		JButton okButton = new JButton( resource.translate("TrackNamesDialog.mark") );
		JButton caButton = new JButton( resource.translate("TrackNamesDialog.close") );

		caButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae)
			{
				dispose();
			}
		});

		okButton.addActionListener(markAction);
		getRootPane().setDefaultButton(okButton);

		c.gridx = 0;
		c.gridy = 0;
		c.weightx = 1.0;
		c.anchor = GridBagConstraints.EAST;
		c.insets = new Insets(0, 0, 0, 5);
		gb.setConstraints(okButton, c);
		jControlPanel.add(okButton);

		c.gridx = 1;
		c.gridy = 0;
		c.weightx = 0.0;
		c.insets = new Insets(0,0,0,0);
		gb.setConstraints(caButton, c);
		jControlPanel.add(caButton);

		normalize(okButton, caButton);
		okButton.requestFocus();

		jPane.add(jControlPanel, BorderLayout.SOUTH);
		
		getContentPane().add(jPane);
	}

	private JPanel createContents()
	{
		JLabel jLabel;
		JList jList;

		JPanel jPane = new JPanel();
		jPane.setBorder( BorderFactory.createEmptyBorder(5, 5, 5, 5) );
		BoxLayout boxLayout = new BoxLayout(jPane, BoxLayout.PAGE_AXIS);
		jPane.setLayout(boxLayout);

		jLabel = new JLabel();
		jLabel.setName("ListLabel");
		jLabel.setText( resource.translate("TrackNamesDialog.list") );
		jPane.add(jLabel);

		jPane.add( Box.createRigidArea(new Dimension(0, 3)) );

		jList = new JList();
		jList.setName("TrackList");
		jList.setCellRenderer( new CellRenderer() );
		jList.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );
		jList.addMouseListener( markAction );
		controls.putComponent(jList);
		JScrollPane jScrollPane = new JScrollPane(jList);
		jScrollPane.setPreferredSize(new Dimension(200, 300));
		jScrollPane.setAlignmentX(JComponent.LEFT_ALIGNMENT);
		jPane.add(jScrollPane);

		return jPane;
	}

	private void fillIn(JSceneryEditor jSceneryEditor)
	{
		JList jList;

		jList = (JList) controls.getComponent("TrackList");
		jList.setModel( jSceneryEditor.getScenery().getTrackNames() );

		list = jList;
	}

	/*
	 * ------------ Cell renderer -----------
	 */
	
	/**
	 * Custom cell renderer for the list.
	 */
	public class CellRenderer extends DefaultListCellRenderer
	{
		private ImageIcon imageTrack;
		private ImageIcon imageSwitch;

		public CellRenderer()
		{
			imageTrack = new ImageIcon("images/tnl_track.png");
			imageSwitch = new ImageIcon("images/tnl_switch.png");
		}

		public Component getListCellRendererComponent(JList list,
			Object value, int index, boolean isSelected, boolean cellHasFocus)
		{
			Component c = super.getListCellRendererComponent(list, value, index,
				isSelected, cellHasFocus);
			
			JLabel label = (JLabel) c;
			if (value instanceof Switch)
			{
				Switch sswitch = (Switch) value;
				label.setIcon(imageSwitch);
				label.setText(sswitch.getName());
			}
			else if (value instanceof Track)
			{
				Track track = (Track) value;
				label.setIcon(imageTrack);
				label.setText(track.getName());
			}

			return label;
		}
	}
	
	/*
	 * -------------- Listeners --------------
	 */
	
	/**
	 * Mark action when user pushed Mark button or
	 * clicked twice on JList.
	 */
	class MarkAction extends MouseAdapter implements ActionListener
	{
		public MarkAction(JSceneryEditor sce)
		{
			editor = sce;
		}

		public void actionPerformed(ActionEvent ae)
		{
			select();
		}

		public void mouseClicked(MouseEvent me)
		{
			if (me.getClickCount() == 2)
			{
				int index = list.locationToIndex(me.getPoint());
				if (index == list.getSelectedIndex())
				{
					select();
				}
			}
		}

    	/**
		 * User has chosen a list item.
		 */
		private void select()
		{
			Object o = list.getSelectedValue();
			editor.setSelectedObject(o);
			if (o instanceof Track)
			{
				Track track = (Track) o;
				Point3d p = track.getPoint1();
				editor.centerViewAt( editor.modelToView(p) );
			}
			dispose();
		}

		private JSceneryEditor editor;
	}

	/*
	 * --------------- Members ---------------
	 */
	
	private JList list;

	/** I18n resources. */
	private JetResources resource;

	private MarkAction markAction;

	private ControlMap controls = new ControlMap();
}
