/*
 * jET - An alternative scenery editor for the electic locomotive simulator, MaSzyna
 * Copyright (C) 2003 2004  Adam Domaski
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.jet.sce;

import javax.vecmath.Point3d;
import javax.vecmath.Vector3d;

/**
 * Base point is the editor handler. New tracks and switches are appended
 * to it.
 *
 * @author Adam Domaski
 * @version 0.1 24/02/2004
 */
public class BasePoint
{
	private Point3d node;

	private Vector3d vect;

	/**
	 * Constructs new base point with specified coordinates.
	 *
	 * @param p 3D point.
	 * @param v 3D vector.
	 */
	public BasePoint(Point3d p, Vector3d v)
	{
		node = p;
		vect = v;
	}

	/**
	 * Constructs new base point with specified coordinates and zeroed
	 * vector.
	 *
	 * @param p 3D point.
	 * @param alpha Angle between vector and Z axis.
	 * @param beta Angle between vector and XZ plate. The angle must be in 
	 *   range <img src="doc-files/pi2.png">.
	 */
	public BasePoint(Point3d p, double alpha, double beta)
	{
		node = p;
		vect = new Vector3d();
		resolve(alpha, beta);
	}

	/**
	 * Gets the point.
	 */
	public Point3d getPoint()
	{
		return node;
	}

	/**
	 * Gets the vector.
	 */
	public Vector3d getVector()
	{
		return vect;
	}

	private void resolve(double a, double b)
	{
		//if (b < 0.5*Math.PI || b > 0.5*Math.PI)
		//	return;

		double rad = 1.0;
		vect.y = rad * Math.sin(b);
		double d = rad * Math.cos(b);
		vect.x = - d * Math.sin(a);
		vect.z = d * Math.cos(a);
	}

	/**
	 * Computes an angle between vector and Z axis.
	 *
	 * @return Angle in radians in range <img src="doc-files/pi1.png">.
	 */
	public double getAlpha()
	{
		double a = Math.asin(-vect.x / Math.sqrt(vect.x*vect.x + vect.z*vect.z));

		if (vect.z < 0.0)
			a = Math.PI - a;

		return a;
	}

	/**
	 * Computes an angle between vector and XZ plate.
	 *
	 * @return Angle in radians.
	 */
	public double getBeta()
	{
		return Math.atan2(vect.y, Math.sqrt(vect.x*vect.x + vect.z*vect.z));
	}

	/**
	 * Returns string representation of object.
	 */
	public String toString()
	{
		StringBuffer sb = new StringBuffer();

		sb.append("BasePoint[p=");
		sb.append(node);
		sb.append(",v=");
		sb.append(vect);
		sb.append("]");

		return sb.toString();
	}

	/**
	 * Equality test.
	 *
	 * @param o Object to compare with.
	 * @return <code>True</code> when base points equal.
	 */
	public boolean equals(Object o)
	{
		if (o instanceof BasePoint)
			return false;

		BasePoint bp = (BasePoint) o;
		if (bp == this)
			return true;

		boolean result = node.equals(bp.getPoint());
		double beta = getBeta();
		double alpha = getAlpha();
		return (result && beta == bp.getBeta() && alpha == bp.getAlpha());
	}
}
