/*
 * jET - An alternative scenery editor for the electic locomotive simulator, MaSzyna
 * Copyright (C) 2003  Adam Domaski
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.jet.sce;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.FontMetrics;
import java.awt.event.MouseMotionListener;
import java.awt.event.MouseEvent;
import java.awt.geom.Point2D;
import java.awt.geom.AffineTransform;
import java.awt.font.TextLayout;
import java.text.DecimalFormat;
import javax.swing.JComponent;
import javax.swing.JViewport;
import javax.swing.JScrollPane;
import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;
import javax.vecmath.Point3d;

/**
 * A rule for scenery editor.
 * Rule resides in front of scroll bar in scenery editor and
 * it's always visible. There is a linear scale measured
 * in meters on the rule. Rule can be vertical and horizontal.
 *
 * @author Adam Domaski
 * @version 0.1 24/08/2003
 */
public class JRule extends JComponent
	implements MouseMotionListener, ChangeListener, SceneryConstants
{
	private int orientation;
	private int increment = 10;
	private int units = 1;
	private DecimalFormat decimalFormat = new DecimalFormat("0");

	/**
	 * Height or width depending on rule oreintation of rule.
	 */
	public static final int SIZE = 24;
	/**
	 * This constant indicates horizontal rule.
	 */
    public static final int HORIZONTAL = 0;
	/**
	 * This constant indicates vertical rule.
	 */
    public static final int VERTICAL = 1;

	/**
	 * Creates new rule with orientation.
	 *
	 * @param o Orientation of the rule.
	 * @see #HORIZONTAL
	 * @see #VERTICAL
	 */
	public JRule(int o)
	{
		setOrientation(o);
	}

	// ----------------- Orientation methods ---------------

	/**
	 * Sets orientation of this rule.
	 *
	 * @param o Orientation of the rule.
	 * @see #HORIZONTAL
	 * @see #VERTICAL
	 */
	public void setOrientation(int o)
	{
		orientation = o;
	}

	/**
	 * Gets orientation of rule.
	 *
	 * @return Orientation which may be vertical or horizontal.
	 * @see #HORIZONTAL
	 * @see #VERTICAL
	 */
	public int getOrientation()
	{
		return orientation;
	}

	// ------------------ Size methods ------------------

	/**
	 * Sets the width for horizontal rule.
	 *
	 * @param pw New width of horizontal rule.
	 */
	public void setPreferredWidth(int pw)
	{
		setPreferredSize(new Dimension(pw+2, SIZE));
	}

	/**
	 * Sets height for vertical rule.
	 *
	 * @param ph New height of vertical rule.
	 */
	public void setPreferredHeight(int ph)
	{
		setPreferredSize(new Dimension(SIZE, ph+2));
	}

	// ----------- Properties methods ---------------

	/**
	 * Sets incrementation.
	 * The incrementation is amount of pixels between peaks of scale.
	 *
	 * @param i Incrementation.
	 */
	public void setIncrement(int i)
	{
		increment = i;
	}

	/**
	 * Gets incrementation.
	 * The incrementation is amount of pixels between peaks of scale.
	 *
	 * @return Incrementation.
	 */
	public int getIncrement()
	{
		return increment;
	}

	// ----------------- Paint method ------------------

	/**
	 * Paints the component.
	 *
	 * @param g Graphic context.
	 */
	public void paintComponent(Graphics g)
	{
		Rectangle rect = g.getClipBounds();

		// Filling
		g.setColor(new Color(255, 220, 153));
		g.fillRect(rect.x, rect.y, rect.width, rect.height);

		paintScale(g);

		// Draw line pointers on the mouse
		g.setColor(new Color(151, 151, 151));
		if (orientation == HORIZONTAL)
		{
			g.drawLine(trackMouse+1, 10, trackMouse+1, 24);
		}
		else
		{
			g.drawLine(10, trackMouse+1, 24, trackMouse+1);
		}
	}

	/**
	 * Paints scale.
	 * It isinvoked inside <code>paintComponent</code> method.
	 *
	 * @param g Graphics to use.
	 */
	protected void paintScale(Graphics g)
	{
		if (!(getParent() instanceof JViewport))
			return;

		if (!(getParent().getParent() instanceof JScrollPane))
			return;

		JViewport viewport = ((JScrollPane) getParent().getParent()).getViewport();

		if (!(viewport.getView() instanceof JSceneryEditor))
			return;

		JSceneryEditor editor = (JSceneryEditor) viewport.getView();
		double scale = editor.getScale();
		int meters = ((int) (1 / scale)) * 5;

		Point point = new Point();
		Rectangle rect = g.getClipBounds();
		point.x = rect.x;
		point.y = rect.y;
		g.setColor(new Color(0, 51, 153));
		FontMetrics metrics = g.getFontMetrics();

		if (orientation == HORIZONTAL)
		{
			Point3d p3d = editor.viewToModel(point);
			p3d.x = ((int) (p3d.x / meters)) * meters;

			while (point.x < rect.x + rect.width)
			{
				Point2D pp = editor.modelToView(p3d);
				point.x = (int) pp.getX() + 1;
				String label = decimalFormat.format(p3d.x);
				int fontWidth = metrics.stringWidth(label) / 2;
				g.drawString(label, point.x - fontWidth, 12);
				g.drawLine(point.x, 16, point.x, 24);
				p3d.x -= meters;
			}
		}
		else
		{
			if (g instanceof Graphics2D)
			{
				Graphics2D g2 = (Graphics2D) g;
	
				Point3d p3d = editor.viewToModel(point);
				p3d.z = ((int) (p3d.z / meters)) * meters;

				while (point.y < rect.y + rect.height)
				{
					Point2D pp = editor.modelToView(p3d);
					point.y = (int) pp.getY() + 1;
					String label = decimalFormat.format(p3d.z);
					int fontWidth = metrics.stringWidth(label) / 2;
					drawVerticalText(g2, label, 12, point.y + fontWidth);
					g2.drawLine(16, point.y, 24, point.y);
					p3d.z -= meters;
				}
			}
		}
	}

	/**
	 * Draws string vertically on the graphic context.
	 *
	 * @param g2 Graphic context to use.
	 * @param str Text.
	 * @param x The X coordinate of the center of drawn text.
	 * @param y The Y coordinate of the center of drawn text.
	 */
	private void drawVerticalText(Graphics2D g2, String str, int x, int y)
	{
		if (g2 == null || str == null)
			return;

		AffineTransform save = g2.getTransform();
		AffineTransform rotate = AffineTransform.getRotateInstance(Math.toRadians(-90), x, y);
		g2.transform(rotate);
		TextLayout tl = new TextLayout(str, g2.getFont(), g2.getFontRenderContext());
		tl.draw(g2, x, y);
		g2.setTransform(save);
	}

	// ----------------- Mouse motion in scenery editor -------

	/**
	 * This task is executed when mouse in scenery editor moves.
	 * It updates a rule with line pointer.
	 *
	 * @param me Mouse event.
	 */
	public void mouseDragged(MouseEvent me)
	{
		mouseMoved(me);
	}

	/**
	 * This task is executed when mouse in scenery editor moves.
	 * It updates a rule with line pointer.
	 *
	 * @param me Mouse event.
	 */
	public void mouseMoved(MouseEvent me)
	{
		int oldValue = trackMouse;
		if (orientation == HORIZONTAL)
		{
			trackMouse = me.getX();
			// More optimized
			repaint(Math.min(oldValue, trackMouse), 10, Math.abs(oldValue - trackMouse)+2, 14);
		}
		else
		{
			trackMouse = me.getY();
			repaint(10, Math.min(oldValue, trackMouse), 14, Math.abs(oldValue - trackMouse)+2);
		}

	}

	private int trackMouse = 0;

	// ---------- change methods ---------------

	/**
	 * Source for this change listener should come from
	 * viewport associated with this rule.
	 *
	 * @param ce Viewport change.
	 */
	public void stateChanged(ChangeEvent ce)
	{
		if (ce.getSource() instanceof JViewport)
		{
			JViewport viewport = (JViewport) ce.getSource();
			Dimension dim = viewport.getViewSize();
			if (orientation == HORIZONTAL)
			{
				setPreferredWidth(dim.width);
			}
			else
			{
				setPreferredHeight(dim.height);
			}
		}

		revalidate();
		repaint();
	}
}
