/*
 * jET - An alternative scenery editor for the electic locomotive simulator, MaSzyna
 * Copyright (C) 2003 2004  Adam Domaski
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.jet.sce;

import java.io.Reader;
import java.io.Writer;
import java.io.IOException;
import java.awt.*;
import java.awt.geom.Point2D;
import javax.swing.*;
import javax.vecmath.Point3d;

/**
 * Scenery editor for <b>jET</b> application.
 *
 * <p>The goal of this component is separating data model and user interface. 
 * It should be developed in Model-View-Controller pattern and it should have
 * generalized form, similar to Swing text components.</p> 
 *
 * @author Adam Domaski.
 * @version 0.1 21/02/2004
 */
public class JSceneryEditor extends JComponent
	implements Scrollable
{
	/*
	 * ---------------- Properties ------------------
	 */

	/**
	 * Bound property name for scale.
	 */
	public final static String SCALE_PROPERTY = "scale";
	/**
	 * Bound property name for grid.
	 */
	public final static String GRID_VISIBLE_PROPERTY = "gridVisible";
	/**
	 * Bound property name for editing.
	 */
	public final static String EDITABLE_PROPERTY = "editable";
	/**
	 * Bound property name for scenery.
	 */
	public final static String SCENERY_MODEL_PROPERTY = "scenery";
	/**
	 * Bound property name for scenery selection model.
	 */
	public final static String SCENERY_SELECTION_PROPERTY = "scenerySelection";
	/**
	 * Bound property name for base point.
	 */
	public final static String BASEPOINT_PROPERTY = "basePoint";

	/*
	 * --------------- Constructors -----------------
	 */

	/**
	 * Creates new scenery editor with blank scenery.
	 */
	public JSceneryEditor()
	{
		scenery = new Scenery();

		initEditor();
	}

	/**
	 * Creates new scenery editor with given scenery.
	 *
	 * @param scenery Scenery.
	 * @see Scenery
	 */
	public JSceneryEditor(Scenery scenery)
	{
		this.scenery = scenery;

		initEditor();
	}

	/**
	 * Inits.
	 */
	private void initEditor()
	{
		selectionModel = createSelectionModel();
		updateUI();
		setEditable(true);
	}

	// ------------- Plugable look and feel --------------

	/**
	 * Gets user insterface object for this component.
	 *
	 * @return User interface for scenery editor.
	 */
	public SceneryEditorUI getUI()
	{
		return (SceneryEditorUI) ui;
	}

	/**
	 * Sets user interface of this component to user interface manager.
	 *
	 * @param ui User interface to set.
	 */
	public void setUI(SceneryEditorUI ui)
	{
		super.setUI(ui);
	}

	/**
	 * Updates user interface manager with a user interface of this component.
	 */
	public void updateUI()
	{
		setUI(new BasicSceneryEditorUI());
		invalidate();
	}

	static final private String uiClassID = "SceneryEditorUI";

	/**
	 * Gets user interface class name.
	 *
	 * @return String containing user interface class name.
	 */
	public String getUIClassID()
	{
		return uiClassID;
	}

	// ---------------- Implementacja przewijania ------------------

	/**
	 * Gets preferred viewport size for this component when it is
	 * inside scroll pane.
	 *
	 * @return Dimension of preferred viewport.
	 */
	public Dimension getPreferredScrollableViewportSize()
	{
		return getPreferredSize();
	}

	/**
	 * Gets pixels of block increment in scroll pane.
	 */
	public int getScrollableBlockIncrement(Rectangle visibleRect, int orientation,
		int direction)
	{
		if (orientation == SwingConstants.HORIZONTAL)
			return visibleRect.width - maxUnitIncrement;
		else
			return visibleRect.height - maxUnitIncrement;
	}

	/**
	 * Returns <code>false</code>.
	 */
	public boolean getScrollableTracksViewportHeight()
	{
		return false;
	}

	/**
	 * Returns <code>false</code>.
	 */
	public boolean getScrollableTracksViewportWidth()
	{
		return false;
	}

	/**
	 * Gets amount of pixels of unit incremeny in scroll pane.
	 */
	public int getScrollableUnitIncrement(Rectangle visibleRect, int orientation,
		int direction)
	{
		//Get the current position.
		int currentPosition = 0;
		if (orientation == SwingConstants.HORIZONTAL) {
			currentPosition = visibleRect.x;
	    } else {
	        currentPosition = visibleRect.y;
	    }

	    //Return the number of pixels between currentPosition
	    //and the nearest tick mark in the indicated direction.
	    if (direction < 0) {
	        int newPosition = currentPosition -
	                         (currentPosition / maxUnitIncrement)
	                          * maxUnitIncrement;
	        return (newPosition == 0) ? maxUnitIncrement : newPosition;
	    } else {
	        return ((currentPosition / maxUnitIncrement) + 1)
	                 * maxUnitIncrement
	                 - currentPosition;
	    }
	}

	/**
	 * Sets max unit incrementation when editor is in the scroll pane.
	 *
	 * @param pixels amount of pixels to scroll.
	 */
    public void setMaxUnitIncrement(int pixels)
	{
        maxUnitIncrement = pixels;
    }

	/**
	 * Gets max unit incrementation when editor is in the scroll pane.
	 *
	 * @return Incrementation amount in pixels.
	 */
	public int getMaxUnitIncrement()
	{
		return maxUnitIncrement;
	}

	/** Maximum amount of scrolling. */
	private int maxUnitIncrement = 20;

	// --------------------- Flagi ----------------

	/**
	 * Returns <code>true</code> when editor is editable.
	 *
	 * @return <code>True</code> if editable.
	 */
	public boolean isEditable()
	{
		return editable;
	}

	/**
	 * Sets edit ability of scenery editor.
	 *
	 * @param b If <code>true</code>, editor will be editable.
	 */
	public void setEditable(boolean b)
	{
	    boolean oldVal = editable;
	    editable = b;
	    firePropertyChange(EDITABLE_PROPERTY, oldVal, b);
	}

	/**
	 * Returns <code>true</code> if grid is enabled.
	 */
	public boolean isGridEnabled()
	{
		return gridEnabled;
	}

	/**
	 * If set to <code>true</code>, the grid will is active.
	 */
	public void setGridEnabled(boolean b)
	{
		boolean old = gridEnabled;
		gridEnabled = b;
		firePropertyChange(GRID_VISIBLE_PROPERTY, old, b);
	}

	/**
	 * Returns color of the grid.
	 */
	public Color getGridColor()
	{
		return gridColor;
	}

	/**
	 * Sets color for the grid.
	 */
	public void setGridColor(Color color)
	{
		gridColor = color;
	}

	// ------------- Scenerie -------------

	/**
	 * Gets scenery of this scenery editor.
	 * <p>Scenery is a data model for this component.</p>
	 *
	 * @return A scenery of this editor.
	 * @see Scenery
	 */
	public Scenery getScenery()
	{
		return scenery;
	}

	/**
	 * Sets a scenery for scenery editor.
	 * <p>Scenery is a data model for this component.</p>
	 *
	 * @param newScenery A new scenery for this editor.
	 * @see Scenery
	 */
	public void setScenery(Scenery newScenery)
	{
		Scenery old = scenery;
		scenery = newScenery;
		firePropertyChange(SCENERY_MODEL_PROPERTY, old, newScenery);
		setScale(0.002);
	}

	/*
	 * ----------------- Selection -----------------
	 */
	
	/**
	 * Creates default selection model for this editor.
	 *
	 * @return Selection model to use for this editor.
	 */
	protected ScenerySelectionModel createSelectionModel()
	{
		return new DefaultScenerySelectionModel(this);
	}
	
	/**
	 * Sets the selection model for this scenery editor.
	 *
	 * @param sm Scenery selection model.
	 */
	public void setSelectionModel(ScenerySelectionModel sm)
	{
		if (sm != selectionModel)
		{
			ScenerySelectionModel old = selectionModel;
			selectionModel = sm;
			firePropertyChange(SCENERY_SELECTION_PROPERTY, old, sm);
		}
	}

	/**
	 * Gets the selection model for this scenery editor.
	 *
	 * @return Scenery selection model.
	 */
	public ScenerySelectionModel getSelectionModel()
	{
		return selectionModel;
	}

	/**
	 * Clears selection.
	 */
	public void clearSelection()
	{
		selectionModel.clearSelection();
	}

	/**
	 * Determines if selection is empty.
	 *
	 * @return <code>True</code>, if selection is empty.
	 */
	public boolean isSelectionEmpty()
	{
		return selectionModel.isSelectionEmpty();
	}

	/**
	 * Gets the current selection.
	 *
	 * @return Selected object.
	 */
	public Object getSelectedObject()
	{
		return selectionModel.getSelectedObject();
	}

	/**
	 * Sets the selection.
	 *
	 * @param obj Object to select.
	 */
	public void setSelectedObject(Object obj)
	{
		selectionModel.setSelectedObject(obj);
	}

	/**
	 * Adds selection listener.
	 *
	 * @param l Scenery selection listener.
	 */
	public void addScenerySelectionListener(ScenerySelectionListener l)
	{
		selectionModel.addScenerySelectionListener(l);
	}

	/**
	 * Removes selection listener.
	 *
	 * @param l Scenery selection listener to remove.
	 */
	public void removeScenerySelectionListener(ScenerySelectionListener l)
	{
		selectionModel.removeScenerySelectionListener(l);
	}

	/*
	 * --------------- Base point ----------------
	 */
	
	/**
	 * Gets base point.
	 *
	 * @return Base point or <code>null</code> when there is no base point.
	 */
	public BasePoint getBasePoint()
	{
		return basePoint;
	}

	/**
	 * Sets base point.
	 * Note that this method notifies registered property listeners.
	 *
	 * @param bp New base point.
	 * @see #BASEPOINT_PROPERTY
	 */
	public void setBasePoint(BasePoint bp)
	{
		BasePoint old = basePoint;
		basePoint = bp;
		firePropertyChange(BASEPOINT_PROPERTY, old, bp);
	}

	/*
	 * ---------- File writing/reading -------------
	 */

	/**
	 * Reads scenery from reader stream.
	 *
	 * @param in Reader to read.
	 * @exception IOException Thrown when some I/O errors occured.
	 */
	public void read(Reader in) throws IOException
	{
		EditorKit kit = getUI().getEditorKit(this);
		Scenery s = kit.createDefaultScenery();

		kit.read(in, s);
		setScenery(s);
	}

	/**
	 * Writes scenery to file.
	 *
	 * @param out Output to write.
	 * @exception IOException Thrown when some I/O errors occured.
	 */
	public void write(Writer out) throws IOException
	{
		Scenery s = getScenery();
		getUI().getEditorKit(this).write(out, s);
	}

	/*
	 * ---------- Model to view transitions -----------
	 */
	
	/**
	 * Translates model coordinates to component cooridantes.
	 *
	 * @param p 3D point.
	 * @return Point in component coordiantes. 
	 */
	public Point2D modelToView(Point3d p)
	{
		return getUI().modelToView(this, p);
	}

	/**
	 * Translates view coordinates to model coordinates.
	 *
	 * @param p Point in view.
	 * @return 3D point in model coordinates.
	 */
	public Point3d viewToModel(Point2D p)
	{
		return getUI().viewToModel(this, p);
	}

	/**
	 * Scrolls the viewport which includes this scenery editor to
	 * show specified point at its center.
	 *
	 * @param p Point to be shown. 
	 */
	public void centerViewAt(Point2D p)
	{
		int x = (int) p.getX();
		int y = (int) p.getY();

		Component comp = this;
		while (comp != null && !(comp instanceof JViewport))
			comp = comp.getParent();
		if (comp == null)
			return;

		JViewport viewport = (JViewport) comp;
		Dimension vDim = viewport.getExtentSize();
		Dimension pDim = this.getPreferredSize();
		Point vPoint = new Point(x, y);
		vPoint.x -= vDim.width / 2;
		vPoint.y -= vDim.height / 2;
		vPoint.x = Math.max(0, vPoint.x);
		vPoint.x = Math.min(vPoint.x, pDim.width - vDim.width);
		vPoint.y = Math.max(0, vPoint.y);
		vPoint.y = Math.min(vPoint.y, pDim.height - vDim.height);
		viewport.setViewSize(pDim);
		viewport.setViewPosition(vPoint);
	}

	/*
	 * ------------------ The scale ------------------
	 */
	
	/**
	 * Sets new scale for scenery editor.
	 * Scale must be in range >= 0 and <= 1.
	 * Method fires a property change <code>SCALE_PROPERTY</code>.
	 *
	 * @param newScale New scale.
	 */
	public void setScale(double newScale)
	{
		if (newScale > 0 && newScale <= 1)
		{
			double old = scale;
			scale = newScale;
			firePropertyChange(SCALE_PROPERTY, old, newScale);

			//((BasicSceneryEditorUI) getUI()).updateSize();
		}
	}

	/**
	 * Gets scenery editor scale.
	 *
	 * @return Scale of scenery editor. Scale is in range
	 *   >=0 and <= 1.
	 */
	public double getScale()
	{
		return scale;
	}

	// ------------- Zmienne --------------

	/** Editable flag. */
	private boolean editable;

	/** Editor grid flag. */
	private boolean gridEnabled;

	/** Grid color. */
	private Color gridColor;

	/** Data model. */
	protected Scenery scenery;

	/** Scale. */
	private double scale = 1.0;

	/** Selection model. */
	protected ScenerySelectionModel selectionModel;

	/** Base point. */
	private BasePoint basePoint;
}
