/*
 * jET - An alternative scenery editor for the electic locomotive simulator, MaSzyna
 * Copyright (C) 2003 2004  Adam Domaski
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.jet.sce;

import java.io.PrintStream;
import java.io.Serializable;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import org.jet.tracks.*;

/**
 * The scenery is the class where data for editor is stored.
 * It has two lists containing tracks and switches. In addition
 * scenery may have custom properties, and description.
 * <p>Be caution about securing read access to scenery. See readLock() 
 * and readUnlock() methods.</p>
 *
 * @author Adam Domaski
 * @version 0.1 15/02/2004
 */
public class Scenery implements Serializable
{
	public Scenery()
	{
		nameIndex = new NamesIndices();
	}

	/*
	 * --------- General methods ----------
	 */

	/**
	 * Inserts array of object to the scenery.
	 *
	 * @param obj Elements to insert to.
	 * @exception IllegalArgumentException When array is null.
	 */
	public void insert(Object[] obj)
	{
		// Sprawd czy tablica nie jest pusta
		if (obj == null)
			throw new IllegalArgumentException("An array is empty");
		
		writeLock();
		try
		{
			int n = obj.length;
			for (int i = 0; i < n; i++)
			{
				addLL(obj[i]);
			}

			SceneryEvent event = new DefaultSceneryEvent( 
				SceneryEvent.INSERT, null, obj);
			fireSceneryChange(event);
		}
		finally
		{
			writeUnlock();
		}
	}

	/**
	 * Removes object from scenery.
	 *
	 * @param obj An array of elements to remove.
	 * @exception IllegalArgumentException When array is null or
	 *   some of elements in array were not in scenery.
	 */
	public void remove(Object[] obj)
	{
		// Sprawd czy tablica nie jest pusta
		if (obj == null)
			throw new IllegalArgumentException("An array is empty");

		// Sprawd czy elementy s w scenerii
		int i = 0;
		boolean cont = true;
		int oL = obj.length;

		while (i < oL && cont)
		{
			cont = contains( obj[i] );
			i++;
		}
		if (!cont)
			throw new IllegalArgumentException("Scenery does not contain some elements");

		// Usu elementy
		writeLock();
		try
		{
			int n = obj.length;
			for (i = 0; i < n; i++)
				removeLL(obj[i]);

			SceneryEvent event = new DefaultSceneryEvent( 
				SceneryEvent.REMOVE, obj, null);
			fireSceneryChange(event);
		}
		finally
		{
			writeUnlock();
		}
	}

	/**
	 * Replaces old object with new ones.
	 * Length of this two arrays has to match.
	 * If some of objects in old array is not in scenery,
	 * it causes to throw an exception.
	 *
	 * @param o Objects stored in scenery.
	 * @param n New object.
	 * @exception IllegalArgumentException When length of the arrays are not equal or
	 *   scenery does not contain some of elements of the first array.
	 */
	public void replace(Object[] o, Object[] n)
	{
		int i = 0;
		boolean cont = true;
		int oL = o.length;

		if (oL != n.length)
			throw new IllegalArgumentException("Array lengths are not equal.");

		while (i < oL && cont)
		{
			cont = contains( o[i] );
			i++;
		}
		if (!cont)
			throw new IllegalArgumentException("Scenery does not contain some elements");

		writeLock();
		try
		{
			i = 0;
			while (i < oL)
			{
				replaceLL(o[i], n[i]);
				i++;
			}

			SceneryEvent event = new DefaultSceneryEvent(
				SceneryEvent.CHANGE, o, n);
			fireSceneryChange(event);
		}
		finally
		{
			writeUnlock();
		}
	}

	/**
	 * Returns iterator of elements's collection given class.
	 *
	 * @param c Class of elements.
	 * @return Iterator or null when class is not supported.
	 */
	public Iterator getIterator(Class c)
	{
		if (c.equals(Switch.class))
			return switches.iterator();
		else if (c.equals(Track.class))
			return tracks.iterator();

		return null;
	}

	/**
	 * Returns element's count of specified class.
	 *
	 * @param c Class of elements.
	 * @return Count of elements or -1 when scenery does not
	 *   contains elements specified class. 
	 */
	public int getCount(Class c)
	{
		if (c.equals(Switch.class))
			return switches.size();
		else if (c.equals(Track.class))
			return tracks.size();

		return -1;
	}

	/*
	 * ---- Low level data manipulation ----
	 */

	/**
	 * Tests if scenery contains specified object.
	 *
	 * @param obj Element to find in scenery.
	 * @return <code>True</code> if scenery contains the object
	 */
	public boolean contains(Object o)
	{
		if (o instanceof Switch)
			return switches.contains(o);
		else if (o instanceof Track)
			return tracks.contains(o);
		
		return false;
	}

	private boolean addLL(Object o)
	{
		boolean success = false;

		if (o instanceof Switch)
			success = switches.add(o);
		else if (o instanceof Track)
			success = tracks.add(o);

		if (o instanceof Track && ((Track) o).getName() != null)
		{
			nameIndex.add(o);
		}

		return success;
	}

	private boolean removeLL(Object o)
	{
		boolean success = false;

		if (o instanceof Switch)
			success = switches.remove(o);
		else if (o instanceof Track)
			success = tracks.remove(o);

		if (o instanceof Track && ((Track) o).getName() != null)
		{
			nameIndex.remove(o);
		}

		return success;
	}

	private void replaceLL(Object o, Object n)
	{
		if (o instanceof Switch)
		{
			switches.remove(o);
			switches.add(n);
		}
		else if (o instanceof Track)
		{
			tracks.remove(o);
			tracks.add(n);
		}

		if (o instanceof Track && ((Track) o).getName() != null)
		{
			nameIndex.remove(o);
		}

		if (n instanceof Track && ((Track) n).getName() != null)
		{
			nameIndex.add(n);
		}
	}

	/*
	 * -------- Name indices retriving ---------
	 */
	
	/**
	 * Returns data model for list of track and switch names.
	 */
	public ListModel getTrackNames()
	{
		return nameIndex;
	}

	/*
	 * -------------- Description --------------
	 */
	
	/**
	 * Sets scenery description.
	 * <p>Scenery description describes scenery and it is usefull to
	 * include some user data.</p>
	 *
	 * @param content The description.
	 */
	public void setDescription(String content)
	{
		description = content;
	}

	/**
	 * Gets scenery description.
	 * <p>Scenery description describes scenery and it is usefull to
	 * include some user data.</p>
	 *
	 * @return The description.
	 */
	public String getDescription()
	{
		return description;
	}

	/*
	 * ---------------- Events -----------------
	 */

	/**
	 * Adds a scenery listener to this document for notification of any changes.
	 *
	 * @param listener The scenery listener to add.
	 * @see Scenery#addSceneryChangeListener
	 */
	public void addSceneryChangeListener(SceneryChangeListener listener)
	{
		listenerList.add(SceneryChangeListener.class, listener);
	}

	/**
	 * Removes a scenery listener.
	 *
	 * @param listener The scenery listener to remove.
	 * @see Scenery#removeSceneryChangeListener
	 */
	public void removeSceneryChangeListener(SceneryChangeListener listener)
	{
		listenerList.remove(SceneryChangeListener.class, listener);
	}

	/**
	 * Returns an array of all registered scenery listeners on this
	 * document.
	 *
	 * @return All of registered scenery listeners on this document.
	 * @see #addSceneryChangeListener
	 * @see #removeSceneryChangeListener
	 */
	public SceneryChangeListener[] getSceneryChangeListeners()
	{
		return (SceneryChangeListener[]) listenerList.getListeners(
			SceneryChangeListener.class);
	}

	/**
	 * Adds an undo listener for notification of any changes.
	 *
	 * @param listener The undo listener to add.
	 * @see Scenery#addUndoableEditListener
	 */
    public void addUndoableEditListener(UndoableEditListener listener)
	{
		listenerList.add(UndoableEditListener.class, listener);
    }

	/**
	 * Returns an array of all registered undo listeners on this
	 * document.
	 *
	 * @return All undo listeners in array.
	 * @see #addUndoableEditListener
	 * @see #removeUndoableEditListener
	 */
    public UndoableEditListener[] getUndoableEditListeners()
	{
        return (UndoableEditListener[])listenerList.getListeners(
                UndoableEditListener.class);
    }

	/**
	 * Removes an undo listener.
	 *
	 * @param listener The undo listener to remove.
	 * @see Scenery#removeUndoableEditListener
	 */
    public void removeUndoableEditListener(UndoableEditListener listener)
	{
		listenerList.remove(UndoableEditListener.class, listener);
    }

	/**
	 * Returns an array of registered <em>foo</em> listeners on
	 * this scenery.
	 * for example you may query a scenery with the following code:
	 *
	 * <pre>SceneryChangeListener[] mls = 
	 (SceneryChangeListener[])(d.getListeners(SceneryChangeListener.class));</pre>
	 *
	 * @param listnerType Class of desired result.
	 * @return The array of listener of specified class.
	 * @exception ClassCastException If <code>listenerType</code> dosen't
	 *   specify a class or interface that implements <code>java.util.EventListener</code>.
	 *
	 * @see #getSceneryChangeListeners
	 * @see #getUndoableEditListeners
	 */
	public EventListener[] getListeners(Class listenerType)
	{
		return listenerList.getListeners(listenerType);
	}
	
	/*
	 * ---------------- Event firing -----------------
	 */
	
	/**
	 * Fires scenery change event to all registered scenery listeners.
	 *
	 * @param sce Scenery event to fire.
	 */
	protected void fireSceneryChange(SceneryEvent sce)
	{
		notifyingListeners = true;
		try
		{
			Object[] listeners = listenerList.getListenerList();
			for (int i = listeners.length-2; i>=0; i-=2)
			{
				if (listeners[i] == SceneryChangeListener.class)
				{
					((SceneryChangeListener) listeners[i+1]).sceneryChanged(sce);
				}
			}
		}
		finally
		{
			notifyingListeners = false;
		}
	}

	/*
	 * ---------------- Properties --------------------
	 */
	
	/**
	 * Scenery may contain some properties. This method retrives it all.
	 *
	 * @return The scenery properties.
	 * @see #setSceneryProperties
	 */
	public Dictionary getSceneryProperties()
	{
		if (sceneryProperties == null)
		{
			sceneryProperties = new Hashtable(2);
		}
		return sceneryProperties;
	}

	/**
	 * Scenery may contain some properties. This method replaces
	 * scenery properties with new one.
	 *
	 * @param x New properties.
	 * @see #getSceneryProperties
	 */
	public void setSceneryProperties(Dictionary x)
	{
		sceneryProperties = x;
	}

	/**
	 * Gets a property with key <code>key</code>.
	 *
	 * @param key A property key.
	 * @return Value of property.
	 * @see #putProperty
	 */
	public final Object getProperty(Object key)
	{
		return getSceneryProperties().get(key);
	}

	/**
	 * Sets property for the scenery.
	 * It allows to remove property specified by a <code>key</code>,
	 * when value is <code>null</code>.
     *
	 * @param key A property key.
	 * @param value A property value. If <code>null</code> property
	 * will be removed.
	 * @see #getProperty
	 */
	public final void putProperty(Object key, Object value)
	{
		if (value != null)
		{
			getSceneryProperties().put(key, value);
		}
		else
		{
			getSceneryProperties().remove(key);
		}
	}

	/*
	 * ---------- Scenery locking/unlocking -----------
	 */
	
	/**
	 * Acquires a scenery lock for writing.
	 *
	 * @exception IllegalStateException Thrown on illegal
	 *   lock attempt.
	 * @see #writeUnlock()
	 */
	protected synchronized final void writeLock()
	{
		try
		{
			while ((numReaders > 0) || (currWriter != null))
			{
				if (Thread.currentThread() == currWriter)
				{
					if (notifyingListeners)
					{
						throw new IllegalStateException("Attempt to mutate in notification");
					}
					numWriters++;
					return;
				}
				wait();
			}
			currWriter = Thread.currentThread();
			numWriters = 1;
		}
		catch (InterruptedException ie)
		{
			throw new Error("Interrupted attempt to aquire write lock");
		}
	}

	/**
	 * Releases a write lock previously obtained via <code>writeLock</code>.
	 *
	 * @see #writeLock()
	 */
	protected synchronized final void writeUnlock()
	{
		if (--numWriters <= 0)
		{
			numWriters = 0;
			currWriter = null;
			notifyAll();
		}
	}

	/**
	 * Acquires a lock to begin reading from the scenery.
	 * There can be multiple readers at the same time.
	 *
	 * @see #readUnlock()
	 */
	public synchronized final void readLock()
	{
		try
		{
			while (currWriter != null)
			{
				if (currWriter == Thread.currentThread())
				{
					return;
				}
				wait();
			}
			numReaders += 1;
		}
		catch (InterruptedException e)
		{
			throw new Error("Interrupted attempt to aquire read lock");
		}
	}

	/**
	 * Does a read unlock.
	 *
	 * @see #readLock()
	 */
	public synchronized final void readUnlock()
	{
		if (currWriter == Thread.currentThread())
		{
			return;
		}
		if (numReaders <= 0)
		{
			throw new Error("State invariant error");
		}
		numReaders -= 1;
		notify();
	}

	/*
	 * ----------------- Debugging --------------------
	 */
	
	/**
	 * Dumps debugging info.
	 *
	 * @param psOut Print stream to output.
	 */
	public void dump(PrintStream psOut)
	{
		Iterator i;
		
		i = tracks.iterator();
		while (i.hasNext())
		{
			psOut.println(i.next());
		}

		i = switches.iterator();
		while (i.hasNext())
		{
			psOut.println(i.next());
		}
	}

	/*
	 * --------- Track and switch names indices.
	 */
	
	/**
	 * The indices for track and switch names, implementing
	 *   a list data model to speed up searching for track names.
	 */
	public class NamesIndices extends AbstractListModel
	{
		private ArrayList index;
		private Comparator comp;

		/**
		 * Creates new instance;
		 */
		public NamesIndices()
		{
			index = new ArrayList();
			comp = new Comparator() {
				public int compare(Object o1, Object o2)
				{
					Track t1 = (Track) o1;
					Track t2 = (Track) o2;

					return t1.getName().compareTo( t2.getName() );
				}
			};
		}

		/**
		 * Gets elements at specified position.
		 */
		public Object getElementAt(int i)
		{
			return index.get(i);
		}

		/**
		 * Gets list size.
		 */
		public int getSize()
		{
			return index.size();
		}

		/**
		 * Adds object to the list and sorts it immediatly.
		 */
		public void add(Object o)
		{
			index.add(o);
			Collections.sort(index, comp);
		}

		/**
		 * Removes object from list.
		 */
		public void remove(Object o)
		{
			index.remove(o);
		}
	}

	/*
	 * ---------- Event object class -------------
	 */
	
	/**
	 * Scenery event standard implementation.
	 */
	public class DefaultSceneryEvent extends EventObject
		implements SceneryEvent
	{
		/**
		 * Creates new scenery event.
		 *
		 * <p>Insert event uses only <code>inserted</code> array. The second one 
		 * should be <code>null</code>.</p>
		 * <p>Change event uses two arrays. The <code>removed</code> array indicates
		 *   elements to be replaced. The <code>inserted</code> array indicates
		 *   elements replacing.</p>
		 * <p>Remove event uses only <code>removed</code> array. The second one
		 * should be set to <code>null</code>.</p>
		 *
		 * @param type Type of event. One of predefined values.
		 * @param removed Array of elements removed from the scenery.
		 * @param inserted Array of elements inserted to the scenery.
		 * @see SceneryEvent#INSERT
		 * @see SceneryEvent#CHANGE
		 * @see SceneryEvent#REMOVE
		 */
		public DefaultSceneryEvent(int type, Object[] removed, Object[] inserted)
		{
			super(Scenery.this);
			this.type = type;
			objectsRemoved = removed;
			objectsInserted = inserted;
		}

		/**
		 * Returns scenery which sent this event.
		 */
		public Scenery getScenery()
		{
			return (Scenery) source;
		}

		/**
		 * Gets type of event.
		 * Event types are predefined values.
		 *
		 * @see SceneryEvent#INSERT
		 * @see SceneryEvent#CHANGE
		 * @see SceneryEvent#REMOVE
		 */
		public int getType()
		{
			return type;
		}

		/**
		 * Gets array of inserted elements.
		 */
		public Object[] getInserted()
		{
			return objectsInserted;
		}

		/**
		 * Gets array of removed elements.
		 */
		public Object[] getRemoved()
		{
			return objectsRemoved;
		}

		private Object[] objectsInserted;

		private Object[] objectsRemoved;

		private int type;
	}

	/*
	 * ------------------ Members ---------------------
	 */
	
	/** Event listener list. */
	protected EventListenerList listenerList = new EventListenerList();

	/** Scenery properties */
	private Dictionary sceneryProperties = null;

	/** Set containing scenery elements. */
	protected Set tracks = new LinkedHashSet(100, 100);

	/** Set containing switches. */
	protected Set switches = new LinkedHashSet(100, 100);

	/** Text description of this scenery. */
	private String description = "";

	/** Current writer thread. */
	private transient Thread currWriter;

	/** Number of readers. */
	private transient int numReaders;

	/** Number of writers. */
	private transient int numWriters;

	/** Flag of notifing listeners by the event dispatching thread. */
	private transient boolean notifyingListeners;

	/** Name indices. */
	private NamesIndices nameIndex;
}
