/*
 * jET - An alternative scenery editor for the electic locomotive simulator, MaSzyna
 * Copyright (C) 2003 2004  Adam Domaski
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.jet.sce;

import java.awt.*;
import java.awt.font.*;
import java.awt.geom.*;

/**
 * This is a box surrounding text.
 * It is like geometric figure.
 *
 * @author Adam Domaski
 * @version 0.1 18/02/2004
 */
public class TextBox
{
	/** The text object. */
	protected TextLayout text;

	private double posX = 0.0;
	private double posY = 0.0;

	/*
	 * ------------- Constructors --------------
	 */
	
	/**
	 * Constructs text box at point (0, 0).
	 *
	 * @param name Track name.
	 * @param font The font.
	 * @param frc Font rendering context.
	 */
	public TextBox(String name, Font font, FontRenderContext frc)
	{
		this.text = new TextLayout(name, font, frc);
	}

	/*
	 * --------------- Location ----------------
	 */

	/**
	 * Sets location to the given point.
	 *
	 * @param x X coordinate.
	 * @param y Y coordinate.
	 */
	public void setLocation(double x, double y)
	{
		posX = x;
		posY = y;
	}

	/**
	 * Sets location of the text box to the point.
	 *
	 * @param p New point.
	 */
	public void setLocation(Point2D p)
	{
		posX = p.getX();
		posY = p.getY();
	}

	/**
	 * Gets the Y coordinate of the location.
	 */
	public double getY()
	{
		return posY;
	}

	/**
	 * Gets the X coordinate of the location.
	 */
	public double getX()
	{
		return posX;
	}

	/**
	 * Gets the location point.
	 */
	public Point2D getLocation()
	{
		return new Point2D.Double(posX, posY);
	}

	/*
	 * ---------- Drawing -----------
	 */

	/**
	 * Draws the text box on the graphics context.
	 * This method draws the text and rectangle around it.
	 */
	public void draw(Graphics2D g)
	{
		text.draw(g, (float) posX, (float) posY);
		g.draw( this.getBounds2D() );
	}

	/**
	 * Sets the text layout for the text box.
	 */
	public void setText(TextLayout text)
	{
		this.text = text;
	}

	/**
	 * Gets the text layout for the text box.
	 */
	public TextLayout getText()
	{
		return text;
	}

	/**
	 * Determines if the text box intersects with given rectangle.
	 *
	 * @param r Rectangle to test.
	 * @return <code>True</code>, if this text box intersects with 
	 *   given rectangle.
	 */
	public boolean intersects(Rectangle r)
	{
		return getBounds().intersects(r);
	}

	/**
	 * Determines if the text box contains given rectangle.
	 *
	 * @param r Rectangle to test.
	 * @return <code>True</code>, if this text box contains rectangle.
	 */
	public boolean contains(Rectangle r)
	{
		return getBounds().contains(r);
	}

	/**
	 * Gets the bounds of the text layout and rectangle anround it.
	 */
	public Rectangle2D getBounds2D()
	{
		Rectangle2D rect = text.getBounds();
		Rectangle2D.Float rectD = new Rectangle2D.Float();
		rectD.x = (float) (posX + rect.getX() - 2);
		rectD.y = (float) (posY + rect.getY() - 2);
		rectD.width = (float) (rect.getWidth() + 4);
		rectD.height = (float) (rect.getHeight() + 4);

		return rectD;
	}

	/**
	 * Gets the bounds of the text layout and rectangle anround it.
	 */
	public Rectangle getBounds()
	{
		Rectangle rect = new Rectangle();
		Rectangle2D rectB = getBounds2D();
		rect.x = (int) rectB.getX();
		rect.y = (int) rectB.getY();
		rect.width = (int) rectB.getWidth() + 2;
		rect.height = (int) rectB.getHeight() + 2;

		return rect;
	}
}
