/*
 * jET - An alternative scenery editor for the electic locomotive simulator, MaSzyna
 * Copyright (C) 2003 2004  Adam Domaski
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.jet.tracks;

import javax.vecmath.*;

import org.jet.JetResources;

/**
 * Switch entity.
 *
 * @author Adam Domaski
 * @version 0.1 11/03/2004
 */
public class Switch extends Track
{
	public static final String SWITCH_TEXTURE_DEFAULT = "Rail_screw_used1";

	public static final String SWITCH_SECOND_TEXTURE_DEFAULT = "Rail_screw_used1";

	/*
	 * ------- Members ---------
	 */

	/** End point of reversible track. */
	protected Point3d node3;

	/** Vector mounted at start point to the reversible track. */
	protected Vector3d vecSwitched;

	/** Vector mounted at end point of reversible track to start point of track. */
	protected Vector3d vec3;

	/** Roll at start point of reversible track. */
	protected double rollSwitched;

	/** Roll at end point of reversible track. */
	protected double roll3;

	/** Radius of reversible track. */
	protected double radiusSwitched;

	/** Angle of reversible track (in degrees). */
	private double reversibleAngle;

	/** I18n resource. */
	private JetResources resource = JetResources.getInstance();

	/*
	 * ----------- Contructors --------------
	 */
	
	/**
	 * Dummy constructor.
	 */
	public Switch()
	{
		super();
		vecSwitched = new Vector3d();
		vec3 = new Vector3d();
		node3 = new Point3d();
	}

	/*
	 * ------------- Geometry ---------------
	 */

	/**
	 * Gets end point of switch.
	 */
	public Point3d getPoint3()
	{
		return node3;
	}

	/**
	 * Sets the end point of the reversible track of switch.
	 */
	public void setPoint3(Point3d p)
	{
		node3 = p;
	}

	/**
	 * Sets the end point of the reversible track of switch.
	 */
	public void setPoint3(double x, double y, double z)
	{
		node3.set(x, y, z);
	}

	/**
	 * Sets the end point of the reversible track of switch.
	 */
	public void setPoint3(double[] array)
	{
		node3.set(array);
	}

	/**
	 * Gets the vector mounted at start point to the end of reversible track.
	 */
	public Vector3d getSwitchedVector()
	{
		return vecSwitched;
	}

	/**
	 * Sets the vector mounted at start point to the end of reversible track.
	 */
	public void setSwitchedVector(Vector3d v)
	{
		vecSwitched = v;
	}

	/**
	 * Sets the vector mounted at start point to the end of reversible track.
	 */
	public void setSwitchedVector(double x, double y, double z)
	{
		vecSwitched.set(x, y, z);
	}

	/**
	 * Sets the vector mounted at start point to the end of reversible track.
	 */
	public void setSwitchedVector(double[] array)
	{
		vecSwitched.set(array);
	}

	/**
	 * Gets the vector mounted at the end of reversible track.
	 */
	public Vector3d getVector3()
	{
		return vec3;
	}

	/**
	 * Sets the vector mounted at the end of reversible track.
	 */
	public void setVector3(Vector3d v)
	{
		vec3 = v;
	}

	/**
	 * Sets the vector mounted at the end of reversible track.
	 */
	public void setVector3(double x, double y, double z)
	{
		vec3.set(x, y, z);
	}

	/**
	 * Sets the vector mounted at the end of reversible track.
	 */
	public void setVector3(double[] array)
	{
		vec3.set(array);
	}

	/**
	 * Gets the roll at start point of reversible track.
	 */
	public double getSwitchedRoll()
	{
		return rollSwitched;
	}

	/**
	 * Sets the roll at start point of reversible track.
	 */
	public void setSwitchedRoll(double d)
	{
		rollSwitched = d;
	}

	/**
	 * Gets the roll at end point of reversible track.
	 */
	public double getRoll3()
	{
		return roll3;
	}

	/**
	 * Sets the roll at end pointof reversible track.
	 */
	public void setRoll3(double d)
	{
		roll3 = d;
	}

	/**
	 * Gets radius of reversible track.
	 */
	public double getSwitchedRadius()
	{
		return radiusSwitched;
	}

	/**
	 * Sets radius of reversible track.
	 */
	public void setSwitchedRadius(double d)
	{
		radiusSwitched = d;
	}

	/*
	 * --------- object methods -------------
	 */
	
	/**
	 * Test if two switches are equal.
	 *
	 * @return <code>True</code> if they are.
	 */
	public boolean equals(Object o)
	{
		if (!(o instanceof Switch))
			return false;

		Switch s = (Switch) o;

		boolean b = super.equals(o) &&
			node3.equals(s.getPoint3()) &&
			vec3.equals(s.getVector3()) &&
			vecSwitched.equals(s.getSwitchedVector()) &&
			radiusSwitched == s.getSwitchedRadius() &&
			roll3 == s.getRoll3() &&
			rollSwitched == s.getSwitchedRoll();

		return b;
	}

	/**
	 * Gets description.
	 */
	public String toString()
	{
		StringBuffer sb = new StringBuffer();
		sb.append("Switch[name=");
		if (name != null && !name.equals("none"))
		{
			sb.append(name);
		}
		sb.append(",P1=(");
		sb.append(node1);
		sb.append("),V1=(");
		sb.append(vec1);
		sb.append("),V2=(");
		sb.append(vec2);
		sb.append("),P2=(");
		sb.append(node2);
		sb.append("),PS=(");
		sb.append(node3);
		sb.append("),VS1=(");
		sb.append(vecSwitched);
		sb.append("),VS2=(");
		sb.append(vec3);
		sb.append("),length=");
		sb.append(length);
		sb.append(",width=");
		sb.append(width);
		sb.append(",friction=");
		sb.append(friction);
		sb.append(",tapDistance=");
		sb.append(tapDistance);
		sb.append(",quality=");
		sb.append(quality);
		sb.append(",damage=");
		sb.append(damage);
		sb.append(",enviroment=");
		sb.append(enviroment);
		sb.append(",visibility=");
		sb.append(visibility);
		sb.append(",texture=");
		sb.append(texture);
		sb.append(",textureLength=");
		sb.append(textureLength);
		sb.append(",secondTexture=");
		sb.append(secondTexture);
		sb.append(",textureHeight=");
		sb.append(texHeight);
		sb.append(",texWidth=");
		sb.append(texWidth);
		sb.append(",texSlope=");
		sb.append(texSlope);
		sb.append(",texHeight=");
		sb.append(texHeight);
		sb.append(",roll1=");
		sb.append(roll1);
		sb.append(",roll2=");
		sb.append(roll2);
		sb.append(",rollS1=");
		sb.append(rollSwitched);
		sb.append(",rollS2=");
		sb.append(roll3);
		sb.append(",radius=");
		sb.append(radius);
		sb.append(",radiusS=");
		sb.append(radiusSwitched);
		sb.append(",switchAngle=");
		sb.append(reversibleAngle);
		sb.append("]");

		return sb.toString();
	}

	/**
	 * Returns true if the side track is straight.
	 */
	public boolean isSideStraight()
	{
		return (vecSwitched.equals(VECTOR_ZERO) && vec3.equals(VECTOR_ZERO));
	}

	private boolean rightSwitch;
}
