/*
 * jET - An alternative scenery editor for the electic locomotive simulator, MaSzyna
 * Copyright (C) 2003 2004  Adam Domaski
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.jet.tracks;

import javax.vecmath.*;
import java.io.Serializable;

/**
 * The track entity.
 *
 * @author Adam Domaski
 * @version 0.1 12/03/2004
 */
public class Track implements Serializable
{
	/*
	 * ------------ Constants -------------
	 */
	
	/** Lowlands enviroment. */
	public static final String FLAT = "flat";

	/** Canyon enviroment. */
	public static final String CANYON = "canyon";

	/** Tunnel enviroment. */
	public static final String TUNNEL = "tunnel";

	/** Mountains enviroment. */
	public static final String MOUNTAINS = "mountains";

	/** Bank enviroment. */
	public static final String BANK = "bank";

	/** Bridge enviroment. */
	public static final String BRIDGE = "bridge";

	/** Maximal track roll in degrees in PKP. */
	public static final double ROLL = 5.96;

	/** Default texture name. */
	public static final String TEXTURE_DEFAULT = "Rail_screw_used1";

	/** Default second texture name. */
	public static final String SECOND_TEXTURE_DEFAULT = "TpBpS-new2.tex";

	/** Vector zero. */
	protected static final Vector3d VECTOR_ZERO = new Vector3d();

	/*
	 * ------------- Members --------------
	 */
	
	/** The name. */
	protected String name = null;

	/** 1<sup>st</sup> end point of the track. */
	protected Point3d node1;
	/** 2<sup>nd</sup> end point of the track. */
	protected Point3d node2;

	/** 1<sup>st</sup> control vector of the track. */
	protected Vector3d vec1;
	/** 2<sup>nd</sup> control vector of the track. */
	protected Vector3d vec2;

	/** Track length. */
	protected double length;

	/** Track width. */
	protected double width;

	/** Friction. */
	protected double friction;

	/** Tap distance. */
	protected double tapDistance;

	/** 
	 * Quality of the track.
	 * Values are combination of bits.
	 */
	protected int quality;

	/**
	 * Track damage flag.
	 * Values are combination of bits.
	 */
	protected int damage;

	/** The enviroment. */
	protected String enviroment;

	/** Visibility flag. */
	protected boolean visibility;

	/** Texture. */
	protected String texture = null;

	/** Texture length. */
	protected double textureLength;

	/** Second texture. */
	protected String secondTexture;

	/** Texture height. */
	protected double texHeight;

	/** Texture width. */
	protected double texWidth;

	/** Texture slope. */
	protected double texSlope;

	/** Roll at 1<sup>st</sup> point. */
	protected double roll1;

	/** Roll at 2<sup>nd</sup> point. */
	protected double roll2;

	/**
	 * Radius.
	 * It should be <code>0</code> for flex and straight tracks.
	 */
	protected double radius;

	/*
	 * ----------- Constructors -------------
	 */

	/**
	 * Constructs new empty track.
	 */
	public Track()
	{
		node1 = new Point3d();
		node2 = new Point3d();
		vec1 = new Vector3d();
		vec2 = new Vector3d();
	}

	/**
	 * Constructs new track with Beizer's curve nodes.
	 *
	 * @param node1 Start point.
	 * @param node2 End point.
	 * @param vec1 The 1<sup>st</sup> control vector.
	 * @param vec2 The 2<sup>nd</sup> control vector.
	 */
	public Track(Point3d node1, Point3d node2, Vector3d vec1, Vector3d vec2)
	{
		this.node1 = node1;
		this.node2 = node2;
		this.vec1 = vec1;
		this.vec2 = vec2;
	}

	/*
	 * ------ Geometry methods ----------
	 */
	
	/**
	 * Gets the track incline.
	 * <p>For flex track it will be not true.</p>
	 */
	public double getIncline()
	{
		double height = Math.abs(node2.y - node1.y);
		double dist = Math.sqrt((node2.x - node1.x) * (node2.x - node1.x) +
			(node2.z - node1.z)*(node2.z - node1.z));

		return height / dist;
	}

	/*
	 * --------- Misc methods -----------
	 */

	/**
	 * Gets description.
	 */
	public String toString()
	{
		StringBuffer sb = new StringBuffer();
		sb.append("Track[name=");
		if (name != null && !name.equals("none"))
		{
			sb.append(name);
		}
		sb.append(",P1=(");
		sb.append(node1);
		sb.append("),V1=(");
		sb.append(vec1);
		sb.append("),V2=(");
		sb.append(vec2);
		sb.append("),P2=(");
		sb.append(node2);
		sb.append("),length=");
		sb.append(length);
		sb.append(",width=");
		sb.append(width);
		sb.append(",friction=");
		sb.append(friction);
		sb.append(",tapDistance=");
		sb.append(tapDistance);
		sb.append(",quality=");
		sb.append(quality);
		sb.append(",damage=");
		sb.append(damage);
		sb.append(",enviroment=");
		sb.append(enviroment);
		sb.append(",visibility=");
		sb.append(visibility);
		sb.append(",texture=");
		sb.append(texture);
		sb.append(",textureLength=");
		sb.append(textureLength);
		sb.append(",secondTexture=");
		sb.append(secondTexture);
		sb.append(",textureHeight=");
		sb.append(texHeight);
		sb.append(",texWidth=");
		sb.append(texWidth);
		sb.append(",texSlope=");
		sb.append(texSlope);
		sb.append(",texHeight=");
		sb.append(texHeight);
		sb.append(",roll1=");
		sb.append(roll1);
		sb.append(",roll2=");
		sb.append(roll2);
		sb.append(",radius=");
		sb.append(radius);
		sb.append("]");

		return sb.toString();
	}

	/**
	 * Equality test.
	 *
	 * @param o Object to check.
	 * @return <code>True</code>, if tracks are equal.
	 */
	public boolean equals(Object o)
	{
		if (!(o instanceof Track))
			return false;

		if (this == o)
			return true;

		Track t = (Track) o;

		boolean b;
		if (name != null)
		{
			b = name.equals( t.getName() );
		}
		else
		{
			b = (name == t.getName());
		}

		b = b &&
			node1.equals( t.getPoint1() ) &&
			node2.equals( t.getPoint2() ) &&
			vec1.equals( t.getVector1() ) &&
			vec2.equals( t.getVector2() ) &&
			roll1 == t.getRoll1() &&
			roll2 == t.getRoll2() &&
			length == t.getLength() &&
			width == t.getWidth() &&
			friction == t.getFriction() &&
			tapDistance == t.getTap() &&
			damage == t.getDamage() &&
			quality == t.getQuality() &&
			enviroment.equals( t.getEnviroment() ) &&
			visibility == t.isVisible() &&
			texture.equals( t.getTexture() ) &&
			textureLength == t.getTextureLength() &&
			secondTexture.equals( t.getSecondTexture() ) &&
			texWidth == t.getTextureWidth() &&
			texHeight == t.getTextureHeight() &&
			texSlope == t.getTextureSlope() &&
			radius == t.getRadius();

		return b;
	}

	/*
	 * --------------- Naming ----------------
	 */

	/**
	 * Sets the name for the track.
	 *
	 * @param name The name for the track.
	 */
	public void setName(String name)
	{
		this.name = name;
	}

	/**
	 * Gets the name for the track.
	 *
	 * @return The name for track.
	 */
	public String getName()
	{
		return name;
	}

	/*
	 * --------------- Geometry -----------------
	 */
	
	/**
	 * Gets the 1<sup>st</sup> point of the Bezier curve.
	 */
	public Point3d getPoint1()
	{
		return node1;
	}

	/**
	 * Sets the 1<sup>st</sup> point of the Bezier curve.
	 */
	public void setPoint1(Point3d p)
	{
		node1 = p;
	}

	/**
	 * Sets the 1<sup>st</sup> point of the Bezier curve.
	 */
	public void setPoint1(double x, double y, double z)
	{
		node1.set(x, y, z);
	}

	/**
	 * Sets the 1<sup>st</sup> point of the Bezier curve.
	 */
	public void setPoint1(double[] array)
	{
		node1.set(array);
	}

	/**
	 * Gets the 2<sup>nd</sup> point of the Bezier curve.
	 */
	public Point3d getPoint2()
	{
		return node2;
	}

	/**
	 * Sets the 2<sup>nd</sup> point of the Bezier curve.
	 */
	public void setPoint2(Point3d p)
	{
		node2 = p;
	}

	/**
	 * Sets the 2<sup>nd</sup> point of the Bezier curve.
	 */
	public void setPoint2(double x, double y, double z)
	{
		node2.set(x, y, z);
	}

	/**
	 * Sets the 2<sup>nd</sup> point of the Bezier curve.
	 */
	public void setPoint2(double[] array)
	{
		node2.set(array);
	}

	/**
	 * Gets the 1<sup>st</sup> control vector of the Bezier curve.
	 */
	public Vector3d getVector1()
	{
		return vec1;
	}

	/**
	 * Sets the 1<sup>st</sup> control vector of the Bezier curve.
	 */
	public void setVector1(Vector3d v)
	{
		vec1 = v;
	}

	/**
	 * Sets the 1<sup>st</sup> control vector of the Bezier curve.
	 */
	public void setVector1(double x, double y, double z)
	{
		vec1.set(x, y, z);
	}

	/**
	 * Sets the 1<sup>st</sup> control vector of the Bezier curve.
	 */
	public void setVector1(double[] array)
	{
		vec1.set(array);
	}

	/**
	 * Gets the 2<sup>nd</sup> control vector of the Bezier curve.
	 */
	public Vector3d getVector2()
	{
		return vec2;
	}

	/**
	 * Sets the 2<sup>nd</sup> control vector of the Bezier curve.
	 */
	public void setVector2(Vector3d v)
	{
		vec2 = v;
	}

	/**
	 * Sets the 2<sup>nd</sup> control vector of the Bezier curve.
	 */
	public void setVector2(double x, double y, double z)
	{
		vec2.set(x, y, z);
	}

	/**
	 * Sets the 2<sup>nd</sup> control vector of the Bezier curve.
	 */
	public void setVector2(double[] array)
	{
		vec2.set(array);
	}

	/**
	 * Gets the roll at the 1<sup>st</sup> point.
	 */
	public double getRoll1()
	{
		return roll1;
	}

	/**
	 * Sets the roll at the 1<sup>st</sup> point.
	 */
	public void setRoll1(double roll)
	{
		roll1 = roll;
	}

	/**
	 * Gets the roll at the 2<sup>nd</sup> point.
	 */
	public double getRoll2()
	{
		return roll2;
	}

	/**
	 * Sets the roll at the 2<sup>nd</sup> point.
	 */
	public void setRoll2(double roll)
	{
		roll2 = roll;
	}

	/*
	 * ---------- Main characteristic ---------- 
	 */
	
	/**
	 * Gets the track length.
	 */
	public double getLength()
	{
		return length;
	}

	/**
	 * Sets the track length.
	 */
	public void setLength(double d)
	{
		length = d;
	}

	/**
	 * Gets track width.
	 */
	public double getWidth()
	{
		return width;
	}

	/**
	 * Sets track width.
	 */
	public void setWidth(double d)
	{
		width = d;
	}

	/**
	 * Gets track friction.
	 */
	public double getFriction()
	{
		return friction;
	}

	/**
	 * Sets track friction.
	 */
	public void setFriction(double d)
	{
		friction = d;
	}

	/**
	 * Gets track tapping.
	 */
	public double getTap()
	{
		return tapDistance;
	}

	/**
	 * Sets track tapping.
	 */
	public void setTap(double d)
	{
		tapDistance = d;
	}

	/**
	 * Gets quality flags.
	 */
	public int getQuality()
	{
		return quality;
	}

	/**
	 * Sets quality flags.
	 */
	public void setQuality(int i)
	{
		quality = i;
	}

	/**
	 * Gets damage track status.
	 */
	public int getDamage()
	{
		return damage;
	}

	/**
	 * Sets damage status.
	 */
	public void setDamage(int i)
	{
		damage = i;
	}

	/**
	 * Gets track enviroment.
	 */
	public String getEnviroment()
	{
		return enviroment;
	}
	
	/**
	 * Sets enviroment.
	 */
	public void setEnviroment(String enviroment)
	{

		this.enviroment = enviroment.toLowerCase();
	}

	/**
	 * Returns visibility flag of this track.
	 */
	public boolean isVisible()
	{
		return visibility;
	}

	/**
	 * Sets visibility flag.
	 */
	public void setVisible(boolean b)
	{
		visibility = b;
	}

	/**
	 * Gets track radius.
	 *
	 * @return <code>0</code>, when track is flex or straight.
	 */
	public double getRadius()
	{
		return radius;
	}

	/**
	 * Sets track radius.
	 */
	public void setRadius(double d)
	{
		radius = d;
	}

	/*
	 * -------------- Textures ----------------
	 */
	
	/**
	 * Gets track texture name.
	 */
	public String getTexture()
	{
		return texture;
	}

	/**
	 * Sets track texture.
	 *
	 * @param name Name of the texture.
	 */
	public void setTexture(String name)
	{
		texture = name;
	}

	/**
	 * Gets texture length.
	 */
	public double getTextureLength()
	{
		return textureLength;
	}

	/**
	 * Sets texture length.
	 */
	public void setTextureLength(double d)
	{
		textureLength = d;
	}

	/**
	 * Gets second texture name
	 */
	public String getSecondTexture()
	{
		return secondTexture;
	}

	/**
	 * Sets second texture name.
	 */
	public void setSecondTexture(String name)
	{
		secondTexture = name;
	}

	/**
	 * Gets texture height.
	 */
	public double getTextureHeight()
	{
		return texHeight;
	}

	/**
	 * Sets texture height.
	 */
	public void setTextureHeight(double d)
	{
		texHeight = d;
	}

	/**
	 * Gets texture width.
	 */
	public double getTextureWidth()
	{
		return texWidth;
	}

	/**
	 * Sets texture width.
	 */
	public void setTextureWidth(double d)
	{
		texWidth = d;
	}

	/**
	 * Gets texture slope.
	 */
	public double getTextureSlope()
	{
		return texSlope;
	}

	/**
	 * Sets texture slope.
	 */
	public void setTextureSlope(double d)
	{
		texSlope = d;
	}

	/*
	 * --------- Misc methods ----------
	 */
	
	/**
	 * Returns true if the track is straight.
	 */
	public boolean isStraight()
	{
		return (vec1.equals(VECTOR_ZERO) && vec2.equals(VECTOR_ZERO));			
	}
}
