/*
 * jET - An alternative scenery editor for the electic locomotive simulator, MaSzyna
 * Copyright (C) 2003 2004  Adam Domaski
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.jet.tracks;

import javax.vecmath.*;

/**
 * Tracks factory can build tracks and switches with specified 
 * properties.
 *
 * @author Adam Domaski
 * @version 0.1 10/03/2004
 */
public class TrackFactory
{
	/**
	 * Handler at left.
	 * In switches it is a common point of switch's tracks.
	 */
	public static final int HANDLER_LEFT = 1;

	/**
	 * Handler at right.
	 * In switches it is the end point of straight track.
	 */
	public static final int HANDLER_RIGHT = 2;

	/**
	 * Handler at side track of the switch.
	 */
	public static final int HANDLER_SIDE = 3;

	/**
	 * Dummy constructor.
	 */
	private TrackFactory() {}

	/**
	 * Gets instance of the track factory.
	 */
	public static TrackFactory getInstance()
	{
		if (anInstance == null)
			anInstance = new TrackFactory();

		return anInstance;
	}

	/**
	 * Creates straight track.
	 * No matter which handler was selected.
	 * <p>It fills in only geometry fields of the track. Programmer must
	 * fill in rest attributes on his own.</p>
	 *
	 * @param len Length of this track.
	 */
	public Track createStraight(double len)
	{
		if (len < 0.0)
			throw new IllegalArgumentException();

		Track track = new Track();
		track.setLength(len);

		Point3d node1 = new Point3d();
		track.setPoint1(node1);

		Point3d node2 = new Point3d();
		node2.z = len;
		track.setPoint2(node2);

		Vector3d vec1 = new Vector3d();
		track.setVector1(vec1);

		Vector3d vec2 = new Vector3d();
		track.setVector2(vec2);

		return track;
	}

	/**
	 * Creates new arc track.
	 *
	 * @param len Length of the track.
	 * @param rad Radius of the track.
	 *    Positive means right arc. Negative means left arc.
	 */
	public Track createArc(double len, double rad)
	{
		if (len < 0.0 || rad == 0.0)
			throw new IllegalArgumentException();

		Track track = new Track();
		track.setLength(len);
		track.setRadius(rad);

		boolean isRight = (rad >= 0.0);
		rad = Math.abs(rad);

		double angle = len / rad;
		double angleP = angle / 2;

		Point3d p1 = new Point3d();
		p1.x = - rad * Math.cos(angleP);
		p1.z = rad * Math.sin(angleP);

		Point3d p2 = new Point3d();
		p2.x = - rad * Math.cos(angleP);
		p2.z = - rad * Math.sin(angleP);

		double ctrlX = - rad * (4.0 - Math.cos(angleP)) / 3.0;
		double ctrlY = - rad * (1.0 - Math.cos(angleP)) * (Math.cos(angleP) - 3.0)
			/ (3.0 * Math.sin(angleP));

		Vector3d v1 = new Vector3d();
		v1.x = ctrlX - p1.x;
		v1.z = ctrlY - p1.z;

		Vector3d v2 = new Vector3d();
		v2.x = ctrlX - p2.x;
		v2.z = - ctrlY - p2.z;

		// Dokonaj transformacji
		Matrix4d rotMatrix = new Matrix4d();
		rotMatrix.rotY( isRight ? -angleP: angleP );
		rotMatrix.setTranslation(new Vector3d(rad, 0, 0));

		rotMatrix.transform(p1);
		rotMatrix.transform(v1);
		rotMatrix.transform(v2);
		rotMatrix.transform(p2);

		if (isRight)
		{
			rotMatrix = new Matrix4d();
			rotMatrix.rotY(Math.PI);

			rotMatrix.transform(p1);
			rotMatrix.transform(v1);
			rotMatrix.transform(v2);
			rotMatrix.transform(p2);

			track.setPoint1(p1);
			track.setPoint2(p2);
			track.setVector1(v1);
			track.setVector2(v2);
		}
		else
		{
			track.setPoint1(p2);
			track.setPoint2(p1);
			track.setVector1(v2);
			track.setVector2(v1);
		}

		return track;
	}

	/**
	 * Creates switch.
	 *
	 * @param handler Handler. One of the predefined values.
	 * @param len1 Length of the straight track.
	 * @param rad1 Radius of the straight track. When it equals 0, it means that
	 *    track is straight. Positive values mean right arc of the main track of this switch.
	 *    Negative - left arc.
	 * @param len2 Length of the side track of the switch.
	 * @param rad2 Radius of the side track of the switch. When it is positive track is right 
	 *    arc and also it means that switch is right. Negative means left arc and left switch.
	 * @return Switch with handler in point (0, 0, 0).
	 * @see #HANDLER_LEFT
	 * @see #HANDLER_RIGHT
	 * @see #HANDLER_SIDE
	 */
	public Switch createSwitch(int handler, double len1, double rad1, 
		double len2, double rad2)
	{
		if (handler != HANDLER_LEFT && handler != HANDLER_RIGHT &&
			handler != HANDLER_SIDE)
			throw new IllegalArgumentException();

		Switch sswitch = new Switch();
		sswitch.setRadius(rad1);
		sswitch.setLength(len1);
		sswitch.setSwitchedRadius(rad2);

		boolean sideIsRight = (rad2 > 0.0);
		int strTrack = getArcType(rad1);

		// Tor zwrotny
		rad2 = Math.abs(rad2);

		double angle = len2 / rad2;
		double angleP = angle / 2;

		Point3d ps1 = new Point3d();
		ps1.x = - rad2 * Math.cos(angleP);
		ps1.z = rad2 * Math.sin(angleP);

		Point3d ps2 = new Point3d();
		ps2.x = - rad2 * Math.cos(angleP);
		ps2.z = - rad2 * Math.sin(angleP);

		double ctrlsX = - rad2 * (4.0 - Math.cos(angleP)) / 3.0;
		double ctrlsY = - rad2 * (1.0 - Math.cos(angleP)) * (Math.cos(angleP) - 3.0)
			/ (3.0 * Math.sin(angleP));

		Vector3d vs1 = new Vector3d();
		vs1.x = ctrlsX - ps1.x;
		vs1.z = ctrlsY - ps1.z;

		Vector3d vs2 = new Vector3d();
		vs2.x = ctrlsX - ps2.x;
		vs2.z = - ctrlsY - ps2.z;

		// Dokonaj transformacji
		Matrix4d rotMatrix = new Matrix4d();
		rotMatrix.rotY( sideIsRight ? -angleP: angleP );
		rotMatrix.setTranslation(new Vector3d(rad2, 0, 0));

		rotMatrix.transform(ps1);
		rotMatrix.transform(vs1);
		rotMatrix.transform(vs2);
		rotMatrix.transform(ps2);

		if (sideIsRight)
		{
			rotMatrix = new Matrix4d();
			rotMatrix.rotY(Math.PI);

			rotMatrix.transform(ps1);
			rotMatrix.transform(vs1);
			rotMatrix.transform(vs2);
			rotMatrix.transform(ps2);
		}
		else
		{
			Point3d robP;
			Vector3d robV;

			robP = ps1;
			ps1 = ps2;
			ps2 = robP;

			robV = vs1;
			vs1 = vs2;
			vs2 = robV;
		}

		// Tor prosty
		Point3d pp1 = new Point3d();
		Point3d pp2 = new Point3d();
		Vector3d vp1 = new Vector3d();
		Vector3d vp2 = new Vector3d();

		if (strTrack == 0)
		{
			pp2.z = len1;
		}
		else
		{
			rad1 = Math.abs(rad1);
			angle = len1 / rad1;
			angleP = angle / 2;

			pp1.x = - rad1 * Math.cos(angleP);
			pp1.z = rad1 * Math.sin(angleP);

			pp2.x = - rad1 * Math.cos(angleP);
			pp2.z = - rad1 * Math.sin(angleP);

			double ctrlpX = - rad1 * (4.0 - Math.cos(angleP)) / 3.0;
			double ctrlpY = - rad1 * (1.0 - Math.cos(angleP)) * (Math.cos(angleP) - 3.0)
			/ (3.0 * Math.sin(angleP));

			vp1.x = ctrlpX - pp1.x;
			vp1.z = ctrlpY - pp1.z;

			vp2.x = ctrlpX - pp2.x;
			vp2.z = - ctrlpY - pp2.z;

			// Transformacja do rodka u.w.
			rotMatrix = new Matrix4d();
			rotMatrix.rotY( strTrack > 0? -angleP: angleP );
			rotMatrix.setTranslation(new Vector3d(rad1, 0, 0));

			rotMatrix.transform(pp1);
			rotMatrix.transform(vp1);
			rotMatrix.transform(vp2);
			rotMatrix.transform(pp2);

			if (strTrack > 0)
			{
			rotMatrix = new Matrix4d();
			rotMatrix.rotY(Math.PI);

			rotMatrix.transform(pp1);
			rotMatrix.transform(vp1);
			rotMatrix.transform(vp2);
			rotMatrix.transform(pp2);
			}
			else
			{
				Point3d robP;
				Vector3d robV;

				robP = pp1;
				pp1 = pp2;
				pp2 = robP;

				robV = vp1;
				vp1 = vp2;
				vp2 = robV;
			}

		}

		sswitch.setPoint1(pp1);
		sswitch.setVector1(vp1);
		sswitch.setVector2(vp2);
		sswitch.setPoint2(pp2);

		sswitch.setPoint3(ps2);
		sswitch.setVector3(vs2);
		sswitch.setSwitchedVector(vs1);

		// Obroty dla uchwytw
		if (handler == HANDLER_RIGHT)
		{
			double ang;
			Point3d p = sswitch.getPoint2();

			// Przesu do punktu (0, 0, 0)
			Matrix4d handMatrix = new Matrix4d();

			if (sswitch.isStraight())
				ang = Math.PI;
			else
			{
				Vector3d v = sswitch.getVector2();
				// Obr
				ang = Math.atan2(-v.x, v.z);
			}

			handMatrix.rotY(ang);
			handMatrix.setTranslation( new Vector3d(p) );

			handMatrix.transform( sswitch.getPoint1() );
			handMatrix.transform( sswitch.getPoint2() );
			handMatrix.transform( sswitch.getPoint3() );
			handMatrix.transform( sswitch.getVector1() );
			handMatrix.transform( sswitch.getVector2() );
			handMatrix.transform( sswitch.getVector3() );
			handMatrix.transform( sswitch.getSwitchedVector() );
		}
		else if (handler == HANDLER_SIDE)
		{
			double ang;
			Point3d p = sswitch.getPoint3();
			Vector3d v = sswitch.getVector3();

			// Przesu do punktu (0, 0, 0)
			Matrix4d handMatrix = new Matrix4d();
			handMatrix.setIdentity();
			Vector3d trV = new Vector3d();
			trV.negate(p);

			ang = Math.atan2(-v.x, v.z);

			// Przenie
			handMatrix.setTranslation( trV );
			handMatrix.transform( sswitch.getPoint1() );
			handMatrix.transform( sswitch.getPoint2() );
			handMatrix.transform( sswitch.getPoint3() );
			handMatrix.transform( sswitch.getVector1() );
			handMatrix.transform( sswitch.getVector2() );
			handMatrix.transform( sswitch.getVector3() );
			handMatrix.transform( sswitch.getSwitchedVector() );

			// Obr
			handMatrix.setZero();
			handMatrix.rotY(ang);

			handMatrix.transform( sswitch.getPoint1() );
			handMatrix.transform( sswitch.getPoint2() );
			handMatrix.transform( sswitch.getPoint3() );
			handMatrix.transform( sswitch.getVector1() );
			handMatrix.transform( sswitch.getVector2() );
			handMatrix.transform( sswitch.getVector3() );
			handMatrix.transform( sswitch.getSwitchedVector() );
		}

		return sswitch;
	}

	/**
	 * Returns type of the arc.
	 *
	 * @return 0 when the radius is 0.0 and it means that track is not
	 *  an arc. -1 When arc is left. 1 when arc is right.
	 */
	private int getArcType(double r)
	{
		if (Math.abs(r) == 0.0)
			return 0;
		else if (r < -0.0)
			return -1;
		else if (r > 0.0)
			return 1;

		return 0;
	}
	
	/*
	 * -------- Members ---------
	 */
	
	/** An instance if this class. */
	private static TrackFactory anInstance = null;
}
