/*
 * jET - An alternative scenery editor for the electic locomotive simulator, MaSzyna
 * Copyright (C) 2003 2004  Adam Domaski
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.jet.tracks;

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.Point2D;
import java.util.*;
import javax.swing.*;
import javax.swing.event.InternalFrameAdapter;
import javax.swing.event.InternalFrameEvent;
import javax.swing.table.*;
import javax.vecmath.*;

import org.jet.*;
import org.jet.sce.*;

/**
 * Track palette.
 *
 * @author Adam Domaski
 * @version 0.1 11/03/2004
 */
public class TrackPalette implements Palette
{
	public TrackPalette(Jet app)
	{
		resource = JetResources.getInstance();

		application = app;

		buttonMap = new HashMap();
		buttonGroup = new ButtonGroup();
		tableModel = new PaletteTableModel();

		createStandalone();

		// Load tracks from script file.
		ScriptLoader scriptLoader = ScriptLoader.getInstance();
		scriptLoader.load();
	}

	private void createStandalone()
	{
		JToggleButton jToggle;

		// Prosta
		jToggle = new JToggleButton();
		jToggle.setText( resource.translate("TrackPalette.other") );
		jToggle.setToolTipText( resource.translate("TrackPalette.straight") );
		jToggle.setMargin(new Insets(3, 3, 3, 3));
		jToggle.addActionListener(new StraightAction());
		putButton(jToggle, 0, 0);
		buttonGroup.add(jToggle);

		// uk
		jToggle = new JToggleButton();
		jToggle.setText( resource.translate("TrackPalette.other") );
		jToggle.setToolTipText( resource.translate("TrackPalette.arc") );
		jToggle.setMargin(new Insets(3, 3, 3, 3));
		jToggle.addActionListener(new ArcAction());
		putButton(jToggle, 1, 0);
		buttonGroup.add(jToggle);

		// Rozjazd
		jToggle = new JToggleButton();
		jToggle.setText( resource.translate("TrackPalette.switch") );
		jToggle.setToolTipText( resource.translate("TrackPalette.switch2") );
		jToggle.setMargin(new Insets(3, 3, 3, 3));
		jToggle.addActionListener(new SwitchAction());
		putButton(jToggle, 2, 0);
		buttonGroup.add(jToggle);

		// Guma
		jToggle = new JToggleButton();
		jToggle.setText( resource.translate("TrackPalette.flex") );
		jToggle.setToolTipText( resource.translate("TrackPalette.flex2") );
		jToggle.setMargin(new Insets(3, 3, 3, 3));
		jToggle.addItemListener(new FlexHandler());
		putButton(jToggle, 2, 1);
		buttonGroup.add(jToggle);
	}

	/*
	 * ------------ Buttons --------------
	 */
	
	/**
	 * Puts a toggle button to palette table at point (x, y).
	 */
	public void putButton(JToggleButton button, int x, int y)
	{
		Point p = new Point(x, y);
		buttonMap.put(p, button);
	}

	/**
	 * Gets toggle button pointed to (x, y).
	 */
	public JToggleButton getButton(int x, int y)
	{
		Point p = new Point(x, y);
		if (!buttonMap.containsKey(p))
			return new BlankButton();
		return (JToggleButton) buttonMap.get(p);
	}

	/*
	 * ---------- Palette interface ----------
	 */

	/**
	 * Gets palette name.
	 */
	public String getName()
	{
		return resource.translate( "TrackPalette.name" );
	}

	/**
	 * Gets table model for palette table component.
	 */
	public TableModel getTableModel()
	{
		return tableModel;
	}

	/*
	 * --------- Inner classes ------------
	 */
	
	/**
	 * Palette table model for track palette.
	 */
	public class PaletteTableModel extends AbstractTableModel
	{
		/**
		 * Returns row count.
		 */
		public int getRowCount()
		{
			int max = 0;

			Iterator i = buttonMap.keySet().iterator();
			while (i.hasNext())
			{
				Point p = (Point) i.next();
				max = Math.max(max, p.y);
			}

			return max+1;
		}

		/**
		 * Table has 3 columns.
		 */
		public int getColumnCount()
		{
			return 3;
		}

		/**
		 * Gets button pointed by (col, row).
		 */
		public Object getValueAt(int row, int col)
		{
			return getButton(col, row);
		}

		/**
		 * Only enabled buttons are editable.
		 */
		public boolean isCellEditable(int row, int col)
		{
			JToggleButton b = getButton(col, row);
			if (!b.isEnabled())
				return false;

			return true;
		}

		/**
		 * Returns <code>JToggleButton.class</code>.
		 */
		public Class getColumnClass(int col)
		{
			return JToggleButton.class;
		}
	}

	/*
	 * --------- Handlers ----------
	 */
	
	class StraightAction implements ActionListener
	{
		public void actionPerformed(ActionEvent ae)
		{
			new CustomStraightProperties(application);
		}
	}

	class ArcAction implements ActionListener
	{
		public void actionPerformed(ActionEvent ae)
		{
			new CustomArcProperties(application);
		}
	}

	class SwitchAction implements ActionListener
	{
		public void actionPerformed(ActionEvent ae)
		{
			new CustomSwitchProperties(application);
		}
	}

	class FlexHandler extends InternalFrameAdapter
		implements ItemListener, ContainerListener
	{
		public FlexHandler()
		{
			mouseHandler = new FlexMouseHandler();
		}

		public void itemStateChanged(ItemEvent ie)
		{
			int state = ie.getStateChange();
			if (state == ItemEvent.SELECTED)
			{
				application.getDesktopPane().addContainerListener(this);

				// Ustaw dla wszystkich okien edytorw nasuch
				Iterator itr = application.getEditorFrames().iterator();
				while (itr.hasNext())
				{
					JInternalFrame jFrame = (JInternalFrame) itr.next();
					jFrame.addInternalFrameListener(this);
				}

				SceneryFrame sceneryFrame = application.getTopmostFrame();
				if (sceneryFrame != null)
				{
					JSceneryEditor jEditor = sceneryFrame.getSceneryEditor();
					jEditor.addMouseListener(mouseHandler);
				}
			}
			else if (state == ItemEvent.DESELECTED)
			{
				application.getDesktopPane().removeContainerListener(this);

				// Ustaw dla wszystkich okien edytorw nasuch
				Iterator itr = application.getEditorFrames().iterator();
				while (itr.hasNext())
				{
					JInternalFrame jFrame = (JInternalFrame) itr.next();
					jFrame.removeInternalFrameListener(this);
				}

				SceneryFrame sceneryFrame = application.getTopmostFrame();
				if (sceneryFrame != null)
				{
					JSceneryEditor jEditor = sceneryFrame.getSceneryEditor();
					jEditor.removeMouseListener(mouseHandler);
				}
			}
		}

		public void internalFrameActivated(InternalFrameEvent ife)
		{
			JInternalFrame jFrame = ife.getInternalFrame();
			if (jFrame instanceof SceneryFrame)
			{
				JSceneryEditor jEditor = ((SceneryFrame) jFrame).getSceneryEditor();
				jEditor.addMouseListener(mouseHandler);
			}
		}

		public void internalFrameDeactivated(InternalFrameEvent ife)
		{
			JInternalFrame jFrame = ife.getInternalFrame();
			if (jFrame instanceof SceneryFrame)
			{
				JSceneryEditor jEditor = ((SceneryFrame) jFrame).getSceneryEditor();
				jEditor.removeMouseListener(mouseHandler);
			}
		}

		public void componentAdded(ContainerEvent ce)
		{
			Component aComp = ce.getChild();
			if (aComp instanceof JInternalFrame)
			{
				JInternalFrame jFrame = (JInternalFrame) aComp;
				jFrame.addInternalFrameListener(this);
			}
		}

		public void componentRemoved(ContainerEvent ce)
		{
			Component aComp = ce.getChild();
			if (aComp instanceof JInternalFrame)
			{
				JInternalFrame jFrame = (JInternalFrame) aComp;
				jFrame.removeInternalFrameListener(this);
			}
		}

		private FlexMouseHandler mouseHandler = null;
	}

	/**
	 * Mouse listener that listens to the topmost editor frame when
	 * flex button is selected. It adds to the editor scenery
	 * flex track.
	 */
	/*
	 * 1) Sprawd czy jest wogle jakiekolwiek zaznaczenie w edytorze.
	 * 2) Jeeli jest, to sprawd czy trafilimy w uchwyt.
	 * 3) Jeeli tak, to zapamitaj punkt i wektor.
	 * 4) Jeeli zosta zapamitany wczeniej punkt i wektor
	 */
	private class FlexMouseHandler extends MouseAdapter
	{
		public void mouseClicked(MouseEvent me)
		{
			Point targetPoint = me.getPoint();
			Object obj = me.getSource();

			if (obj instanceof JSceneryEditor)
			{
				JSceneryEditor jEditor = (JSceneryEditor) obj;

				// Sprawd zaznaczenie
				if (!jEditor.isSelectionEmpty())
				{
					Object selectedObject = jEditor.getSelectedObject();
					if (selectedObject instanceof Switch)
					{
						Switch sswitch = (Switch) selectedObject;

						// sprawd czy trafilimy w uchwyt
						if (checkPoint( jEditor.modelToView(sswitch.getPoint1()), targetPoint ))
						{
							if (!isOld())
								setOld(selectedObject, sswitch.getPoint1(), sswitch.getVector1());
							else
								compute(jEditor, selectedObject, sswitch.getPoint1(), sswitch.getVector1());
						}
						else if (checkPoint( jEditor.modelToView(sswitch.getPoint2()), targetPoint ))
						{
							if (!isOld())
								setOld(selectedObject, sswitch.getPoint2(), sswitch.getVector2());
							else
								compute(jEditor, selectedObject, sswitch.getPoint2(), sswitch.getVector2());
						}
						else if (checkPoint(  jEditor.modelToView(sswitch.getPoint3()), targetPoint ))
						{
							if (!isOld())
								setOld(selectedObject, sswitch.getPoint3(), sswitch.getVector3());
							else
								compute(jEditor, selectedObject, sswitch.getPoint3(), sswitch.getVector3());
						}
						else
						{
							//clear();
						}
					}
					else if (selectedObject instanceof Track)
					{
						Track track = (Track) selectedObject;

						if (checkPoint( jEditor.modelToView(track.getPoint1()), targetPoint ))
						{
							if (!isOld())
								setOld(selectedObject, track.getPoint1(), track.getVector1());
							else
								compute(jEditor, selectedObject, track.getPoint1(), track.getVector1());
						}
						else if (checkPoint( jEditor.modelToView(track.getPoint2()), targetPoint ))
						{
							if (!isOld())
								setOld(selectedObject, track.getPoint2(), track.getVector2());
							else
								compute(jEditor, selectedObject, track.getPoint2(), track.getVector2());
						}
						else
						{
							//clear();
						}
					}
				}
				else
				{
					//clear();
				}
			}
		}

		private boolean checkPoint(Point2D p, Point target)
		{
			Rectangle rect = new Rectangle((int) p.getX(), (int) p.getY(), 0, 0);
			rect.grow(4, 4);
			return rect.contains(target);
		}

		private void setOld(Object selection, Point3d p, Vector3d v)
		{
			oldSelection = selection;
			oldPoint = p;
			oldVector = v;
		}

		// TODO:
		// Brak poprawnego algorytmu obliczania toru flex
		private void compute(JSceneryEditor jEditor, Object selection, Point3d p, Vector3d v)
		{
			if (selection != oldSelection)
			{
				Track track = new Track();

				Vector3d v1 = new Vector3d(oldVector);
				v1.negate();
				if (v1.x == -0.0)
					v1.x = 0.0;
				if (v1.y == -0.0)
					v1.y = 0.0;
				if (v1.z == -0.0)
					v1.z = 0.0;

				// if
				if (oldSelection instanceof Track)
				{
					Track strTrack = (Track) oldSelection;
					if (strTrack.isStraight())
					{
						Point3d pTemp;

						if (strTrack.getPoint1().equals(oldPoint))
						{
							pTemp = strTrack.getPoint2();
						}
						else
						{
							pTemp = strTrack.getPoint1();
						}

						v1.x = 0.1 * (oldPoint.x - pTemp.x);
						v1.y = 0.1 * (oldPoint.y - pTemp.y);
						v1.z = 0.1 * (oldPoint.z - pTemp.z);						
					}
				}

				Vector3d v2 = new Vector3d(v);
				v2.negate();
				if (v2.x == -0.0)
					v2.x = 0.0;
				if (v2.y == -0.0)
					v2.y = 0.0;
				if (v2.z == -0.0)
					v2.z = 0.0;

				if (selection instanceof Track)
				{
					Track strTrack = (Track) selection;
					if (strTrack.isStraight())
					{
						Point3d pTemp;

						if (strTrack.getPoint1().equals(p))
						{
							pTemp = strTrack.getPoint2();
						}
						else
						{
							pTemp = strTrack.getPoint1();
						}

						v2.x = 0.1 * (p.x - pTemp.x);
						v2.y = 0.1 * (p.y - pTemp.y);
						v2.z = 0.1 * (p.z - pTemp.z);						
					}
				}

				track.setPoint1( new Point3d(oldPoint) );
				track.setPoint2( new Point3d(p) );

				track.setVector1( v1 );
				track.setVector2( v2 );

				track.setLength(0.0);
				track.setRadius(0.0);
				track.setWidth(1.435);
				track.setTap(20.0);
				track.setFriction(0.25);
				track.setQuality(20);
				track.setDamage(0);
				track.setEnviroment( Track.FLAT );
				track.setVisible(true);
				track.setTexture( Track.TEXTURE_DEFAULT );
				track.setTextureLength(4.0);
				track.setSecondTexture( Track.SECOND_TEXTURE_DEFAULT );
				track.setTextureWidth(0.5);
				track.setTextureHeight(0.2);
				track.setTextureSlope(1.1);
				track.setRoll1(0.0);
				track.setRoll2(0.0);

				Scenery scenery = jEditor.getScenery();

				Object[] aObj = new Object[1];
				aObj[0] = track;
				scenery.insert(aObj);

				jEditor.setSelectedObject(track);
			}

			clear();
		}

		public void clear()
		{
			oldPoint = null;
			oldVector = null;
			oldSelection = null;
		}

		private boolean isOld()
		{
			return (oldPoint != null && oldVector != null);
		}

		private Point3d oldPoint = null;
		private Vector3d oldVector = null;
		private Object oldSelection = null;
	}

	/**
	 * Blank inactive toggle button.
	 */
	class BlankButton extends JToggleButton
	{
		public BlankButton()
		{
			super();
			setEnabled(false);
		}
	}

	/*
	 * --------- Members ----------
	 */
	
	/** I18n resources. */
	private JetResources resource;

	/** Map containing buttons */
	private Map buttonMap;

	/** Button group. */
	private ButtonGroup buttonGroup;

	/** Table model. */
	private PaletteTableModel tableModel;

	/** Application. */
	private Jet application;
}
