/*
 * jET - An alternative scenery editor for the electic locomotive simulator, MaSzyna
 * Copyright (C) 2003 2004  Adam Domaski
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.jet.tracks;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.text.DecimalFormat;
import javax.swing.text.*;
import prv.doman.util.HexadecimalFormat;
import prv.doman.util.DegreeFormat;
import javax.vecmath.*;

import org.jet.*;
import org.jet.sce.*;

/**
 * Generic track properties dialog.
 *
 * @author Adam Domaski
 * @version 0.1 16/02/2004
 */
public class TrackProperties extends JetDialog
{
	/**
	 * Constructs generic track properties dialog and shows it.
	 *
	 * @param frame Parent frame.
	 * @param editor Topmost scenery editor.
	 */
	public TrackProperties(Frame frame, JSceneryEditor editor)
	{
		super(frame);

		resource = JetResources.getInstance();
		setTitle( resource.translate("TrackProperties.title") );

		setResizable(false);
		setDefaultCloseOperation(HIDE_ON_CLOSE);

		editable = editor.isEditable();
		this.editor = editor;

		initialize();
		fillIn((Track) editor.getSelectedObject());

		pack();
		centerDialog();
		setVisible(true);
	}

	/**
	 * Constructs empty dialog box.
	 *
	 * @param frame Parent frame.
	 */
	protected TrackProperties(Frame frame)
	{
		super(frame);
	}

	/*
	 * ------------ Methods ------------
	 */

	/**
	 * Returns true if dialog allows to edit track.
	 */
	public boolean isDialogEditable()
	{
		return editable;
	}

	/**
	 * Initialize dialog controls.
	 */
	private void initialize()
	{
		JPanel pane = new JPanel();
		pane.setLayout(new BorderLayout());

		JPanel container = createContent();
		pane.add(container);

		JPanel controlPanel = new JPanel();		
		if (isDialogEditable())
		{
			GridBagLayout gb = new GridBagLayout();
			GridBagConstraints c = new GridBagConstraints();
			controlPanel.setLayout(gb);
			controlPanel.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));

			JButton okButton = new JButton( resource.translate("TrackProperties.ok") );
			JButton caButton = new JButton( resource.translate("TrackProperties.cancel") );

			caButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent ae)
				{
					dispose();
				}
			});

			getRootPane().setDefaultButton(okButton);
			okButton.addActionListener(new OKAction());

			c.gridx = 0;
			c.gridy = 0;
			c.weightx = 1.0;
			c.anchor = GridBagConstraints.EAST;
			c.insets = new Insets(0, 0, 0, 5);
			gb.setConstraints(okButton, c);
			controlPanel.add(okButton);

			c.gridx = 1;
			c.gridy = 0;
			c.weightx = 0.0;
			c.insets = new Insets(0, 0, 0, 0);
			gb.setConstraints(caButton, c);
			controlPanel.add(caButton);

			normalize(okButton, caButton);
			okButton.requestFocus();
		}
		else
		{
			JButton btn = new JButton( resource.translate("TrackProperties.close") );
			btn.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent ae)
				{
					dispose();
				}
			});
			getRootPane().setDefaultButton(btn);
			controlPanel.add(btn);
		}
		pane.add(controlPanel, BorderLayout.SOUTH);

		getContentPane().add(pane);
	}

	/**
	 * Creates main content.
	 */
	private JPanel createContent()
	{
		JPanel jPanel;
		JLabel jLabel;
		JTextField jText;

		JPanel pane = new JPanel();
		pane.setName("TrackPropertiesPanel");
		pane.setBorder( BorderFactory.createEmptyBorder(1, 1, 1, 1) );
		
		JTabbedPane jTabbedPane = new JTabbedPane();
		jTabbedPane.setName("TrackPropertiesTabbedPane");

		JPanel jGeneralPanel = new JPanel();
		jGeneralPanel.setName("GeneralPanel");
		jGeneralPanel.setBorder( BorderFactory.createEmptyBorder(3, 3, 3, 3) );
		jGeneralPanel.setLayout(new GridLayout(2, 1, 0, 5));

		jPanel = new JPanel();
		GridBagLayout gb = new GridBagLayout();
		GridBagConstraints c = new GridBagConstraints();
		Insets insets = new Insets(1, 2, 1, 2);
		jPanel.setLayout(gb);
		jPanel.setBorder( BorderFactory.createCompoundBorder(
			BorderFactory.createEtchedBorder(),
			BorderFactory.createEmptyBorder(3, 3, 3, 3)) );
		
		jLabel = new JLabel();
		jLabel.setName("NameLabel");
		jLabel.setText( resource.translate("TrackProperties.name") );
		c.gridx = 0;
		c.gridy = 0;
		c.anchor = GridBagConstraints.WEST;
		c.insets = insets;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createName();
		jText.setName("NameTextBox");
		jText.setColumns(10);
		jText.setEditable( editable );
		controls.putComponent(jText);
		c.gridx = 1;
		c.gridy = 0;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jLabel = new JLabel();
		jLabel.setName("LengthLabel");
		jLabel.setText( resource.translate("TrackProperties.length") );
		c.gridx = 0;
		c.gridy = 1;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createLength();
		jText.setName("LengthTextBox");
		jText.setEditable( editable );
		jText.setColumns(7);
		controls.putComponent(jText);
		c.gridx = 1;
		c.gridy = 1;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jLabel = new JLabel();
		jLabel.setName("RadiusLabel");
		jLabel.setText( resource.translate("TrackProperties.radius") );
		c.gridx = 0;
		c.gridy = 2;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createRadius();
		jText.setName("RadiusTextBox");
		jText.setEditable( editable );
		jText.setColumns(7);
		controls.putComponent(jText);
		c.gridx = 1;
		c.gridy = 2;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jLabel = new JLabel();
		jLabel.setName("WidthLabel");
		jLabel.setText( resource.translate("TrackProperties.width") );
		c.gridx = 0;
		c.gridy = 3;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createWidth();
		jText.setName("WidthTextBox");
		jText.setEditable( editable );
		jText.setColumns(7);
		controls.putComponent(jText);
		c.gridx = 1;
		c.gridy = 3;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jLabel = new JLabel();
		jLabel.setName("FrictionLabel");
		jLabel.setText( resource.translate("TrackProperties.friction") );
		c.gridx = 2;
		c.gridy = 0;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createFriction();
		jText.setName("FrictionTextBox");
		jText.setColumns(7);
		jText.setEditable( editable );
		controls.putComponent(jText);
		c.gridx = 3;
		c.gridy = 0;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jLabel = new JLabel();
		jLabel.setName("TapLabel");
		jLabel.setText( resource.translate("TrackProperties.tap") );
		c.gridx = 2;
		c.gridy = 1;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createTap();
		jText.setName("TapTextBox");
		jText.setColumns(7);
		jText.setEditable( editable );
		controls.putComponent(jText);
		c.gridx = 3;
		c.gridy = 1;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jLabel = new JLabel();
		jLabel.setName("QualityLabel");
		jLabel.setText( resource.translate("TrackProperties.quality") );
		c.gridx = 2;
		c.gridy = 2;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createQuality();
		jText.setName("QualityTextBox");
		jText.setColumns(7);
		jText.setEditable( editable );
		controls.putComponent(jText);
		c.gridx = 3;
		c.gridy = 2;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jLabel = new JLabel();
		jLabel.setName("DamageLabel");
		jLabel.setText( resource.translate("TrackProperties.damage") );
		c.gridx = 2;
		c.gridy = 3;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createDamage();
		jText.setName("DamageTextBox");
		jText.setColumns(7);
		jText.setEditable( editable );
		controls.putComponent(jText);
		c.gridx = 3;
		c.gridy = 3;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jPanel.setFocusTraversalPolicy(new GeneralTabFocusTraversal());
		jGeneralPanel.add(jPanel);

		jPanel = new JPanel();
		jPanel.setBorder( BorderFactory.createCompoundBorder(
			BorderFactory.createEtchedBorder(),
			BorderFactory.createEmptyBorder(3, 3, 3, 3)) );
		gb = new GridBagLayout();
		jPanel.setLayout(gb);

		jLabel = new JLabel();
		jLabel.setName("EnviromentLabel");
		jLabel.setText( resource.translate("TrackProperties.enviroment") );
		c.gridx = 0;
		c.gridy = 0;
		c.insets = insets;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		JComboBox jComboBox = new JComboBox();
		jComboBox.setName("EnviromentComboBox");
		jComboBox.addItem(Track.BANK);
		jComboBox.addItem(Track.BRIDGE);
		jComboBox.addItem(Track.CANYON);
		jComboBox.addItem(Track.FLAT);
		jComboBox.addItem(Track.MOUNTAINS);
		jComboBox.addItem(Track.TUNNEL);
		jComboBox.setEnabled( editable );
		controls.putComponent(jComboBox);
		c.gridx = 1;
		c.gridy = 0;
		gb.setConstraints(jComboBox, c);
		jPanel.add(jComboBox);

		jGeneralPanel.add(jPanel);

		jTabbedPane.addTab(resource.translate("TrackProperties.general"), jGeneralPanel);

		JPanel jTexturesPanel = new JPanel();
		jTexturesPanel.setName("TexturesPanel");
		jTexturesPanel.setBorder( BorderFactory.createEmptyBorder(3, 3, 3, 3) );
		jTexturesPanel.setLayout(new GridLayout(2, 1, 0, 5));

		/* Tekstury - grny panel */

		jPanel = new JPanel();
		jPanel.setBorder( BorderFactory.createCompoundBorder(
			BorderFactory.createEtchedBorder(),
			BorderFactory.createEmptyBorder(3, 3, 3, 3)) );
		gb = new GridBagLayout();
		jPanel.setLayout(gb);

		jLabel = new JLabel();
		jLabel.setName("VisibilityLabel");
		jLabel.setText( resource.translate("TrackProperties.visibility") );
		c.gridx = 0;
		c.gridy = 0;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		JPanel jVisibilityPanel = new JPanel();
		jVisibilityPanel.setBorder( BorderFactory.createEtchedBorder() );
		jVisibilityPanel.setLayout( new GridLayout(1, 2) );

		JRadioButton jRadioButton;
		ButtonGroup bGroup = new ButtonGroup();

		jRadioButton = new JRadioButton();
		jRadioButton.setName("VisibleRadioButton");
		jRadioButton.setText( resource.translate("TrackProperties.visible") );
		jRadioButton.setEnabled( editable );
		controls.putComponent(jRadioButton);
		bGroup.add(jRadioButton);
		jVisibilityPanel.add(jRadioButton);

		jRadioButton = new JRadioButton();
		jRadioButton.setName("InvisibleRadioButton");
		jRadioButton.setText( resource.translate("TrackProperties.invisible") );
		jRadioButton.setEnabled( editable );
		controls.putComponent(jRadioButton);
		bGroup.add(jRadioButton);
		jVisibilityPanel.add(jRadioButton);

		c.gridx = 1;
		c.gridy = 0;
		gb.setConstraints(jVisibilityPanel, c);
		jPanel.add(jVisibilityPanel);
		
		jTexturesPanel.add(jPanel);

		/* Tekstury - dolny panel */

		jPanel = new JPanel();
		jPanel.setBorder( BorderFactory.createCompoundBorder(
			BorderFactory.createEtchedBorder(),
			BorderFactory.createEmptyBorder(3, 3, 3, 3)) );
		gb = new GridBagLayout();
		jPanel.setLayout(gb);

		jLabel = new JLabel();
		jLabel.setName("TextureLabel");
		jLabel.setText( resource.translate("TrackProperties.texture") );
		c.gridx = 0;
		c.gridy = 0;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jComboBox = createTexture();
		jComboBox.setName("TextureComboBox");
		jComboBox.setEnabled( editable );
		controls.putComponent(jComboBox);
		c.gridx = 1;
		c.gridy = 0;
		gb.setConstraints(jComboBox, c);
		jPanel.add(jComboBox);

		jLabel = new JLabel();
		jLabel.setName("TextureLengthLabel");
		jLabel.setText( resource.translate("TrackProperties.textureLength") );
		c.gridx = 0;
		c.gridy = 1;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createTextureLength();
		jText.setName("TextureLengthEditBox");
		jText.setEditable( editable );
		jText.setColumns(7);
		controls.putComponent( jText );
		c.gridx = 1;
		c.gridy = 1;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jLabel = new JLabel();
		jLabel.setName("SecondTextureLabel");
		jLabel.setText( resource.translate("TrackProperties.secondTexture") );
		c.gridx = 0;
		c.gridy = 2;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jComboBox = createSecondTexture();
		jComboBox.setName("SecondTextureComboBox");
		jComboBox.setEnabled( editable );
		controls.putComponent( jComboBox );
		c.gridx = 1;
		c.gridy = 2;
		gb.setConstraints(jComboBox, c);
		jPanel.add(jComboBox);

		jLabel = new JLabel();
		jLabel.setName("TextureWidthLabel");
		jLabel.setText( resource.translate("TrackProperties.textureWidth") );
		c.gridx = 2;
		c.gridy = 0;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createTextureWidth();
		jText.setName("TextureWidthTextBox");
		jText.setEditable( editable );
		jText.setColumns(7);
		controls.putComponent( jText );
		c.gridx = 3;
		c.gridy = 0;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jLabel = new JLabel();
		jLabel.setName("TextureHeightLabel");
		jLabel.setText( resource.translate("TrackProperties.textureHeight") );
		c.gridx = 2;
		c.gridy = 1;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createTextureHeight();
		jText.setName("TextureHeightTextBox");
		jText.setEditable( editable );
		jText.setColumns(7);
		controls.putComponent( jText );
		c.gridx = 3;
		c.gridy = 1;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jLabel = new JLabel();
		jLabel.setName("TextureSlopeLabel");
		jLabel.setText( resource.translate("TrackProperties.textureSlope") );
		c.gridx = 2;
		c.gridy = 2;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createTextureSlope();
		jText.setName("TextureSlopeTextBox");
		jText.setEditable( editable );
		jText.setColumns(7);
		controls.putComponent( jText );
		c.gridx = 3;
		c.gridy = 2;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jTexturesPanel.add(jPanel);

		jTabbedPane.addTab(resource.translate("TrackProperties.textures"), jTexturesPanel);

		/* Geometria */
		JPanel jGeometryPanel = new JPanel();
		jGeometryPanel.setName("GeometryPanel");
		jGeometryPanel.setBorder( BorderFactory.createEmptyBorder(3, 3, 3, 3) );
		jGeometryPanel.setLayout(new GridLayout(3, 1, 0, 5));

		jPanel = new JPanel();
		jPanel.setBorder( BorderFactory.createCompoundBorder(
			BorderFactory.createEtchedBorder(),
			BorderFactory.createEmptyBorder(3, 3, 3, 3)) );
		gb = new GridBagLayout();
		jPanel.setLayout(gb);

		jLabel = new JLabel();
		jLabel.setName("Point1Label");
		jLabel.setText( resource.translate("TrackProperties.point1") );
		c.gridx = 0;
		c.gridy = 0;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createCoordinate();
		jText.setName("Point1XTextBox");
		jText.setColumns(6);
		jText.setEditable( editable );
		controls.putComponent(jText);
		c.gridx = 1;
		c.gridy = 0;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jText = createCoordinate();
		jText.setName("Point1YTextBox");
		jText.setColumns(6);
		jText.setEditable( editable );
		controls.putComponent(jText);
		c.gridx = 2;
		c.gridy = 0;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jText = createCoordinate();
		jText.setName("Point1ZTextBox");
		jText.setColumns(6);
		jText.setEditable( editable );
		controls.putComponent(jText);
		c.gridx = 3;
		c.gridy = 0;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jLabel = new JLabel();
		jLabel.setName("Vector1Label");
		jLabel.setText( resource.translate("TrackProperties.vector1") );
		c.gridx = 0;
		c.gridy = 1;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createCoordinate();
		jText.setName("Vector1XTextBox");
		jText.setColumns(6);
		jText.setEditable( editable );
		controls.putComponent(jText);
		c.gridx = 1;
		c.gridy = 1;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jText = createCoordinate();
		jText.setName("Vector1YTextBox");
		jText.setColumns(6);
		jText.setEditable( editable );
		controls.putComponent(jText);
		c.gridx = 2;
		c.gridy = 1;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jText = createCoordinate();
		jText.setName("Vector1ZTextBox");
		jText.setColumns(6);
		jText.setEditable( editable );
		controls.putComponent(jText);
		c.gridx = 3;
		c.gridy = 1;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jLabel = new JLabel();
		jLabel.setName("Roll1Label");
		jLabel.setText( resource.translate("TrackProperties.roll1") );
		c.gridx = 0;
		c.gridy = 2;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createRoll();
		jText.setName("Roll1TextBox");
		jText.setColumns(6);
		jText.setEditable( editable );
		controls.putComponent(jText);
		c.gridx = 1;
		c.gridy = 2;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jGeometryPanel.add(jPanel);

		jPanel = new JPanel();
		jPanel.setBorder( BorderFactory.createCompoundBorder(
			BorderFactory.createEtchedBorder(),
			BorderFactory.createEmptyBorder(3, 3, 3, 3)) );
		gb = new GridBagLayout();
		jPanel.setLayout(gb);

		jLabel = new JLabel();
		jLabel.setName("Vector2Label");
		jLabel.setText( resource.translate("TrackProperties.vector2") );
		c.gridx = 0;
		c.gridy = 0;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createCoordinate();
		jText.setName("Vector2XTextBox");
		jText.setColumns(6);
		jText.setEditable( editable );
		controls.putComponent(jText);
		c.gridx = 1;
		c.gridy = 0;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jText = createCoordinate();
		jText.setName("Vector2YTextBox");
		jText.setColumns(6);
		jText.setEditable( editable );
		controls.putComponent(jText);
		c.gridx = 2;
		c.gridy = 0;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jText = createCoordinate();
		jText.setName("Vector2ZTextBox");
		jText.setColumns(6);
		jText.setEditable( editable );
		controls.putComponent(jText);
		c.gridx = 3;
		c.gridy = 0;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jLabel = new JLabel();
		jLabel.setName("Point2Label");
		jLabel.setText( resource.translate("TrackProperties.point2") );
		c.gridx = 0;
		c.gridy = 1;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createCoordinate();
		jText.setName("Point2XTextBox");
		jText.setColumns(6);
		jText.setEditable( editable );
		controls.putComponent(jText);
		c.gridx = 1;
		c.gridy = 1;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jText = createCoordinate();
		jText.setName("Point2YTextBox");
		jText.setColumns(6);
		jText.setEditable( editable );
		controls.putComponent(jText);
		c.gridx = 2;
		c.gridy = 1;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jText = createCoordinate();
		jText.setName("Point2ZTextBox");
		jText.setColumns(6);
		jText.setEditable( editable );
		controls.putComponent(jText);
		c.gridx = 3;
		c.gridy = 1;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jLabel = new JLabel();
		jLabel.setName("Roll2Label");
		jLabel.setText( resource.translate("TrackProperties.roll2") );
		c.gridx = 0;
		c.gridy = 2;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createRoll();
		jText.setName("Roll2TextBox");
		jText.setColumns(6);
		jText.setEditable( editable );
		controls.putComponent(jText);
		c.gridx = 1;
		c.gridy = 2;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jGeometryPanel.add(jPanel);

		jPanel = new JPanel();
		jPanel.setBorder( BorderFactory.createCompoundBorder(
			BorderFactory.createEtchedBorder(),
			BorderFactory.createEmptyBorder(3, 3, 3, 3)) );
		gb = new GridBagLayout();
		jPanel.setLayout(gb);

		jLabel = new JLabel();
		jLabel.setName("GradeLabel");
		jLabel.setText( resource.translate("TrackProperties.grade") );
		c.gridx = 0;
		c.gridy = 1;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createGrade();
		jText.setName("GradeTextBox");
		jText.setColumns(6);
		jText.setEditable( false );
		controls.putComponent(jText);
		c.gridx = 1;
		c.gridy = 1;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jGeometryPanel.add(jPanel);

		jTabbedPane.addTab(resource.translate("TrackProperties.geometry"), jGeometryPanel);

		pane.add(jTabbedPane);

		return pane;
	}

	/**
	 * Creates name text box.
	 */
	protected JTextField createName()
	{
		return new JTextField();
	}

	/**
	 * Creates length text box.
	 *
	 * <blockquote><table cellpadding="3">
	 *   <tr>
	 *     <td><b>Minimum value</b></td>
	 *     <td>0 m</td>
	 *   </tr>
	 *   <tr class="TableSubHeadingColor">
	 *     <td><b>Maximum value</b></td>
	 *     <td>500 m</td>
	 *   </tr>
	 * </table></blockquote>
	 */
	protected JFormattedTextField createLength()
	{
		DecimalFormat longFormat = new DecimalFormat("0.000");
		NumberFormatter nf = new NumberFormatter(longFormat);
		nf.setMinimum( new Double(0.0) );
		nf.setMaximum( new Double(500.0) );
		JFormattedTextField tf = new JFormattedTextField(nf);

		return tf;
	}

	/**
	 * Creates text field for radius.
	 */
	protected JFormattedTextField createRadius()
	{
		return JetDialog.createLongFormatTextBox();
	}

	/**
	 * Creates text field for track width.
	 *
	 * <blockquote><table cellpadding="3">
	 *   <tr>
	 *     <td><b>Minimum value</b></td>
	 *     <td>0 m</td>
	 *   </tr>
	 * </table></blockquote>
	 */
	protected JFormattedTextField createWidth()
	{
		DecimalFormat longFormat = new DecimalFormat("0.000");
		NumberFormatter nf = new NumberFormatter(longFormat);
		nf.setMinimum( new Double(0.0) );
		JFormattedTextField tf = new JFormattedTextField(nf);
		return tf;
	}

	/**
	 * Creates text field for track friction.
	 */
	protected JTextField createFriction()
	{
		DecimalFormat medFormat = new DecimalFormat("0.0#");
		NumberFormatter nf = new NumberFormatter(medFormat);
		nf.setMinimum( new Double(0.0) );
		JFormattedTextField tf = new JFormattedTextField(nf);
		return tf;
	}

	/**
	 * Creates text field for track tap.
	 *
	 * <blockquote><table cellpadding="3">
	 *   <tr>
	 *     <td><b>Minimum value</b></td>
	 *     <td>0 m</td>
	 *   </tr>
	 * </table></blockquote>
	 */
	protected JTextField createTap()
	{
		DecimalFormat longFormat = new DecimalFormat("0.000");
		NumberFormatter nf = new NumberFormatter(longFormat);
		nf.setMinimum( new Double(0.0) );
		JFormattedTextField tf = new JFormattedTextField(nf);
		return tf;
	}

	/**
	 * Creates text field for track quality.
	 */
	protected JFormattedTextField createQuality()
	{
		HexadecimalFormat hf = new HexadecimalFormat();
		hf.setDecoration(HexadecimalFormat.DECORATION_PREFIX);
		JFormattedTextField tf = new JFormattedTextField(hf);
		return tf;
	}

	/**
	 * Creates text field for track attrition.
	 */
	protected JFormattedTextField createDamage()
	{
		HexadecimalFormat hf = new HexadecimalFormat();
		hf.setDecoration(HexadecimalFormat.DECORATION_PREFIX);
		JFormattedTextField tf = new JFormattedTextField(hf);
		return tf;
	}

	/**
	 * Creates combo box for track main texture.
	 */
	protected JComboBox createTexture()
	{
		JComboBox cmbTextures = new JComboBox();
		cmbTextures.setEditable(true);
		MutableComboBoxModel mcbm = SimulatorCatalog.getInstance().getBmpModel();
		if (mcbm != null)
			cmbTextures.setModel(mcbm);
		//Dimension dim = cmbTextures.getPreferredSize();
		//dim.width = 150;
		//cmbTextures.setPreferredSize(dim);
		return cmbTextures;
	}

	/**
	 * Creates text field for texture length.
	 *
	 * <blockquote><table cellpadding="3">
	 *   <tr>
	 *     <td><b>Minimum value</b></td>
	 *     <td>0 m</td>
	 *   </tr>
	 * </table></blockquote>
	 */
	protected JTextField createTextureLength()
	{
		DecimalFormat longFormat = new DecimalFormat("0.000");
		NumberFormatter nf = new NumberFormatter(longFormat);
		nf.setMinimum( new Double(0.0) );
		JFormattedTextField tf = new JFormattedTextField(nf);
		return tf;
	}

	/**
	 * Creates combo box for second texture.
	 */
	protected JComboBox createSecondTexture()
	{
		JComboBox cmbSecTex = new JComboBox();
		cmbSecTex.setEditable(true);
		MutableComboBoxModel mcbm = SimulatorCatalog.getInstance().getTgaModel();
		if (mcbm != null)
			cmbSecTex.setModel(mcbm);
		//Dimension dim = cmbSecTex.getPreferredSize();
		//dim.width = 150;
		//cmbSecTex.setPreferredSize(dim);
		return cmbSecTex;
	}

	/**
	 * Creates text field for texture height.
	 *
	 * <blockquote><table cellpadding="3">
	 *   <tr>
	 *     <td><b>Minimum value</b></td>
	 *     <td>0 m</td>
	 *   </tr>
	 * </table></blockquote>
	 */
	protected JTextField createTextureHeight()
	{
		DecimalFormat longFormat = new DecimalFormat("0.000");
		NumberFormatter nf = new NumberFormatter(longFormat);
		nf.setMinimum( new Double(0.0) );
		JFormattedTextField tf = new JFormattedTextField(nf);
		return tf;
	}

	/**
	 * Creates text field for texture width.
	 *
	 * <blockquote><table cellpadding="3">
	 *   <tr>
	 *     <td><b>Minimum value</b></td>
	 *     <td>0 m</td>
	 *   </tr>
	 * </table></blockquote>
	 */
	protected JTextField createTextureWidth()
	{
		DecimalFormat longFormat = new DecimalFormat("0.000");
		NumberFormatter nf = new NumberFormatter(longFormat);
		nf.setMinimum( new Double(0.0) );
		JFormattedTextField tf = new JFormattedTextField(nf);
		return tf;
	}

	/**
	 * Creates text field for texture slope.
	 *
	 * <blockquote><table cellpadding="3">
	 *   <tr>
	 *     <td><b>Minimum value</b></td>
	 *     <td>0 m</td>
	 *   </tr>
	 * </table></blockquote>
	 */
	protected JTextField createTextureSlope()
	{
		DecimalFormat longFormat = new DecimalFormat("0.000");
		NumberFormatter nf = new NumberFormatter(longFormat);
		nf.setMinimum( new Double(0.0) );
		JFormattedTextField tf = new JFormattedTextField(nf);
		return tf;
	}

	/**
	 * Creates text field for roll.
	 */
	protected JTextField createRoll()
	{
		DegreeFormat df = new DegreeFormat();
		JFormattedTextField tf = new JFormattedTextField(df);
		return tf;
	}

	/**
	 * Creates text field for corrdinate of the tuple.
	 */
	protected JFormattedTextField createCoordinate()
	{
		DecimalFormat longFormat = new DecimalFormat("0.000");
		NumberFormatter nf = new NumberFormatter(longFormat);
		JFormattedTextField tf = new JFormattedTextField(nf);
		return tf;
	}

	/**
	 * Creates text field for grade of track.
	 */
	protected JFormattedTextField createGrade()
	{
		DecimalFormat longFormat = new DecimalFormat("0.0\u2030");
		NumberFormatter nf = new NumberFormatter(longFormat);
		JFormattedTextField tf = new JFormattedTextField(nf);
		return tf;
	}


	/**
	 * Fills in the form.
	 */
	protected void fillIn(Track track)
	{
		JTextField				jText;
		JFormattedTextField		jFormatText;
		JComboBox				jComboBox;
		JRadioButton			jRadioButton;

		jText = (JTextField) controls.getComponent("NameTextBox");
		jText.setText( track.getName() );

		jFormatText = (JFormattedTextField) controls.getComponent("LengthTextBox");
		jFormatText.setValue( new Double(track.getLength()) );

		jFormatText = (JFormattedTextField) controls.getComponent("RadiusTextBox");
		jFormatText.setValue( new Double(track.getRadius()) );

		jFormatText = (JFormattedTextField) controls.getComponent("WidthTextBox");
		jFormatText.setValue( new Double(track.getWidth()) );

		jFormatText = (JFormattedTextField) controls.getComponent("FrictionTextBox");
		jFormatText.setValue( new Double(track.getFriction()) );

		jFormatText = (JFormattedTextField) controls.getComponent("TapTextBox");
		jFormatText.setValue( new Double(track.getTap()) );

		jFormatText = (JFormattedTextField) controls.getComponent("QualityTextBox");
		jFormatText.setValue( new Integer(track.getQuality()) );

		jFormatText = (JFormattedTextField) controls.getComponent("DamageTextBox");
		jFormatText.setValue( new Integer(track.getDamage()) );

		jComboBox = (JComboBox) controls.getComponent("EnviromentComboBox");
		jComboBox.setSelectedItem( track.getEnviroment() );

		if (track.isVisible())
		{
			jRadioButton = (JRadioButton) controls.getComponent("VisibleRadioButton");
		}
		else
		{
			jRadioButton = (JRadioButton) controls.getComponent("InvisibleRadioButton");
		}
		jRadioButton.setSelected(true);

		jComboBox = (JComboBox) controls.getComponent("TextureComboBox");
		jComboBox.setSelectedItem( track.getTexture() );

		jFormatText = (JFormattedTextField) controls.getComponent("TextureLengthEditBox");
		jFormatText.setValue( new Double(track.getTextureLength()) );

		jComboBox = (JComboBox) controls.getComponent("SecondTextureComboBox");
		jComboBox.setSelectedItem( track.getSecondTexture() );

		jFormatText = (JFormattedTextField) controls.getComponent("TextureWidthTextBox");
		jFormatText.setValue( new Double(track.getTextureWidth()) );

		jFormatText = (JFormattedTextField) controls.getComponent("TextureHeightTextBox");
		jFormatText.setValue( new Double(track.getTextureHeight()) );

		jFormatText = (JFormattedTextField) controls.getComponent("TextureSlopeTextBox");
		jFormatText.setValue( new Double(track.getTextureSlope()) );

		jFormatText = (JFormattedTextField) controls.getComponent("Point1XTextBox");
		jFormatText.setValue( new Double(track.getPoint1().x) );

		jFormatText = (JFormattedTextField) controls.getComponent("Point1YTextBox");
		jFormatText.setValue( new Double(track.getPoint1().y) );

		jFormatText = (JFormattedTextField) controls.getComponent("Point1ZTextBox");
		jFormatText.setValue( new Double(track.getPoint1().z) );

		jFormatText = (JFormattedTextField) controls.getComponent("Vector1XTextBox");
		jFormatText.setValue( new Double(track.getVector1().x) );

		jFormatText = (JFormattedTextField) controls.getComponent("Vector1YTextBox");
		jFormatText.setValue( new Double(track.getVector1().y) );

		jFormatText = (JFormattedTextField) controls.getComponent("Vector1ZTextBox");
		jFormatText.setValue( new Double(track.getVector1().z) );

		jFormatText = (JFormattedTextField) controls.getComponent("Vector2XTextBox");
		jFormatText.setValue( new Double(track.getVector2().x) );

		jFormatText = (JFormattedTextField) controls.getComponent("Vector2YTextBox");
		jFormatText.setValue( new Double(track.getVector2().y) );

		jFormatText = (JFormattedTextField) controls.getComponent("Vector2ZTextBox");
		jFormatText.setValue( new Double(track.getVector2().z) );

		jFormatText = (JFormattedTextField) controls.getComponent("Point2XTextBox");
		jFormatText.setValue( new Double(track.getPoint2().x) );

		jFormatText = (JFormattedTextField) controls.getComponent("Point2YTextBox");
		jFormatText.setValue( new Double(track.getPoint2().y) );

		jFormatText = (JFormattedTextField) controls.getComponent("Point2ZTextBox");
		jFormatText.setValue( new Double(track.getPoint2().z) );

		jFormatText = (JFormattedTextField) controls.getComponent("Roll1TextBox");
		jFormatText.setValue( new Double(track.getRoll1()) );

		jFormatText = (JFormattedTextField) controls.getComponent("Roll2TextBox");
		jFormatText.setValue( new Double(track.getRoll2()) );

		jFormatText = (JFormattedTextField) controls.getComponent("GradeTextBox");
		jFormatText.setValue( new Double(track.getIncline()) );

		if (editable)
		{
			oldTrack = track;

			setConstraints();
		}
	}

	/**
	 * Sets the constraints for text fields.
	 */
	private void setConstraints()
	{
		JFormattedTextField jFormatText;
		
		jFormatText = (JFormattedTextField) controls.getComponent("Roll1TextBox");
		setConstraints(jFormatText, new Double(-Track.ROLL), new Double(Track.ROLL));
		jFormatText = (JFormattedTextField) controls.getComponent("Roll2TextBox");
		setConstraints(jFormatText, new Double(-Track.ROLL), new Double(Track.ROLL));
	}

	protected void setConstraints(JFormattedTextField tft, Comparable min)
	{
		InternationalFormatter formatter = 
			(InternationalFormatter) tft.getFormatter();
		formatter.setMinimum(min);
	}

	/**
	 * Gets controls map.
	 */
	public ControlMap getControls()
	{
		return controls;
	}

	/*
	 * --------- Inner classes ---------
	 */
	
	protected class GeneralTabFocusTraversal extends FocusTraversalPolicy
	{
		public Component getComponentAfter(Container container, Component aComponent)
		{
			System.out.println(container);

			String name = aComponent.getName();
			if (name.equals("NameTextBox"))
				return controls.getComponent("LengthTextBox");
			else if (name.equals("RadiusTextBox"))
				return controls.getComponent("WidthTextBox");
			else if (name.equals("LengthTextBox"))
				return controls.getComponent("RadiusTextBox");
			else if (name.equals("WidthTextBox"))
				return controls.getComponent("FrictionTextBox");
			else if (name.equals("FrictionTextBox"))
				return controls.getComponent("TapTextBox");
			else if (name.equals("TapTextBox"))
				return controls.getComponent("QualityTextBox");
			else if (name.equals("QualityTextBox"))
				return controls.getComponent("DamageTextBox");

			return null;
		}

		public Component getComponentBefore(Container container, Component aComponent)
		{
			String name = aComponent.getName();
			if (name.equals("LengthTextBox"))
				return controls.getComponent("NameTextBox");
			else if (name.equals("RadiusTextBox"))
				return controls.getComponent("LengthTextBox");
			else if (name.equals("WidthTextBox"))
				return controls.getComponent("RadiusTextBox");
			else if (name.equals("FrictionTextBox"))
				return controls.getComponent("WidthTextBox");
			else if (name.equals("TapTextBox"))
				return controls.getComponent("FrictionTextBox");
			else if (name.equals("QualityTextBox"))
				return controls.getComponent("TapTextBox");
			else if (name.equals("DamageTextBox"))
				return controls.getComponent("QualityTextBox");

			return null;
		}

		public Component getDefaultComponent(Container container)
		{
			return controls.getComponent("NameTextBox");
		}

		public Component getFirstComponent(Container container)
		{
			return controls.getComponent("NameTextBox");
		}

		public Component getLastComponent(Container conatainer)
		{
			return controls.getComponent("DamageTextBox");
		}
	}

	/*
	 * ----------- OK action ------------
	 */
	
	class OKAction implements ActionListener
	{
		public void actionPerformed(ActionEvent ae)
		{
			double					d;
			int						i;
			Point3d					p3d;
			Vector3d				v3d;
			JFormattedTextField		jFormatText;
			JTextField				jText;
			JComboBox				jComboBox;
			JRadioButton			jButton;

			Track track = new Track();

			// Name
			jText = (JTextField) controls.getComponent("NameTextBox");
			String name = jText.getText();
			if (name.equals(""))
				name = null;
			track.setName( name );

			// Length
			jFormatText = (JFormattedTextField) controls.getComponent("LengthTextBox");
			d = ((Number) jFormatText.getValue()).doubleValue();
			track.setLength( d );

			// Radius
			jFormatText = (JFormattedTextField) controls.getComponent("RadiusTextBox");
			d = ((Number) jFormatText.getValue()).doubleValue();
			track.setRadius( d );

			// Width
			jFormatText = (JFormattedTextField) controls.getComponent("WidthTextBox");
			d = ((Number) jFormatText.getValue()).doubleValue();
			track.setWidth( d );

			// Tap
			jFormatText = (JFormattedTextField) controls.getComponent("TapTextBox");
			d = ((Number) jFormatText.getValue()).doubleValue();
			track.setTap( d );

			// Friction
			jFormatText = (JFormattedTextField) controls.getComponent("FrictionTextBox");
			d = ((Number) jFormatText.getValue()).doubleValue();
			track.setFriction( d );

			// Damage
			jFormatText = (JFormattedTextField) controls.getComponent("DamageTextBox");
			i = ((Number) jFormatText.getValue()).intValue();
			track.setDamage( i );

			// Quality
			jFormatText = (JFormattedTextField) controls.getComponent("QualityTextBox");
			i = ((Number) jFormatText.getValue()).intValue();
			track.setQuality( i );

			// Enviroment
			jComboBox = (JComboBox) controls.getComponent("EnviromentComboBox");
			track.setEnviroment( (String) jComboBox.getSelectedItem() );

			// Visibility
			jButton = (JRadioButton) controls.getComponent("VisibleRadioButton");
			track.setVisible( jButton.isSelected() );

			// Texture
			jComboBox = (JComboBox) controls.getComponent("TextureComboBox");
			track.setTexture( (String) jComboBox.getSelectedItem() );

			// Second texture
			jComboBox = (JComboBox) controls.getComponent("SecondTextureComboBox");
			track.setSecondTexture( (String) jComboBox.getSelectedItem() );

			// Texture length
			jFormatText = (JFormattedTextField) controls.getComponent("TextureLengthEditBox");
			d = ((Number) jFormatText.getValue()).doubleValue();
			track.setTextureLength(d);

			// Texture width
			jFormatText = (JFormattedTextField) controls.getComponent("TextureWidthTextBox");
			d = ((Number) jFormatText.getValue()).doubleValue();
			track.setTextureWidth(d);

			// Texture height
			jFormatText = (JFormattedTextField) controls.getComponent("TextureHeightTextBox");
			d = ((Number) jFormatText.getValue()).doubleValue();
			track.setTextureHeight(d);

			// Texture slope
			jFormatText = (JFormattedTextField) controls.getComponent("TextureSlopeTextBox");
			d = ((Number) jFormatText.getValue()).doubleValue();
			track.setTextureSlope(d);

			// Roll 1
			jFormatText = (JFormattedTextField) controls.getComponent("Roll1TextBox");
			d = ((Number) jFormatText.getValue()).doubleValue();
			track.setRoll1(d);

			// Roll 2
			jFormatText = (JFormattedTextField) controls.getComponent("Roll2TextBox");
			d = ((Number) jFormatText.getValue()).doubleValue();
			track.setRoll2(d);

			// Point 1
			p3d = new Point3d();
			jFormatText = (JFormattedTextField) controls.getComponent("Point1XTextBox");
			p3d.x = ((Number) jFormatText.getValue()).doubleValue();
			jFormatText = (JFormattedTextField) controls.getComponent("Point1YTextBox");
			p3d.y = ((Number) jFormatText.getValue()).doubleValue();
			jFormatText = (JFormattedTextField) controls.getComponent("Point1ZTextBox");
			p3d.z = ((Number) jFormatText.getValue()).doubleValue();
			track.setPoint1( p3d );

			// Vector 1
			v3d = new Vector3d();
			jFormatText = (JFormattedTextField) controls.getComponent("Vector1XTextBox");
			v3d.x = ((Number) jFormatText.getValue()).doubleValue();
			jFormatText = (JFormattedTextField) controls.getComponent("Vector1YTextBox");
			v3d.y = ((Number) jFormatText.getValue()).doubleValue();
			jFormatText = (JFormattedTextField) controls.getComponent("Vector1ZTextBox");
			v3d.z = ((Number) jFormatText.getValue()).doubleValue();
			track.setVector1( v3d );

			// Vector 2
			v3d = new Vector3d();
			jFormatText = (JFormattedTextField) controls.getComponent("Vector2XTextBox");
			v3d.x = ((Number) jFormatText.getValue()).doubleValue();
			jFormatText = (JFormattedTextField) controls.getComponent("Vector2YTextBox");
			v3d.y = ((Number) jFormatText.getValue()).doubleValue();
			jFormatText = (JFormattedTextField) controls.getComponent("Vector2ZTextBox");
			v3d.z = ((Number) jFormatText.getValue()).doubleValue();
			track.setVector2( v3d );

			// Point 2
			p3d = new Point3d();
			jFormatText = (JFormattedTextField) controls.getComponent("Point2XTextBox");
			p3d.x = ((Number) jFormatText.getValue()).doubleValue();
			jFormatText = (JFormattedTextField) controls.getComponent("Point2YTextBox");
			p3d.y = ((Number) jFormatText.getValue()).doubleValue();
			jFormatText = (JFormattedTextField) controls.getComponent("Point2ZTextBox");
			p3d.z = ((Number) jFormatText.getValue()).doubleValue();
			track.setPoint2( p3d );

			if (!track.equals(oldTrack))
			{
				// Odznacz stary tor.
				editor.clearSelection();

				Object[] oldArray = new Object[1];
				oldArray[0] = oldTrack;
				Object[] newArray = new Object[1];
				newArray[0] = track;

				editor.getScenery().replace(oldArray, newArray);

				// Zaznacz nowy tor.
				editor.setSelectedObject(track);
			}

			dispose();
		}
	}

	/*
	 * ------------ Members -------------
	 */
	
	/** Editable flag. */
	protected boolean editable;

	/** I18n resource. */
	protected JetResources resource;

	/** Controls map. */
	protected ControlMap controls = new ControlMap();

	/** Old track. */
	private Track oldTrack;

	/** Scenery. */
	private JSceneryEditor editor;
}
